/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package misc;

import pipe.analysis.Matrix;
import vanted.petrinetelements.InArc;
import vanted.petrinetelements.OutArc;
import vanted.petrinetelements.PetriNet;

// TODO: Auto-generated Javadoc
/**
 * The Class MatrixOperation.
 * 
 * @author Kevin Pucknat
 */
public class MatrixOperation {
	
	/** The pn. */
	private final PetriNet pn;
	
	/** The farray. */
	private double[][] farray;
	
	/** The barray. */
	private double[][] barray;
	
	/** The farray int. */
	private int[][] farrayInt;
	
	/** The barray int. */
	private int[][] barrayInt;

	/**
	 * Instantiates a new matrix operation.
	 *
	 * @param pn the pn
	 */
	public MatrixOperation(PetriNet pn) {
		this.pn = pn;
	}
	
	//from the view of a transition: the arc is outgoing/forward
	/**
	 * Gets the forward matrix.
	 *
	 * @return the f matrix
	 */
	public double[][] getFMatrix(){		
		if(farray!=null)return farray;
			
		farray = new double[pn.getAllPlaces().size()][pn.getAllTransitions().size()];
				
		int placeNr=0;
		int trNr=0;
		
		for(int i=0;i<pn.getAllArcs().size();i++){
			if(pn.getAllArcs().get(i) instanceof OutArc){	
				placeNr = pn.getPlacePosition(pn.getAllArcs().get(i).getPlace());				
				trNr = pn.getTransitionPosition(pn.getAllArcs().get(i).getTransition());				
				if(placeNr ==-1 || trNr==-1)continue;				
				farray[placeNr][trNr]=Double.parseDouble(pn.getAllArcs().get(i).getArcWeight().getValue().toString());
			}			
		}		
		return farray;
	}
	
	//from the view of a transition: the arc is outgoing/forward
	/**
	 * Gets the int forward matrix.
	 *
	 * @return the f matrix int
	 */
	public int[][] getFMatrixInt(){	
		if(farrayInt!=null)return farrayInt;
		
		farrayInt = new int[pn.getAllPlaces().size()][pn.getAllTransitions().size()];
		
		int placeNr=0;
		int trNr=0;
		
		for(int i=0;i<pn.getAllArcs().size();i++){
			if(pn.getAllArcs().get(i) instanceof OutArc){	
				placeNr = pn.getPlacePosition(pn.getAllArcs().get(i).getPlace());
				trNr = pn.getTransitionPosition(pn.getAllArcs().get(i).getTransition());				
				if(placeNr ==-1 || trNr==-1)continue;
				farrayInt[placeNr][trNr]=Integer.parseInt(pn.getAllArcs().get(i).getArcWeight().getValue().toString());
			}			
		}
		
		return farrayInt;
	}
	
	//from the view of a transition: the arc is incoming/backward
	/**
	 * Gets the backward matrix.
	 *
	 * @return the b matrix
	 */
	public double[][] getBMatrix(){		
		if(barray!=null)return barray;
		
		barray = new double[pn.getAllPlaces().size()][pn.getAllTransitions().size()];
						
		int placeNr=0;
		int trNr=0;
		
		for(int i=0;i<pn.getAllArcs().size();i++){
			if(pn.getAllArcs().get(i) instanceof InArc){	
				placeNr = pn.getPlacePosition(pn.getAllArcs().get(i).getPlace());
				trNr = pn.getTransitionPosition(pn.getAllArcs().get(i).getTransition());
				if(placeNr ==-1 || trNr==-1)continue;
				barray[placeNr][trNr]=Double.parseDouble(pn.getAllArcs().get(i).getArcWeight().getValue().toString());			
			}		
		}
		
		return barray;
	}
	
	//from the view of a transition: the arc is incoming/backward
	/**
	 * Gets the int backward matrix.
	 *
	 * @return the b matrix int
	 */
	public int[][] getBMatrixInt(){		
		if(barrayInt!=null)return barrayInt;
		
		barrayInt = new int[pn.getAllPlaces().size()][pn.getAllTransitions().size()];
		
		int placeNr=0;
		int trNr=0;
		
		for(int i=0;i<pn.getAllArcs().size();i++){
			if(pn.getAllArcs().get(i) instanceof InArc){
				placeNr = pn.getPlacePosition(pn.getAllArcs().get(i).getPlace());
				trNr = pn.getTransitionPosition(pn.getAllArcs().get(i).getTransition());
				if(placeNr ==-1 || trNr==-1)continue;
				barrayInt[placeNr][trNr]=Integer.parseInt(pn.getAllArcs().get(i).getArcWeight().getValue().toString());		
			}		
		}
		
		return barrayInt;
	}
	
	
	/**
	 * Gets the incidence matrix.
	 *
	 * @return the incidence matrix
	 */
	public Matrix getIncidenceMatrix(){		
		Matrix matrix = new Matrix(getFMatrix());
		matrix = matrix.minus(new Matrix(getBMatrix()));
		return matrix;
	}
}
