/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package petri_jfern;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import vanted.petrinetelements.AbstractArc;

// TODO: Auto-generated Javadoc
/**
 * The Class ArcMap is a helper class to combine the VANTED arcs and the rakiura arcs.
 *
 * @author Kevin Pucknat
 */
public class ArcMap {
	
	/** The arc map. */
	private HashMap<AbstractArc, org.rakiura.cpn.AbstractArc> arcMap = new HashMap<AbstractArc, org.rakiura.cpn.AbstractArc>();
	
	/** The arc map reverse. */
	private HashMap<org.rakiura.cpn.AbstractArc, AbstractArc> arcMapReverse = new HashMap<org.rakiura.cpn.AbstractArc, AbstractArc>();
	
	/**
	 * Adds the arc.
	 *
	 * @param arc the arc (vanted)
	 * @param a the arc (rakiura)
	 */
	public void addArc(AbstractArc arc, org.rakiura.cpn.AbstractArc a){		
		arcMap.put(arc, a);
		arcMapReverse.put(a, arc);
	}
	
	/**
	 * Removes the arc v (vanted).
	 *
	 * @param arc the arc
	 */
	public void removeArcV(AbstractArc arc){		
		arcMapReverse.remove(arcMap.remove(arc));
	}
	
	/**
	 * Removes the arc r (rakiura).
	 *
	 * @param a the a
	 */
	public void removeArcR(org.rakiura.cpn.AbstractArc a){
		arcMap.remove(arcMapReverse.remove(a));
	}
	
	/**
	 * Gets the arc v (vanted).
	 *
	 * @param a the a (rakiura)
	 * @return the arc v (vanted)
	 */
	public AbstractArc getArcV(org.rakiura.cpn.AbstractArc a){
		return arcMapReverse.get(a);
	}
	
	/**
	 * Gets the arc r (rakiura).
	 *
	 * @param arc the arc (vanted)
	 * @return the arc r (rakiura)
	 */
	public org.rakiura.cpn.AbstractArc getarcR(AbstractArc arc){		
		return arcMap.get(arc);
	}
	
	/**
	 * Gets the all arcs v (vanted).
	 *
	 * @return the all arcs v (rakiura)
	 */
	public List<AbstractArc> getAllArcsV(){		
		return new ArrayList<AbstractArc>(arcMap.keySet());
	}
	
	/**
	 * Gets the allarcs r (rakiura).
	 *
	 * @return the allarcs r (rakiura)
	 */
	public List<org.rakiura.cpn.AbstractArc> getAllarcsR(){		
		return new ArrayList<org.rakiura.cpn.AbstractArc>(arcMapReverse.keySet());
	}
	
	/**
	 * Contains key r.
	 *
	 * @param a the a (rakiura)
	 * @return true, if successful
	 */
	public boolean containsKeyR(org.rakiura.cpn.AbstractArc a){
		return arcMapReverse.containsKey(a);		
	}
	
	/**
	 * Contains key v.
	 *
	 * @param arc the arc (vanted)
	 * @return true, if successful
	 */
	public boolean containsKeyV(AbstractArc arc){
		return arcMap.containsKey(arc);		
	}
}
