/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package petri_jfern;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import vanted.petrinetelements.Place;

// TODO: Auto-generated Javadoc
/**
 * The Class PlaceMap is a helper class to combine the VANTED places and the rakiura places.
 *
 * @author Kevin Pucknat
 */
public class PlaceMap {
	
	/** The place map. */
	private HashMap<Place, org.rakiura.cpn.Place> placeMap = new HashMap<Place, org.rakiura.cpn.Place>();
	
	/** The place map reverse. */
	private HashMap<org.rakiura.cpn.Place, Place> placeMapReverse = new HashMap<org.rakiura.cpn.Place, Place>();
	
	/**
	 * Adds the place.
	 *
	 * @param place the place (vanted)
	 * @param p the p (rakiura)
	 */
	public void addPlace(Place place, org.rakiura.cpn.Place p){
		placeMap.put(place, p);
		placeMapReverse.put(p, place);
	}
	
	/**
	 * Removes the place v (vanted, rakiura).
	 *
	 * @param place the place (vanted)
	 */
	public void removePlaceV(Place place){		
		placeMapReverse.remove(placeMap.remove(place));
	}
	
	/**
	 * Removes the place r (vanted, rakiura).
	 *
	 * @param p the place (rakiura)
	 */
	public void removePlaceR(org.rakiura.cpn.Place p){
		placeMap.remove(placeMapReverse.remove(p));
	}
	
	/**
	 * Gets the place v.
	 *
	 * @param p the place (rakiura)
	 * @return the place v (vanted)
	 */
	public Place getPlaceV(org.rakiura.cpn.Place p){
		return placeMapReverse.get(p);
	}
	
	/**
	 * Gets the place r (rakiura).
	 *
	 * @param place the place (vanted)
	 * @return the place r (rakiura)
	 */
	public org.rakiura.cpn.Place getPlaceR(Place place){
		return placeMap.get(place);
	}
	
	/**
	 * Gets the all places v (vanted).
	 *
	 * @return the all places v (vanted)
	 */
	public List<Place> getAllPlacesV(){		
		return new ArrayList<Place>(placeMap.keySet());
	}
	
	/**
	 * Gets the all places r (rakiura).
	 *
	 * @return the all places r (rakiura)
	 */
	public List<org.rakiura.cpn.Place> getAllPlacesR(){		
		return new ArrayList<org.rakiura.cpn.Place>(placeMapReverse.keySet());
	}
	
	/**
	 * Contains key r.
	 *
	 * @param p the place (rakiura)
	 * @return true, if successful
	 */
	public boolean containsKeyR(org.rakiura.cpn.Place p){
		return placeMapReverse.containsKey(p);		
	}
	
	/**
	 * Contains key v.
	 *
	 * @param place the place (vanted)
	 * @return true, if successful
	 */
	public boolean containsKeyV(Place place){
		return placeMap.containsKey(place);		
	}
}
