/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package petri_jfern;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.AttributeHelper;

import vanted.petrinetelements.Transition;

// TODO: Auto-generated Javadoc
/**
 * The Class TransitionMap is a helper class to combine the VANTED transitions and the rakiura transitions.
 *
 * @author Kevin Pucknat
 */
public class TransitionMap {
	
	/** The transition map. */
	private HashMap<Transition, org.rakiura.cpn.Transition> transitionMap = new HashMap<Transition, org.rakiura.cpn.Transition>();
	
	/** The transition map reverse. */
	private HashMap<org.rakiura.cpn.Transition, Transition> transitionMapReverse = new HashMap<org.rakiura.cpn.Transition, Transition>();
	
	/** The color map. */
	private HashMap<Transition, List<Color>> colorMap = new HashMap<Transition, List<Color>>();
	
	/**
	 * Adds the transitions.
	 *
	 * @param transition the transition (vanted)
	 * @param t the transition (rakiura)
	 */
	public void addTransition(Transition transition, org.rakiura.cpn.Transition t){		
		transitionMap.put(transition, t);
		transitionMapReverse.put(t, transition);
		List<Color> colors = new ArrayList<Color>();
		for(int i=0; i<transition.getNodes().size(); i++){			
			colors.add(AttributeHelper.getFillColor(transition.getNodes().get(i)));
		}
		colorMap.put(transition, colors);
	}
	
	/**
	 * Removes the transition v (vanted, rakiura).
	 *
	 * @param transition the transition (vanted)
	 */
	public void removeTransitionV(Transition transition){		
		transitionMapReverse.remove(transitionMap.remove(transition));
	}
	
	/**
	 * Removes the transition r (vanted, rakiura).
	 *
	 * @param t the transition (vanted)
	 */
	public void removeTransitionR(org.rakiura.cpn.Transition t){
		transitionMap.remove(transitionMapReverse.remove(t));
	}
	
	/**
	 * Gets the transition v (vanted).
	 *
	 * @param t the transition (rakiura)
	 * @return the transition v (vanted)
	 */
	public Transition getTransitionV(org.rakiura.cpn.Transition t){
		return transitionMapReverse.get(t);
	}
	
	/**
	 * Gets the transition r (rakiura).
	 *
	 * @param transition the transition (vanted)
	 * @return the transition r (rakiura)
	 */
	public org.rakiura.cpn.Transition getTransitionR(Transition transition){		
		return transitionMap.get(transition);
	}
	
	/**
	 * Gets the all transitions v (vanted).
	 *
	 * @return the all transitions v (vanted)
	 */
	public List<Transition> getAllTransitionsV(){		
		return new ArrayList<Transition>(transitionMap.keySet());
	}
	
	/**
	 * Gets the all transitions r (rakiura).
	 *
	 * @return the all transitions r (rakiura)
	 */
	public List<org.rakiura.cpn.Transition> getAllTransitionsR(){		
		return new ArrayList<org.rakiura.cpn.Transition>(transitionMapReverse.keySet());
	}
	
	/**
	 * Contains key r.
	 *
	 * @param t the transition (rakiura)
	 * @return true, if successful
	 */
	public boolean containsKeyR(org.rakiura.cpn.Transition t){
		return transitionMapReverse.containsKey(t);		
	}
	
	/**
	 * Contains key v.
	 *
	 * @param transition the transition (vanted)
	 * @return true, if successful
	 */
	public boolean containsKeyV(Transition transition){
		return transitionMap.containsKey(transition);		
	}
	
	/**
	 * Gets the color.
	 *
	 * @param transition the transition
	 * @return the color
	 */
	public List<Color> getColor(Transition transition){
		return colorMap.get(transition);
	}
}
