/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package pnml.loader;

import java.awt.Dimension;
import java.awt.Point;

import org.AttributeHelper;
import org.graffiti.event.ListenerManager;
import org.graffiti.graph.AdjListGraph;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import vanted.Parser;

public abstract class AbstractPetriNetParser {	
	protected AdjListGraph graph;
	
	public AdjListGraph createGraph(NodeList nodeList){
		if(nodeList==null)return null;
		
		graph = new AdjListGraph(new ListenerManager());
		
		Element element;
		Node node;
		
		for (int i = 0; i < nodeList.getLength(); i++) {
			node = nodeList.item(i);
			
			if (node instanceof Element) {				
				element = (Element) node;	
				
				makeElements(element.getElementsByTagName("place"), "place");				
				makeElements(element.getElementsByTagName("transition"), "transition");
				makeElements(element.getElementsByTagName("arc"), "arc");
			}
		}
		
		return graph;
	}
	
	private void makeElements(NodeList list, String elementName){
		for(int i=0; i<list.getLength();i++){
			if(elementName.equals("place"))
				createPlace((Element)list.item(i));
			else if(elementName.equals("transition"))
				createTransition((Element)list.item(i));
			else if(elementName.equals("arc"))
				createArc((Element)list.item(i));
		}
	}
	
	public long getID(String id){		
		Object o = Parser.parse2Int(id);
		if(o instanceof Integer)
			return (Integer)o;
		else
			return o.hashCode();		
	}
	
	public void createNodeAttr(org.graffiti.graph.Node node, Element element, String str){
		PNMLHelper graphic = new PNMLHelper();
		Point p = graphic.getPosition(element);
		Dimension d = graphic.getDimension(element);
			
		AttributeHelper.setPosition(node, p.getX(), p.getY());
		AttributeHelper.setSize(node, d.getWidth(), d.getHeight());
		AttributeHelper.setLabel(node, PNMLHelper.getValue("name", element));

		node.setID(getID(element.getAttribute("id")));
	}	

	public abstract void createPlace(Element element);
	public abstract void createTransition(Element element);
	public abstract void createArc(Element element);
}
