/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package pnml.loader;

import java.awt.Dimension;
import java.awt.Point;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import vanted.Parser;

public class PNMLHelper {
	public static String getValue(String tag, Element element){
		return getValueByTag("text", getElement(tag, element));
	}
	
	public static Element getElement(String tag, Element element){		
		if(element==null)return null;
		if(element.getElementsByTagName(tag).getLength()>0)
			return element.getElementsByTagName(tag).item(0) instanceof Element ? (Element)element.getElementsByTagName(tag).item(0) : null;
		
		return null;
	}
	
	public static String getValueByTag(String tag, Element element){
		if(element==null)return "";
		NodeList list = null;		
		if(element.getElementsByTagName(tag).getLength()>0)
			list = element.getElementsByTagName(tag).item(0).getChildNodes();
		
		if(list!=null && list.getLength()>0)
			return list.item(0).getNodeValue();
		
		return ""; 
	}
	
	public Point getPosition(Element element){		
		if(element==null)return null;		
				
		Element e = getElement("position", element);
		double x = (x = Parser.parse2Double(e.getAttribute("x"))) != -1 ? x : 0;
		double y = (y = Parser.parse2Double(e.getAttribute("y"))) != -1 ? y : 0;
	
		return new Point((int)x, (int)y);
	}
	
	public Dimension getDimension(Element element){		
		if(element==null)return null;		
	
		Element e = getElement("dimension", element);		
		double x = (x = Parser.parse2Double(e.getAttribute("x"))) != -1 ? x : 25;
		double y = (y = Parser.parse2Double(e.getAttribute("y"))) != -1 ? y : 25;
		
		return new Dimension((int)x, (int)y);
	}
	
	
	public String getTokenType(Element element){
		Element toolspecific = getToolspecific(element);
				
		return getValue("tokenType", toolspecific);		
	}
	
	public String getCapacityContinous(Element element){
		Element toolspecific = getToolspecific(element);
				
		return getValue("capacityContinuous", toolspecific);		
	}
	
	public String getMarkingContinous(Element element){
		Element toolspecific = getToolspecific(element);
				
		return getValue("initialMarkingContinuous", toolspecific);		
	}
	
	public String getInscriptionContinuous(Element element){
		Element toolspecific = getToolspecific(element);		
		
		return getValue("inscriptionContinuous", toolspecific);		
	}
	
	public Element getToolspecific(Element element){
		Element e = getElement("toolspecific", element);
//		if(e!=null){
//		System.out.println(e.getAttribute("tool"));
//		System.out.println(e.getAttribute("version"));
//		}
		return e;
	}
}
