/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package pnml.loader;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;

import org.ReleaseInfo;
import org.graffiti.editor.GravistoService;
import org.graffiti.editor.MainFrame;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;


/**
 * Transforms a given input stream into a Petri-Net <code>Document</code> presentation.
 * This class builds up a xml file (xls file for layout) from the input stream and creates a <code>Document</code> from the xml file.
 * @author Kevin Pucknat
 */

public class PNMLTransformer {   
   /** 
    * Transform a PNML file into a Document
    * @returns Document of the Petri-Net
    * @param input the input stream of PNML
    * @throws IOException
    * @throws TransformerException
    * @return
    */
   public Document transformPNML(InputStream input) throws IOException, TransformerException{      
      URL url = GravistoService.getResource(this.getClass(), "GenerateObjectList", "xsl");
      StreamSource xsltSource = new StreamSource(url.openStream());    		 
      Transformer transformer = TransformerFactory.newInstance().newTransformer(xsltSource);

      File output = new File(ReleaseInfo.getAppSubdirFolderWithFinalSep("addons", "PetriNet") + "ObjectList.xml");
      StreamSource source = new StreamSource(input);
      StreamResult result = new StreamResult(output);

      transformer.transform(source, result);

      Document document = getDOMfromFile(output);    

//      if (output != null)
//         output.delete();
      
      return document;    
   }
   
   /**
    * Constructs a <code>Document</code> from a PNML file
    * @param pnml File
    * @return A <code>Document</code> for the Petri-Net
    * @throws IOException
    */
   private Document getDOMfromFile(File pnml) throws IOException{   	 
   	 Document document = null;
       try {
      	  DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
           DocumentBuilder builder = domFactory.newDocumentBuilder();
           document = builder.parse(pnml);
           
           XPath xPath = XPathFactory.newInstance().newXPath();

           NodeList result1 = (NodeList) xPath.evaluate("/pnml/net", document , XPathConstants.NODESET);
           
           if(result1.getLength()!=1){         	
         	  MainFrame.showMessageDialog("The pnml file can not be loaded, because the pnml namespace is not defined!", "PNML error");
         	  return null;
           }           

       } catch (Exception e) {
           e.printStackTrace();
       }
       
      return document ;
   }
}