/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package pnml.loader;

import org.graffiti.graph.AdjListGraph;
import org.graffiti.graph.Graph;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import vanted.petrinetelements.misc.PetriNetType;

public class PetriNetBuilder {

	public Graph build(Document pnmlDoc) {
		NodeList nodeList;
		if(pnmlDoc==null)return new AdjListGraph();
		try {
			nodeList = pnmlDoc.getDocumentElement().getChildNodes();
			AbstractPetriNetParser parser = null;
									
			switch(getPetriNetType(nodeList)){
				case DISCRETE: parser = new PetriNetParserDiscrete();
					break;
				case CONTINUOUS: parser = new PetriNetParserContinuous();
					break;				
				default:	parser = new PetriNetParserDiscrete();
					break;
			}
			
			return parser.createGraph(nodeList);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	
	private PetriNetType getPetriNetType(NodeList nodeList){
		if(nodeList==null)return PetriNetType.EMPTY;
		Element element;
		Node node;
		
		for (int i = 0; i < nodeList.getLength(); i++) {
			node = nodeList.item(i);
			
			if (node instanceof Element) {
				element = (Element) node;
				
				if(element.getElementsByTagName("tokenType").getLength()>0){					
					Node v = element.getElementsByTagName("tokenType").item(0);				
					
					if(v instanceof Element)	
						return PetriNetType.fromName(getValue("text", (Element)v));					
				}
			}
		}
		return PetriNetType.EMPTY;
	}	 	
	
	private String getValue(String tag, Element element){
		NodeList list = null;		
		if(element.getElementsByTagName(tag).getLength()>0)
			list = element.getElementsByTagName(tag).item(0).getChildNodes();
		
		if(list!=null && list.getLength()>0)
			return list.item(0).getNodeValue();
		
		return null;  
	}
}
