/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package pnml.loader;

import java.awt.Color;

import org.AttributeHelper;
import org.graffiti.graph.Edge;
import org.graffiti.graph.Node;
import org.w3c.dom.Element;

import vanted.Parser;
import vanted.petrinetelements.AbstractArc;
import vanted.petrinetelements.ArcWeight;
import vanted.petrinetelements.Capacity;
import vanted.petrinetelements.NodeShape;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.TokenContinuous;

public class PetriNetParserContinuous extends AbstractPetriNetParser{	
	
	public void createPlace(Element element) {
		Node node = graph.addNode(AttributeHelper.getDefaultGraphicsAttributeForKeggNode(0, 0));
		createNodeAttr(node, element, "p");
		AttributeHelper.setShape(node, NodeShape.Place.getTxt());
		
		double capacity = Parser.parse2Double(PNMLHelper.getValue("capacityContinuous", element));
		String initialMarking = PNMLHelper.getValue("initialMarkingContinuous", element);		
				
		Place place = new Place(node);
		if(capacity>0)
			place.addCapacity(new Capacity(capacity));

		if (initialMarking.length() > 0) {			
			String[] stringArray = initialMarking.split(",");						
			place.addToken(new TokenContinuous(Parser.parse2Double(stringArray[0])));	
		}
	}

	public void createTransition(Element element) {
		org.graffiti.graph.Node node = graph.addNode(AttributeHelper.getDefaultGraphicsAttributeForKeggNode(0, 0));
		createNodeAttr(node, element, null);		
		AttributeHelper.setShape(node, NodeShape.Transition.getTxt());		
	}

	public void createArc(Element element) {
		PNMLHelper graphic = new PNMLHelper();
			
		graphic.getToolspecific(element);		
		
		String id = element.getAttribute("id");;
		String sourceNodeID = element.getAttribute("source");
		String targetNodeID = element.getAttribute("target");		
	
		Node sourceNode = null;
		Node targetNode = null;

		for (Node node : graph.getNodes()) {
			if (node.getID() == getID(sourceNodeID))
				sourceNode = node;
			else if (node.getID() == getID(targetNodeID))
				targetNode = node;
		}

		Edge edge = graph.addEdge(sourceNode, targetNode, true, AttributeHelper.getDefaultGraphicsAttributeForEdge(Color.BLACK, Color.BLACK, true));
		AttributeHelper.setLabel(edge, element.getAttribute("name"));
		edge.setID(getID(id));

		double inscription = (inscription = Parser.parse2Double(graphic.getInscriptionContinuous(element)))!=-1 ? inscription : 0;
		if(inscription>0 && inscription!=1){
			AbstractArc arc = new AbstractArc(edge) {};
			arc.addArcWeight(new ArcWeight(inscription));
		}			
	}
}
