/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package pnml.loader;

import java.awt.Color;

import org.AttributeHelper;
import org.graffiti.graph.Edge;
import org.graffiti.graph.Node;
import org.w3c.dom.Element;

import vanted.Parser;
import vanted.petrinetelements.AbstractArc;
import vanted.petrinetelements.ArcWeight;
import vanted.petrinetelements.Capacity;
import vanted.petrinetelements.NodeShape;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.TokenDiscrete;

public class PetriNetParserDiscrete extends AbstractPetriNetParser{
		
	public void createPlace(Element element) {
		Node node = graph.addNode(AttributeHelper.getDefaultGraphicsAttributeForKeggNode(0, 0));
		createNodeAttr(node, element, null);
		AttributeHelper.setShape(node, NodeShape.Place.getTxt());
		
		int capacity = Parser.parse2Int2(PNMLHelper.getValue("capacity", element));			
		String initialMarking = PNMLHelper.getValue("initialMarking", element);		
		
		Place place = new Place(node);
		if(capacity>0)
			place.addCapacity(new Capacity(capacity));

		if (initialMarking.length() > 0) {			
			String[] stringArray = initialMarking.split(",");			
			if (stringArray.length == 1) {				
				int tokenCount = Integer.valueOf(stringArray[0]);
				for (int i=0; i<tokenCount; i++) {
					place.addToken(new TokenDiscrete(1));
				}				
			} else {				
				for (int i=0; i<stringArray.length; ) {
					int tokenCount = Integer.valueOf(stringArray[i + 1]);
					for (int j=0; j < tokenCount; j++) 
						place.addToken(new TokenDiscrete(1));					
					i += 2;
				}
			}
		}
	}

	public void createTransition(Element element) {
		org.graffiti.graph.Node node = graph.addNode(AttributeHelper.getDefaultGraphicsAttributeForKeggNode(0, 0));
		createNodeAttr(node, element, null);
		AttributeHelper.setShape(node, NodeShape.Transition.getTxt());		
	}

	public void createArc(Element element) {
		String id = element.getAttribute("id");
		String sourceNodeID = element.getAttribute("source");
		String targetNodeID = element.getAttribute("target");		
						
		Node sourceNode = null;
		Node targetNode = null;

		for (Node node : graph.getNodes()) {
			if (node.getID() == getID(sourceNodeID))
				sourceNode = node;
			else if (node.getID() == getID(targetNodeID))
				targetNode = node;
		}

		Edge edge = graph.addEdge(sourceNode, targetNode, true, AttributeHelper.getDefaultGraphicsAttributeForEdge(Color.BLACK, Color.BLACK, true));
		AttributeHelper.setLabel(edge, element.getAttribute("name"));
		edge.setID(getID(id));
			
		int inscription = (inscription = Parser.parse2Int2(element.getAttribute("inscription")))!=-1 ? inscription : 0;
		if(inscription>0){
			AbstractArc arc = new AbstractArc(edge) {};
			arc.addArcWeight(new ArcWeight(inscription));
		}					
	}	
}
