/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package pnml.loader;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;

import javax.xml.transform.TransformerException;

import org.ErrorMsg;
import org.graffiti.graph.Graph;
import org.graffiti.plugin.io.AbstractInputSerializer;

// TODO: Auto-generated Javadoc
/**
 * This class provides a reader for graphs in xml_pnml format.
 * @author Kevin Pucknat
 * @see org.graffiti.plugin.io.AbstractIOSerializer
 */
public class XML_PNML_Loader
					extends AbstractInputSerializer {
	
	/** The supported extensions. */
	private String[] extensions = { ".xml", ".pnml" };	
	
	/**
	 * Constructs a new <code>XML_PNML_Loader</code>.
	 */
	public XML_PNML_Loader() {
	}
		
	/**
	 * Returns the extensions supported by this reader.
	 * 
	 * @return the extensions supported by this reader.
	 */
	public String[] getExtensions() {
		return this.extensions;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.graffiti.plugin.io.Serializer#getFileTypeDescriptions()
	 */
	public String[] getFileTypeDescriptions() {
		return new String[] { "Petri-Net ML (XML)" ,"Petri-Net ML"};
	}
	
	/**
	 * Reads in a graph from the given input stream.
	 * 
	 * @param in the <code>InputStream</code>
	 * @param g the graph
	 * @throws IOException I/O exception
	 */
	@Override
	public void read(InputStream in, Graph g)
						throws IOException {
		g.addGraph(read(in));
	}
	
	/**
	 * Reads in a graph from the given input stream. 
	 *
	 * @param in The input stream
	 * @return graph
	 * @throws IOException I/O exception
	 */
	@Override
	public Graph read(InputStream in)
						throws IOException {		
		try{
			PNMLTransformer pnml = new PNMLTransformer();	
			PetriNetBuilder builder = new PetriNetBuilder();			
			
			return builder.build(pnml.transformPNML(in));
		}catch (TransformerException e) {
			ErrorMsg.addErrorMessage(e);
			try {
				in.close();
			} catch (Exception err) {
				// empty
			}			
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.io.InputSerializer#read(java.io.Reader, org.graffiti.graph.Graph)
	 */
	@Override
	public void read(Reader reader, Graph newGraph) throws Exception {		
	}	
	
	@Override
	public boolean validFor(InputStream reader) {
		
      InputStreamReader in = new InputStreamReader(reader);
		
		BufferedReader bufferedReader = new BufferedReader(in);
		StringBuffer finalString = new StringBuffer();
		try {
			while (bufferedReader.ready()) {
				finalString = finalString.append(bufferedReader.readLine());
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		String content = finalString.toString();
		content = content.trim();
		try {
			reader.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			bufferedReader.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (content.contains("pnml")) {
			return true;
		} else {
			return false;
		}
	}
}
