/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package pnml.writer;

import java.io.IOException;
import java.io.OutputStream;
import java.net.URL;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.AttributeHelper;
import org.graffiti.editor.GravistoService;
import org.graffiti.graph.Node;
import org.w3c.dom.DOMException;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import vanted.petrinetelements.AbstractArc;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.Transition;

public abstract class AbstractPNMLParser {
	
	
	public void createPNML(PetriNet pn, OutputStream output) throws IOException{
		StreamSource xsltSource = null;
		Transformer transformer = null;
		try {
			DocumentBuilderFactory builderFactory = DocumentBuilderFactory.newInstance();

			DocumentBuilder builder = builderFactory.newDocumentBuilder();
			DOMImplementation dimpl = builder.getDOMImplementation();	
			
			Document doc = dimpl.createDocument("http://www.informatik.hu-berlin.de/top/pnml/ptNetb", "pnml", null); 
			
			Element root = doc.getDocumentElement();
			
			Element netElement = doc.createElement("net");
			netElement.setAttribute("id", "empty");
			netElement.setAttribute("type", "http://www.informatik.hu-berlin.de/top/pnml/ptNetb");

			List<Place> places = pn.getAllPlaces();
			for (Place place : places) 
				netElement.appendChild(createPlaceElement(place, doc));			

			List<Transition> transitions = pn.getAllTransitions();
			for (Transition transition : transitions) 
				netElement.appendChild(createTransitionElement(transition, doc));
			

			List<AbstractArc> arcs = pn.getAllArcs();
			for (AbstractArc arc : arcs) {
				Element newArc = createArcElement(arc, doc);
				netElement.appendChild(newArc);
			}
		    
			root.appendChild(netElement);
			
			doc.normalize();
			// Create Transformer with XSL Source File
			URL url = GravistoService.getResource(this.getClass(), "GeneratePNML", "xsl");
			xsltSource = new StreamSource(url.openStream());

			transformer = TransformerFactory.newInstance().newTransformer(xsltSource);
			// Write file and do XSLT transformation to generate correct PNML
			DOMSource xmlSource = new DOMSource(doc);
			StreamResult result = new StreamResult(output);
			transformer.transform(xmlSource, result);
			
			output.close();
		} catch (ParserConfigurationException e) {
			System.err.println(e);			
		} catch (DOMException e) {
			System.err.println(e);
		} catch (TransformerConfigurationException e) {
			System.err.println(e);
		} catch (TransformerException e) {
			System.err.println(e);
		}
	}
	
	public void createNodeAttr(Node node, Element element) {
		Double positionXInput = AttributeHelper.getPositionX(node);
		Double positionYInput = AttributeHelper.getPositionY(node);
		Double dimWidth = AttributeHelper.getWidth(node);
		Double dimHeight = AttributeHelper.getHeight(node);

		element.setAttribute("label", AttributeHelper.getLabel(node, ""));
		element.setAttribute("positionX", (positionXInput != null ? String.valueOf(positionXInput) : "0"));
		element.setAttribute("positionY", (positionYInput != null ? String.valueOf(positionYInput) : "0"));
		element.setAttribute("dimensionX", (dimWidth != null ? String.valueOf(dimWidth) : "25"));
		element.setAttribute("dimensionY", (dimHeight != null ? String.valueOf(dimHeight) : "25"));		
	}
	
	/**
	 * Creates a Place Element for a PNML Petri-Net DOM
	 * 
	 * @param place
	 *            Input Place
	 * @param document
	 *            Any DOM to enable creation of Elements and Attributes
	 * @return Place Element for a PNML Petri-Net DOM
	 */
	public abstract Element createPlaceElement(Place place, Document document);
	
	/**
	 * Creates a Transition Element for a PNML Petri-Net DOM
	 * 
	 * @param transition
	 *            Input Transition
	 * @param document
	 *            Any DOM to enable creation of Elements and Attributes
	 * @return Transition Element for a PNML Petri-Net DOM
	 */
	public abstract Element createTransitionElement(Transition transition, Document document);
	
	/**
	 * Creates a Arc Element for a PNML Petri-Net DOM
	 * 
	 * @param arc
	 *            Input Arc
	 * @param document
	 *            Any DOM to enable creation of Elements and Attributes
	 * @return Arc Element for a PNML Petri-Net DOM
	 */
	public abstract Element createArcElement(AbstractArc arc, Document document);
}
