/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package pnml.writer;

import org.AttributeHelper;
import org.graffiti.graph.Node;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import vanted.Parser;
import vanted.petrinetelements.AbstractArc;
import vanted.petrinetelements.AbstractToken;
import vanted.petrinetelements.InArc;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.TokenContinuous;
import vanted.petrinetelements.Transition;

public class PNMLParserContinuous extends AbstractPNMLParser{

	@Override
	public Element createPlaceElement(Place place, Document document) {
		if(place==null || document==null)return null;
		Element placeElement = document.createElement("place");

		Node nodeG = place.getNode();

		String idInput = "p" + nodeG.getID();

		placeElement.setAttribute("id", (idInput != null ? idInput : "error"));
		createNodeAttr(nodeG, placeElement);

		double value = 0;
		double capacity = Parser.parse2Double(place.getCapacity().getValue().toString());

		for (AbstractToken token : place.getTokens()) {
			if (token instanceof TokenContinuous)
				value += ((TokenContinuous) token).getValue();
		}

		placeElement.setAttribute("tokenType", "continuous");
		if (value > 0) {
			placeElement.setAttribute("initialMarkingContinuous", value + "");
			placeElement.setAttribute("initialMarking", Math.round(value) + "");
		}
		if (capacity > 0) {
			//set a tool specific capacity
			placeElement.setAttribute(
					"capacityContinuous",
					(capacity != PetriNet.CAPACITY_MAX_VALUE ? String
							.valueOf(capacity) : ""));

			//set a rounded capacity
			placeElement.setAttribute(
					"capacity",
					(capacity != PetriNet.CAPACITY_MAX_VALUE ? String
							.valueOf(Math.round(capacity)) : ""));
		}
			
		return placeElement;
	}

	@Override
	public Element createTransitionElement(Transition transition, Document document) {
		if(transition==null || document==null)return null;
		Element transitionement = document.createElement("transition");

		Node nodeG = transition.getNode();
	
		String idInput = "t" + nodeG.getID();

		transitionement.setAttribute("id", (idInput != null ? idInput	: "error"));
		createNodeAttr(nodeG, transitionement);
	
		return transitionement;
	}

	@Override
	public Element createArcElement(AbstractArc arc, Document document) {
		if(arc==null || document==null)return null;
		Element arcElement = document.createElement("arc");
	
		String idInput = "a" + arc.getID();
		String sourceInput;
		String targetInput;
		if (arc instanceof InArc) {
			sourceInput = "p" + arc.getPlace().getID();
			targetInput = "t" + arc.getTransition().getID();
		} else {
			targetInput = "p" + arc.getPlace().getID();
			sourceInput = "t" + arc.getTransition().getID();
		}

		arcElement.setAttribute("id", (idInput != null ? idInput : "error"));
		arcElement.setAttribute("source", (sourceInput != null ? sourceInput : ""));
		arcElement.setAttribute("target", (targetInput != null ? targetInput : ""));
		arcElement.setAttribute("label", AttributeHelper.getLabel(arc.getEdge(), ""));
		
		double inscription = Parser.parse2Double(arc.getArcWeight().getValue().toString());
		if (inscription > 0){
			//set a tool specific inscription
			arcElement.setAttribute("toolspecific", null);
			arcElement.setAttribute("tokenType", "continuous");
			arcElement.setAttribute("inscriptionContinuous",
					inscription + "");

			//set a rounded inscription
			arcElement.setAttribute("inscription",
					String.valueOf(Math.round(inscription)));
		}
		
		return arcElement;
	}
	
}
