/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package pnml.writer;

import org.AttributeHelper;
import org.graffiti.graph.Node;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import vanted.Parser;
import vanted.petrinetelements.AbstractArc;
import vanted.petrinetelements.AbstractToken;
import vanted.petrinetelements.InArc;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.TokenDiscrete;
import vanted.petrinetelements.Transition;

public class PNMLParserDiscrete extends AbstractPNMLParser{

	@Override
	public Element createPlaceElement(Place place, Document document) {
		if(place==null || document==null)return null;
			
		Element placeElement = document.createElement("place");			

		Node nodeG = place.getNode();
	
		String idInput = "p" + nodeG.getID();

		placeElement.setAttribute("id", (idInput != null ? idInput : "error"));
		createNodeAttr(nodeG, placeElement);

		int valueDiscrete = 0;
		
		double capacity = Double.valueOf(place.getCapacity().getValue()
				.toString());

		for (AbstractToken token : place.getTokens()) {
			if (token instanceof TokenDiscrete)
				valueDiscrete += ((TokenDiscrete) token).getValue();		
		}
	
		placeElement.setAttribute("tokenType", "discrete");
		
		if (valueDiscrete != 0)
			placeElement.setAttribute("initialMarking", valueDiscrete + "");
		if (capacity != 0)
			placeElement.setAttribute(
					"capacity",
					(capacity != PetriNet.CAPACITY_MAX_VALUE ? String.valueOf((int) capacity) : ""));	
		
		return placeElement;
	}

	@Override
	public Element createTransitionElement(Transition transition, Document document) {
		if(transition==null || document==null)return null;
			
		Element transitionElement = null;
		
		transitionElement = document.createElement("transition");		

		Node nodeG = transition.getNode();
		
		String idInput = "t" + nodeG.getID();

		transitionElement.setAttribute("id", (idInput != null ? idInput : "error"));
		createNodeAttr(nodeG, transitionElement);
	
		return transitionElement;		
	}

	@Override
	public Element createArcElement(AbstractArc arc, Document document) {
		if(arc==null || document==null)return null;
		Element arcElement = document.createElement("arc");

		String idInput = "a" + arc.getID();
		String sourceInput;
		String targetInput;
		if (arc instanceof InArc) {
			sourceInput = "p" + arc.getPlace().getID();
			targetInput = "t" + arc.getTransition().getID();
		} else {
			targetInput = "p" + arc.getPlace().getID();
			sourceInput = "t" + arc.getTransition().getID();
		}

		arcElement
				.setAttribute("id", (idInput != null ? idInput : "error"));
		arcElement.setAttribute("source",
				(sourceInput != null ? sourceInput : ""));
		arcElement.setAttribute("target",
				(targetInput != null ? targetInput : ""));
		arcElement.setAttribute("label",
				AttributeHelper.getLabel(arc.getEdge(), ""));
		
		int inscription = Parser.parse2Int2(arc.getArcWeight().getValue().toString());
		if (inscription>0){
			arcElement.setAttribute("tokenType", "discrete");	//check this constraint
			arcElement.setAttribute("inscription", String.valueOf(inscription));
		}

		return arcElement;
	}	
}
