/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

import org.AttributeHelper;
import org.Vector2d;
import org.graffiti.graph.Edge;
import org.graffiti.graph.Graph;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;

// TODO: Auto-generated Javadoc
/**
 * The Class GraphHelper layouts the parallel edges of the reachability graph.
 */
//based on VANTED class IntroduceSelfEdgeBends version 1.6
public class GraphHelper {
	// self-loop
	/** The xdir. */
	private static double xdir = 15;
	
	/** The ydir. */
	private static double ydir = 5;
	
	/**
	 * Self loop.
	 *
	 * @param rGraph the r graph
	 * @return the graph
	 */
	public static Graph selfLoop(Graph rGraph) {
		HashMap<String, Integer> knownCombinations = new HashMap<String, Integer>();
		for (GraphElement ge : rGraph.getGraphElements()) {
			if (ge instanceof Edge) {
				Edge e = (Edge) ge;
				if (e.getSource() == e.getTarget()) {
					Integer knownFrequency = knownCombinations.get(e.getSource().getID() + "§" + e.getTarget().getID());
					if (knownFrequency == null) {
						knownFrequency = new Integer(1);
					} else {
						knownFrequency = new Integer(knownFrequency + 1);
					}
					knownCombinations.put(e.getSource().getID() + "§" + e.getTarget().getID(), new Integer(knownFrequency));
					knownCombinations.put(e.getTarget().getID() + "§" + e.getSource().getID(), new Integer(knownFrequency));
					
					AttributeHelper.removeEdgeBends(e);
					AttributeHelper.addEdgeBends(e, getSelfLoopBends(e, xdir * knownFrequency, ydir * knownFrequency * knownFrequency));
					AttributeHelper.setEdgeBendStyle(e, "org.graffiti.plugins.views.defaults.SmoothLineEdgeShape");
				}
			}
		}
		return rGraph;
	}
	
	/**
	 * Gets the self loop bends.
	 *
	 * @param e the e
	 * @param xdir the xdir
	 * @param ydir the ydir
	 * @return the self loop bends
	 */
	private static Collection<Vector2d> getSelfLoopBends(Edge e, double xdir, double ydir) {
		Collection<Vector2d> result = new ArrayList<Vector2d>();
		Node a = e.getSource();
		Vector2d np = AttributeHelper.getPositionVec2d(a);
		Vector2d size = AttributeHelper.getSize(a);
		Vector2d p1 = new Vector2d(np.x + size.x / 2 + xdir, np.y - size.y / 2 - ydir);
		Vector2d p2 = new Vector2d(np.x + size.x / 2 + xdir, np.y + size.y / 2 + ydir);
		result.add(p1);
		result.add(p2);
		return result;
	}
}
