/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.addon;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import javax.swing.JTabbedPane;

import org.graffiti.editor.MainFrame;
import org.graffiti.event.ListenerNotFoundException;
import org.graffiti.graph.Graph;
import org.graffiti.plugin.inspector.InspectorTab;
import org.graffiti.plugin.view.View;
import org.graffiti.session.Session;
import org.graffiti.session.SessionListener;

import pipe.analysis.reachability.ReachabilityGraphGenerator;
import vanted.gui.AnalysisGUI;
import vanted.gui.ReconstructionGUI;
import vanted.gui.SimulationGUI;

// TODO: Auto-generated Javadoc
/**
 * The Class PetriAddonTab creates the main tab.
 *
 * @author Kevin Pucknat
 */
public class PetriAddonTab extends InspectorTab
					implements
					SessionListener{

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** The reconstruction gui. */
	private ReconstructionGUI reGUI;

	/** The simulation gui. */
	private SimulationGUI simuGUI;

	/** The analysis gui. */
	private AnalysisGUI anaGUI;

	/** The graph. */
	private Graph graph;


	/**
	 * Instantiates a new petri addon tab, creates the tabs: reconstruction, simulation and analysis.
	 */
	public PetriAddonTab() {	
		setLayout(new TableLayout(new double[][] { 
				{TableLayoutConstants.FILL}, 		// Columns
				{TableLayoutConstants.PREFERRED} })); 	// Rows

		JTabbedPane tabbedPane = new JTabbedPane();
		reGUI= new ReconstructionGUI();		
		tabbedPane.addTab("Reconstruction", reGUI);
		
		simuGUI = new SimulationGUI();
		tabbedPane.addTab("Simulation", simuGUI);
		
		anaGUI = new AnalysisGUI();

		tabbedPane.addTab("Analysis", anaGUI);
		
		add(tabbedPane, "0,0");	
				
		validate();
	}
		
	/**
	 * Checks if the graph is a reachability graph, true if the graph name contains the word "reachabilityGraph".
	 *
	 * @param graph the graph
	 * @return true, if is reachability graph
	 */
	public static boolean isReachabilityGraph(Graph graph){
		if(graph.getName().contains(ReachabilityGraphGenerator.reachabilityGraphName)){
			MainFrame.showMessageDialog("<html>Select a valid Petri-Net window.", "Petri-Net");
			return true;
		}
		
		return false;
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.inspector.InspectorTab#getName()
	 */
	@Override
	public String getName() {
		return getTitle();
	}

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.inspector.InspectorTab#getTitle()
	 */
	@Override
	public String getTitle() {
		return "Petri-Net";
	}

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.inspector.InspectorTab#visibleForView(org.graffiti.plugin.view.View)
	 */
	@Override
	public boolean visibleForView(View v) {
		return v!=null;
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.session.SessionListener#sessionChanged(org.graffiti.session.Session)
	 */
	public void sessionChanged(Session s) {		
		if (s != null) {
			if (graph != null) {
				try {
					graph.getListenerManager().removeGraphListener(reGUI);
				} catch (ListenerNotFoundException e) {
//					ErrorMsg.addErrorMessage(e);
				}
			}
						
			graph = s.getGraph();
			anaGUI.graphChanged(graph);
			reGUI.graphChanged(graph);
			simuGUI.graphChanged(graph);	
			graph.getListenerManager().addDelayedGraphListener(reGUI);
		} else
			graph = null;
	}

	/* (non-Javadoc)
	 * @see org.graffiti.session.SessionListener#sessionDataChanged(org.graffiti.session.Session)
	 */
	public void sessionDataChanged(Session s) {
		sessionChanged(s); 
	}
}
