/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute;

import info.clearthought.layout.TableLayout;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeListener;

import org.graffiti.plugin.Displayable;
import org.graffiti.plugin.editcomponent.AbstractValueEditComponent;

// TODO: Auto-generated Javadoc
/**
 * The Class AbstractAttributeEditor handles the abstract attribute editor.
 *
 * @author Kevin Pucknat
 */
public abstract class AbstractAttributeEditor extends AbstractValueEditComponent implements ActionListener, ChangeListener{
	
	/** The spinner. */
	protected JSpinner spinner;	
	
	/** The btn. */
	protected JButton btn;
	
	/** The jp. */
	protected final JPanel jp;
	
	/** The value. */
	protected double value;
	
	/**
	 * Instantiates a new abstract attribute editor.
	 *
	 * @param disp the disp
	 */
	public AbstractAttributeEditor(Displayable disp) {
		super(disp);
		jp = new JPanel();
		jp.setOpaque(false);
		jp.setLayout(TableLayout.getLayout(TableLayout.FILL, TableLayout.FILL));
		
		displayable = null; 
		this.setDisplayable(disp);
	}	
		
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.editcomponent.ValueEditComponent#getComponent()
	 */
	public JComponent getComponent() {
		return jp;
	}

	/**
	 * Gets the spinner integer.
	 *
	 * @param value the value
	 * @return the spinner integer
	 */
	protected JSpinner getSpinnerInteger(int value){		
		return new JSpinner(new SpinnerNumberModel(value, 1, 1, 1)); 
	}
	

	/**
	 * Builds the gui.
	 */
	protected void buildGUI(){
		spinner.setMinimumSize(new Dimension(0, 20));
		spinner.setPreferredSize(new Dimension(50, 20));
		spinner.setMaximumSize(new Dimension(2000, 20));
		jp.add(TableLayout.getSplit(spinner, btn, TableLayout.FILL, TableLayout.FILL), "0,0");
	}
	
	
	/*
	 * @see org.graffiti.plugin.editcomponent.AbstractValueEditComponent#setShowEmpty(boolean)
	 */
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.editcomponent.AbstractValueEditComponent#setShowEmpty(boolean)
	 */
	@Override
	public void setShowEmpty(boolean showEmpty) {
		if (this.showEmpty != showEmpty) {
			super.setShowEmpty(showEmpty);
		}
		
		this.setEditFieldValue();
	}
	
	
	/**
	 * Sets the displayable.
	 *
	 * @param disp the new displayable
	 */
	@Override
	public void setDisplayable(Displayable disp) {
		this.displayable = disp;
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent e) {		
		if(spinner.isEnabled()){			
			spinner.setEnabled(false);			
			setShowEmpty(true);
		}else{
			spinner.setEnabled(true);			
			setShowEmpty(false);
		}
	}	
}
