/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute;

import org.AttributeHelper;
import org.graffiti.attributes.StringAttribute;
import org.graffiti.graph.Edge;

import vanted.petrinetelements.ArcWeight;

// TODO: Auto-generated Javadoc
/**
 * The Class ArcWeightAttribute handles the arc-weight attribute.
 *
 * @author Kevin Pucknat
 */
public class ArcWeightAttribute extends StringAttribute{
	
	/** The Constant path. */
	public static final String path = "petrinet";
	
	/** The Constant name. */
	public static final String name = "ArcWeight";	
	
	/** The Constant positionAttributeName. */
	public static final String positionAttributeName = "arcweightposition";
	
	/** The Constant sizeAttributeName. */
	public static final String sizeAttributeName = "arcweightsize";
	
	/** The Constant colorAttributeName. */
	public static final String colorAttributeName = "arcweightcolor";
	
	/** The Constant positionAttributeNameEdge. */
	public static final String positionAttributeNameEdge = "arcweightedgepos";

	/** The arc-weight. */
	private ArcWeight arcWeight;
	
	/**
	 * Instantiates a new arc-weight attribute.
	 */
	public ArcWeightAttribute() {
		super();
	}
	
	/**
	 * Instantiates a new arc-weight attribute.
	 *
	 * @param id the id
	 */
	public ArcWeightAttribute(String id) {
		super(id);
	}
	
	/**
	 * Instantiates a new arc-weight attribute.
	 *
	 * @param id the id
	 * @param value the value
	 */
	public ArcWeightAttribute(String id, String value) {
		super(id, value);
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#doSetValue(java.lang.Object)
	 */
	@Override
	protected void doSetValue(Object o) throws IllegalArgumentException {
		assert o != null;
		
		try {
			doSetValue((String) o);
		} catch (ClassCastException cce) {
			throw new IllegalArgumentException("Invalid value type.");
		}
	}
	
	/**
	 * Sets the arc-weight value.
	 *
	 * @param value the value
	 * @throws IllegalArgumentException the illegal argument exception
	 */
	protected void doSetValue(String value) throws IllegalArgumentException {
		setString(value);	
		arcWeight = new ArcWeight(value);			
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getString()
	 */
	@Override
	public String getString() {		
		String str = super.getString();
		
		if(str==null)
			return "";
			
		return str;
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getValue()
	 */
	@Override
	public Object getValue() {
		return getString();
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#setString(java.lang.String)
	 */
	@Override
	public void setString(String value) {
		super.setString(value);
//		doSetValue(value);
	}
	
	/**
	 * Gets the arc-weight.
	 *
	 * @return the arc-weight
	 */
	public ArcWeight getArcWeight() {
		if(arcWeight!=null)
			return arcWeight;
		else{
			doSetValue(getString());
			return arcWeight;
		}
	}
	

	/**
	 * Adds the arc-weight.
	 *
	 * @param arcWeight the arc-weight
	 */
	public void addArcWeight(ArcWeight arcWeight) {
		setString(arcWeight.getValue().toString());		
		this.arcWeight = arcWeight;
	}


	/**
	 * Removes the arc-weight.
	 *
	 * @return true, if successful
	 */
	public boolean removeArcWeight() {
		setString("");
		arcWeight = null;	
		if(getAttributable() instanceof Edge){
			Edge edge = (Edge)getAttributable();
			AttributeHelper.deleteAttribute(edge, ArcWeightAttribute.path, ArcWeightAttribute.name);
			AttributeHelper.deleteAttribute(edge, ArcWeightAttribute.path, ArcWeightAttribute.colorAttributeName);
			AttributeHelper.deleteAttribute(edge, ArcWeightAttribute.path, ArcWeightAttribute.sizeAttributeName);
		}
		return true;
	}
}
