/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.text.DecimalFormat;
import java.text.NumberFormat;

import org.AttributeHelper;
import org.color.ColorUtil;
import org.graffiti.attributes.Attributable;
import org.graffiti.attributes.Attribute;
import org.graffiti.graph.Edge;
import org.graffiti.graph.GraphElement;
import org.graffiti.plugin.attributecomponent.AbstractAttributeComponent;
import org.graffiti.plugin.view.ShapeNotFoundException;

import vanted.attribute.token.TokenAttributeContinuous;
import vanted.petrinetelements.PetriNet;

// TODO: Auto-generated Javadoc
/**
 * The Class ArcWeightAttributeComponent visualizes the arc-weight.
 *
 * @author Kevin Pucknat
 */
public class ArcWeightAttributeComponent extends AbstractAttributeComponent {
	
	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;


	/** The point. */
	private Point point = new Point();
	
	/** The strlength. */
	private int strlength=10;
	
	/** The arcweightsize. */
	private int arcweightsize = 18;
	
	/** The color. */
	private Color color = Color.BLUE;

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#attributeChanged(org.graffiti.attributes.Attribute)
	 */
	@Override
	public void attributeChanged(Attribute attr) throws ShapeNotFoundException {
		if (attr.getParent().getName().equals(attr.getParent().getName()))
			recreate();
	}


	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#recreate()
	 */
	@Override
	public void recreate() throws ShapeNotFoundException {
		GraphElement ge = (GraphElement) this.attr.getAttributable();
		
		if(ge instanceof Edge){
			if(AttributeHelper.hasAttribute((Edge)ge, ArcWeightAttribute.path, ArcWeightAttribute.sizeAttributeName))
				arcweightsize = (Integer) AttributeHelper.getAttributeValue(ge, ArcWeightAttribute.path, ArcWeightAttribute.sizeAttributeName, 18, 18);
		
			if(AttributeHelper.hasAttribute((Edge)ge, ArcWeightAttribute.path, ArcWeightAttribute.colorAttributeName))
				color = ColorUtil.getColorFromHex((String) AttributeHelper.getAttributeValue(ge, ArcWeightAttribute.path, ArcWeightAttribute.colorAttributeName, ColorUtil
						.getHexFromColor(color), "", true));
		}
		
		updatePosition(ge);
		
		setSize(strlength, arcweightsize);
		validate();
		repaint();
	}

	/**
	 * Update the arc-weight position.
	 *
	 * @param attributable the attributable
	 */
	protected void updatePosition(Attributable attributable) {
		if (attributable instanceof Edge) {
			double flatness = 1d;
			LabelComponentHelper.updatePositionForEdgeMapping(getAttribute(), geShape,
								ArcWeightAttribute.positionAttributeNameEdge,
								ArcWeightAttribute.path,
								flatness, getWidth(), getHeight(), point);
			setLocation((int) (point.getX() + shift.getX()), -1 + (int) (point.getY() + shift.getY()));
		}
	}

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#setShift(java.awt.Point)
	 */
	@Override
	public void setShift(Point shift) {
		this.shift = shift;
		if (attr != null && attr.getAttributable() != null)
			updatePosition(attr.getAttributable());
	}

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.view.AttributeComponent#adjustComponentSize()
	 */
	@Override
	public void adjustComponentSize() {
		if (attr != null && attr.getAttributable() != null)
			updatePosition(attr.getAttributable());
	}

	
	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g) {
		super.paintComponent(g);
		
		if(new PetriNet(((Edge)attr.getAttributable()).getGraph()).getHideArcWeight().getValue())
			return;
		
		double weight = 1;
		if (attr.getValue() != null && !attr.getValue().toString().equals(""))
			weight = Double.parseDouble(attr.getValue().toString());
		
		if(weight!=1){
			NumberFormat formatter = new DecimalFormat(TokenAttributeContinuous.decimal);
			String str = formatter.format(weight);
			
			g.setColor(color);
			g.setFont(new Font("arial", Font.BOLD, arcweightsize));	
			
			strlength = g.getFontMetrics().stringWidth(str);		
			setSize(strlength, g.getFontMetrics().getAscent());
			adjustComponentSize();

			g.drawString(str, 0, g.getFontMetrics().getAscent()-g.getFontMetrics().getDescent()+g.getFontMetrics().getLeading());	
			
		}
	}
}
