/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute;


import org.AttributeHelper;
import org.graffiti.attributes.StringAttribute;
import org.graffiti.graph.Node;

import vanted.petrinetelements.Capacity;

// TODO: Auto-generated Javadoc
/**
 * The Class CapacityAttribute handles the capacity attribute.
 *
 * @author Kevin Pucknat
 */
public class CapacityAttribute extends StringAttribute {
	
	/** The capacity. */
	private Capacity capacity;
		
	/** The Constant path. */
	public static final String path = "petrinet";
	
	/** The Constant name. */
	public static final String name = "place_capacity";
	
	/** The Constant positionAttributeName. */
	public static final String positionAttributeName = "capacityposition";
	
	/** The Constant colorAttributeName. */
	public static final String colorAttributeName = "capacitycolor";	
	
	/** The Constant sizeAttributeName. */
	public static final String sizeAttributeName = "capacitysize";
	
	/**
	 * Instantiates a new capacity attribute.
	 */
	public CapacityAttribute() {
		super();
	}
	
	/**
	 * Instantiates a new capacity attribute.
	 *
	 * @param id the id
	 */
	public CapacityAttribute(String id) {
		super(id);
	}
	
	/**
	 * Instantiates a new capacity attribute.
	 *
	 * @param id the id
	 * @param value the value
	 */
	public CapacityAttribute(String id, String value) {
		super(id, value);
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#doSetValue(java.lang.Object)
	 */
	@Override
	protected void doSetValue(Object o) throws IllegalArgumentException {
		assert o != null;
		
		try {
			doSetValue((String) o);
		} catch (ClassCastException cce) {
			throw new IllegalArgumentException("Invalid value type.");
		}
	}
	

	/**
	 * Sets the capacity value.
	 *
	 * @param value the value
	 * @throws IllegalArgumentException the illegal argument exception
	 */
	protected void doSetValue(String value) throws IllegalArgumentException {
		setString(value);
		capacity = new Capacity(value);			
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getString()
	 */
	@Override
	public String getString() {		
		String str = super.getString();
		
		if(str==null)
			return "";
			
		return str;
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getValue()
	 */
	@Override
	public Object getValue() {
		return getString();
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#setString(java.lang.String)
	 */
	@Override
	public void setString(String value) {
		super.setString(value);
	}

	/**
	 * Gets the capacity.
	 *
	 * @return the capacity
	 */
	public Capacity getCapacity() {
		if(capacity!=null)
			return capacity;
		else{
			doSetValue(getString());
			return capacity;
		}
	}
	

	/**
	 * Adds the capacity.
	 *
	 * @param capacity the capacity
	 */
	public void addCapacity(Capacity capacity) {
		setString(capacity.getValue().toString());		
		this.capacity = capacity;
	}

	/**
	 * Removes the capacity and all other attributes: name, position, color and size.
	 *
	 * @return true, if successful
	 */
	public boolean removeCapacity() {
		setString("");		
		capacity = null;
		if(getAttributable() instanceof Node){
			Node node = (Node)getAttributable();
			AttributeHelper.deleteAttribute(node, CapacityAttribute.path, CapacityAttribute.name);
			AttributeHelper.deleteAttribute(node, CapacityAttribute.path, CapacityAttribute.positionAttributeName);
			AttributeHelper.deleteAttribute(node, CapacityAttribute.path, CapacityAttribute.colorAttributeName);
			AttributeHelper.deleteAttribute(node, CapacityAttribute.path, CapacityAttribute.sizeAttributeName);
		}
				
		return true;
	}
}
