/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.text.DecimalFormat;
import java.text.NumberFormat;

import org.AttributeHelper;
import org.color.ColorUtil;
import org.graffiti.attributes.Attributable;
import org.graffiti.attributes.Attribute;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;
import org.graffiti.graphics.GraphicAttributeConstants;
import org.graffiti.plugin.attributecomponent.AbstractAttributeComponent;
import org.graffiti.plugin.view.ShapeNotFoundException;

import vanted.attribute.token.TokenAttributeContinuous;
import vanted.petrinetelements.PetriNet;

// TODO: Auto-generated Javadoc
/**
 * The Class CapacityAttributeComponent visualizes the capacity.
 *
 * @author Kevin Pucknat
 */
public class CapacityAttributeComponent extends AbstractAttributeComponent {
	
	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** The point. */
	private Point point = new Point();
	
	/** The strlength. */
	private int strlength=10;
	
	/** The capacitysize. */
	private int capacitysize = 18;
	
	/** The color. */
	private Color color = Color.GREEN;
	
	/** The align. */
	private String align = GraphicAttributeConstants.BORDER_TOP_RIGHT;

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#attributeChanged(org.graffiti.attributes.Attribute)
	 */
	@Override
	public void attributeChanged(Attribute attr) throws ShapeNotFoundException {
		if (attr.getParent().getName().equals(attr.getParent().getName()))
			recreate();
	}

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#recreate()
	 */
	@Override
	public void recreate() throws ShapeNotFoundException {
		GraphElement ge = (GraphElement) this.attr.getAttributable();
		if(ge instanceof Node){
			if(AttributeHelper.hasAttribute((Node)ge, CapacityAttribute.path, CapacityAttribute.sizeAttributeName))
				capacitysize = (Integer) AttributeHelper.getAttributeValue(ge, CapacityAttribute.path, CapacityAttribute.sizeAttributeName, 18, 18, true);
				
			if(AttributeHelper.hasAttribute((Node)ge, CapacityAttribute.path, CapacityAttribute.colorAttributeName))
				color = ColorUtil.getColorFromHex((String) AttributeHelper.getAttributeValue(ge, CapacityAttribute.path, CapacityAttribute.colorAttributeName, ColorUtil
						.getHexFromColor(color), "", true));
		}
		
		updatePosition(ge);
		
		setSize(strlength, capacitysize);	
		
		validate();
		repaint();
	}

	/**
	 * Update the capacity position.
	 *
	 * @param attributable the attributable
	 */
	protected void updatePosition(Attributable attributable) {
		if (attributable instanceof Node) {
			Node n = (Node) attributable;

			if(AttributeHelper.hasAttribute(attributable, CapacityAttribute.path, CapacityAttribute.positionAttributeName))
				align = (String) AttributeHelper.getAttributeValue(attributable, CapacityAttribute.path, CapacityAttribute.positionAttributeName,
								GraphicAttributeConstants.BORDER_TOP_RIGHT, "", true);

			LabelComponentHelper.calculateLabelPositionForNode(getWidth(), getHeight(), align, n, geShape, point, null);

			setLocation((int) (point.getX() + shift.getX()), (int) (point.getY() + shift.getY()));

		}
	}

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#setShift(java.awt.Point)
	 */
	@Override
	public void setShift(Point shift) {
		this.shift = shift;
		if (attr != null && attr.getAttributable() != null)
			updatePosition(attr.getAttributable());
	}

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.view.AttributeComponent#adjustComponentSize()
	 */
	@Override
	public void adjustComponentSize() {
		if (attr != null && attr.getAttributable() != null)
			updatePosition(attr.getAttributable());
	}

	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g) {
		super.paintComponent(g);

		if(new PetriNet(((Node)attr.getAttributable()).getGraph()).getHideCapacity().getValue())
			return;
		
		double capacity = 0; 
		if (attr.getValue() != null && !attr.getValue().toString().equals("")){
			capacity=Double.parseDouble(attr.getValue().toString());
			if(capacity>0){
				NumberFormat formatter = new DecimalFormat(TokenAttributeContinuous.decimal);
				String str = formatter.format(capacity);

				g.setColor(color);
				g.setFont(new Font("arial", Font.BOLD, capacitysize));	
				
				strlength = g.getFontMetrics().stringWidth(str);		
				setSize(strlength, g.getFontMetrics().getAscent());
				adjustComponentSize();

				g.drawString(str, 0, g.getFontMetrics().getAscent()-g.getFontMetrics().getDescent()+g.getFontMetrics().getLeading());
			}				
		}
	}
}
