/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.List;

import org.AttributeHelper;
import org.color.ColorUtil;
import org.graffiti.attributes.Attributable;
import org.graffiti.attributes.Attribute;
import org.graffiti.editor.MainFrame;
import org.graffiti.graph.Edge;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;
import org.graffiti.graphics.GraphicAttributeConstants;
import org.graffiti.plugin.attributecomponent.AbstractAttributeComponent;
import org.graffiti.plugin.view.ShapeNotFoundException;
import org.graffiti.session.Session;

import pipe.analysis.reachability.ReachabilityGraphGenerator;
import vanted.attribute.token.TokenAttributeContinuous;
import vanted.gui.AnalysisGUI;
import vanted.petrinetelements.NodePN;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.Transition;
import vanted.petrinetelements.misc.DrawingNode;

// TODO: Auto-generated Javadoc
/**
 * The Class DrawAttributeComponent visualizes the drawing.
 *
 * @author Kevin Pucknat
 */
public class DrawAttributeComponent extends AbstractAttributeComponent {
	
	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;
	
	/** The Constant edgeNumber. Is set as a reachability graph attribute.*/
	public static final int edgeNumber = -3;
	
	/** The Constant nodeNumber. Is set as a reachability graph attribute.*/
	public static final int nodeNumber = -2;
	
	/** The loc. */
	private Point point = new Point();
	
	/** The strlength. */
	private int strlength=10;
	
	/** The drawvaluesize. */
	private int drawvaluesize = 18;
	
	/** The color. */
	private Color color = Color.MAGENTA;
	
	/** The align. */
	private String align = GraphicAttributeConstants.CENTERED;

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#attributeChanged(org.graffiti.attributes.Attribute)
	 */
	@Override
	public void attributeChanged(Attribute attr) throws ShapeNotFoundException {
		if (attr.getParent().getName().equals(attr.getParent().getName()))
			recreate();
	}

	/**
	 * Is called every time the attribute value is changed and if it is not a
	 * change in size, shift or position.
	 *
	 * @throws ShapeNotFoundException the shape not found exception
	 */
	@Override
	public void recreate() throws ShapeNotFoundException {
		GraphElement ge = (GraphElement) this.attr.getAttributable();
				
		color = ColorUtil.getColorFromHex((String) AttributeHelper.getAttributeValue(ge, attr.getParent().getPath(), "drawcolor", ColorUtil
				.getHexFromColor(color), "", true));
		
		drawvaluesize = (int) (geShape.getBounds().width * 0.7);
		updatePosition(ge);
		
		setSize(strlength, drawvaluesize);			
		validate();
		repaint();
	}

	/**
	 * Update the draw position.
	 *
	 * @param attributable the attributable
	 */
	protected void updatePosition(Attributable attributable) {
		if (attributable instanceof Node) {
			Node n = (Node) attributable;
			if (AttributeHelper.hasAttribute(attributable, DrawingNodeAttribute.path, DrawingNodeAttribute.positionAttributeName) && !(attr instanceof IgnoreNodeAttribute))
				align = (String) AttributeHelper.getAttributeValue(attributable, DrawingNodeAttribute.path, DrawingNodeAttribute.positionAttributeName,
								GraphicAttributeConstants.CENTERED, "", true);

			LabelComponentHelper.calculateLabelPositionForNode(getWidth(), getHeight(), align, n, geShape, point, null);
			setLocation((int) (point.getX() + shift.getX()), (int) (point.getY() + shift.getY()));
		}
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.view.AttributeComponent#highlight(boolean, java.awt.event.MouseEvent)
	 */
	@Override
	public void highlight(boolean value, MouseEvent e) {		 
		if (attr.getValue() != null){
			if(attr instanceof DrawingNodeAttribute && !attr.getValue().toString().isEmpty()){
				
				int pos = -1;
				if(attr.getAttributable() instanceof Edge)
						pos = ((Edge)attr.getAttributable()).getGraph().getName().indexOf(AnalysisGUI.seperator);
				String r = "";
				if(pos>0)
					r = ((Node)attr.getAttributable()).getGraph().getName().substring(((Node)attr.getAttributable()).getGraph().getName().indexOf(AnalysisGUI.seperator), ((Node)attr.getAttributable()).getGraph().getName().length());				
				
				for (Session session : MainFrame.getEditorSessions()) {
					if (session.getGraph().getName().contains(r) && !session.getGraph().getName().contains(ReachabilityGraphGenerator.reachabilityGraphName)) {						
						if(value){
							if(attr.getAttributable() instanceof Node){
								PetriNet pn = new PetriNet();
								pn.createPetriNet(session.getGraph(), true);
								showDrawing(pn,(Node)attr.getAttributable());
								AttributeHelper.setOutlineColor(attr.getAttributable(), Color.RED);
							}
						}else{
							for(Node node : session.getGraph().getNodes())
								new NodePN(node).removeDraw();
							AttributeHelper.setOutlineColor(attr.getAttributable(), Color.BLACK);
						}
					}
				}	
			}
			else if(attr instanceof DrawingEdgeAttribute){			
				int pos = -1;
				Edge edge=null;
				if(attr.getAttributable() instanceof Edge){
					edge = (Edge)attr.getAttributable();
					pos = (edge).getGraph().getName().indexOf(AnalysisGUI.seperator);
				}
				
				String r = "";
				if(edge!=null && pos>0)
					r = (edge).getGraph().getName().substring(pos, (edge).getGraph().getName().length());				
				
				for (Session session : MainFrame.getEditorSessions()) {
					if (!r.isEmpty() && session.getGraph().getName().contains(r) && !session.getGraph().getName().contains(ReachabilityGraphGenerator.reachabilityGraphName)) {
						PetriNet pn = new PetriNet();
						pn.createPetriNet(session.getGraph(), true);

						Transition tr=null;
						for(Transition transition : pn.getAllTransitions()){
							if(transition.getIdentity().getValue() == ((Edge)attr.getAttributable()).getID()){
								tr = transition;								
							}
						}
						
						if(tr==null)return;
						
						if(value){								
							tr.addDraw(new DrawingNode(edgeNumber));
							AttributeHelper.setFillColor(tr.getNode(), Color.RED);							
							AttributeHelper.setOutlineColor(attr.getAttributable(), Color.RED);
						}
						else{
							AttributeHelper.setFillColor(tr.getNode(), Color.WHITE);
							AttributeHelper.setOutlineColor(attr.getAttributable(), Color.BLACK);
						}
					}
				}			
			}
		}
	}
	
	/**
	 * Show drawing, adds the <code>DrawingNode</code> attribute to a place.
	 *
	 * @param pn the pn
	 * @param node the node
	 */
	private void showDrawing(PetriNet pn, Node node) {
		List<Place> places = pn.getAllPlaces();		
		int pos = 0;
		for (String str : (new NodePN(node)).getReachability().getValue()
				.split(ReachabilityAttribute.seperator)) {
			if (!str.isEmpty()){
				places.get(pos).addDraw(new DrawingNode(Double.valueOf(str)));				
			}
			pos++;
		}		
	}	
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#setShift(java.awt.Point)
	 */
	@Override
	public void setShift(Point shift) {
		this.shift = shift;
		if (attr != null && attr.getAttributable() != null)
			updatePosition(attr.getAttributable());
	}

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.view.AttributeComponent#adjustComponentSize()
	 */
	@Override
	public void adjustComponentSize() {
		if (attr != null && attr.getAttributable() != null)
			updatePosition(attr.getAttributable());
	}

	
	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g) {
		super.paintComponent(g);
		
		double value = 0; 
		if (attr.getValue() != null && !attr.getValue().toString().isEmpty()){				
			if(attr instanceof DrawingNodeAttribute){								
				value=Double.valueOf(attr.getValue().toString());
				
				if(value==nodeNumber || value==edgeNumber){	// don't draw in the reachability graph window					
					return;
				}
				
				if(value!=0){
					NumberFormat formatter = new DecimalFormat(TokenAttributeContinuous.decimal);
					String str = formatter.format(value);
				
					if(str.contains("-1"))
						str = str.replace("-1", "\u03C9");
														
					g.setFont(new Font("arial", Font.BOLD, drawvaluesize));					
					strlength = g.getFontMetrics().stringWidth(str);		
					setSize(strlength, g.getFontMetrics().getAscent());
					adjustComponentSize();
					
					g.setColor(color);

					g.setClip(-strlength, -g.getFontMetrics().getAscent(), strlength * 2, g.getFontMetrics().getAscent() * 2);
					g.drawString(str, 0, g.getFontMetrics().getAscent()-g.getFontMetrics().getDescent()+g.getFontMetrics().getLeading());
				}
			}	
			
			//hide node
			if(attr instanceof IgnoreNodeAttribute){
				g.setColor(Color.RED);
				
				if(geShape.getBounds().width > 150 || geShape.getBounds().height > 150){		
					g.setFont(new Font("arial", Font.BOLD, (int)(Math.sqrt(Math.max(geShape.getBounds().width, geShape.getBounds().height)))*2));
					align = GraphicAttributeConstants.BELOWLEFT;
				}else{
					g.setFont(new Font("arial", Font.BOLD, (int)(geShape.getBounds().width*0.9)));
					align = GraphicAttributeConstants.CENTERED;
				}
											
				strlength = g.getFontMetrics().stringWidth("X");	
				setSize(strlength, drawvaluesize);	
				adjustComponentSize();
				g.drawString("X", 0, g.getFontMetrics().getAscent()-g.getFontMetrics().getLeading()*2);				
			}
		}
	}	
}
