/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute;

import org.graffiti.attributes.StringAttribute;

import vanted.petrinetelements.misc.DrawingNode;

// TODO: Auto-generated Javadoc
/**
 * The Class DrawingNodeAttribute handles the drawing node attribute.
 *
 * @author Kevin Pucknat
 */
public class DrawingNodeAttribute extends StringAttribute {
	
	/** The draw. */
	private DrawingNode draw;
	
	/** The Constant path. */
	public static final String path = "petrinet";
	
	/** The Constant name. */
	public static final String name = "DrawingNode";
	
	/** The Constant positionAttributeName. */
	public static final String positionAttributeName = "drawposition";
	
	/**
	 * Instantiates a new drawing node attribute.
	 */
	public DrawingNodeAttribute() {
		super();
	}
	
	/**
	 * Instantiates a new drawing node attribute.
	 *
	 * @param id the id
	 */
	public DrawingNodeAttribute(String id) {
		super(id);
	}
	
	/**
	 * Instantiates a new drawing node attribute.
	 *
	 * @param id the id
	 * @param value the value
	 */
	public DrawingNodeAttribute(String id, String value) {
		super(id, value);
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#doSetValue(java.lang.Object)
	 */
	@Override
	protected void doSetValue(Object o) throws IllegalArgumentException {
		assert o != null;
		
		try {
			doSetValue((String) o);
		} catch (ClassCastException cce) {
			throw new IllegalArgumentException("Invalid value type.");
		}
	}
	

	/**
	 * Sets the drawing node value.
	 *
	 * @param value the value
	 * @throws IllegalArgumentException the illegal argument exception
	 */
	protected void doSetValue(String value) throws IllegalArgumentException {
		setString(value);
		draw = new DrawingNode(Double.valueOf(value));			
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getString()
	 */
	@Override
	public String getString() {		
		String str = super.getString();
		
		if(str==null)
			return "";
			
		return str;
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getValue()
	 */
	@Override
	public Object getValue() {
		return getString();
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#setString(java.lang.String)
	 */
	@Override
	public void setString(String value) {
		super.setString(value);
	}

	/**
	 * Gets the draw.
	 *
	 * @return the draw
	 */
	public DrawingNode getDraw() {
		return draw;
	}
	

	/**
	 * Adds the draw.
	 *
	 * @param draw the draw
	 */
	public void addDraw(DrawingNode draw) {
		setString(String.valueOf(draw.getValue()));		
		this.draw = draw;
	}


	/**
	 * Removes the draw.
	 *
	 * @return true, if successful
	 */
	public boolean removeDraw() {
		setString("");
		draw = null;
		return true;
	}
}
