/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute;

import org.graffiti.attributes.StringAttribute;

import vanted.petrinetelements.misc.HideArcWeight;

// TODO: Auto-generated Javadoc
/**
 * The Class HideArcWeightAttribute handles the hide arc-weight attribute.
 *
 * @author Kevin Pucknat
 */
public class HideArcWeightAttribute extends StringAttribute {
	
	/** The hide arc weight. */
	private HideArcWeight hideArcWeight;
	
	/** The Constant path. */
	public static final String path = "petrinet";
	
	/** The Constant name. */
	public static final String name = "HideArcWeight";
	
	/**
	 * Instantiates a new hide arc weight attribute.
	 */
	public HideArcWeightAttribute() {
		super();
	}
	
	/**
	 * Instantiates a new hide arc weight attribute.
	 *
	 * @param id the id
	 */
	public HideArcWeightAttribute(String id) {
		super(id);
	}
	
	/**
	 * Instantiates a new hide arc weight attribute.
	 *
	 * @param id the id
	 * @param value the value
	 */
	public HideArcWeightAttribute(String id, String value) {
		super(id, value);
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#doSetValue(java.lang.Object)
	 */
	@Override
	protected void doSetValue(Object o) throws IllegalArgumentException {
		assert o != null;
		
		try {
			doSetValue((String) o);
		} catch (ClassCastException cce) {
			throw new IllegalArgumentException("Invalid value type.");
		}
	}
	

	/**
	 * Sets the hide arc-weight value.
	 *
	 * @param value the value
	 * @throws IllegalArgumentException the illegal argument exception
	 */
	protected void doSetValue(String value) throws IllegalArgumentException {
		setString(value);
		hideArcWeight = new HideArcWeight(Boolean.valueOf(value));			
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getString()
	 */
	@Override
	public String getString() {		
		String str = super.getString();
		
		if(str==null)
			return "";
			
		return str;
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getValue()
	 */
	@Override
	public Object getValue() {
		return getString();
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#setString(java.lang.String)
	 */
	@Override
	public void setString(String value) {
		super.setString(value);
	}

	/**
	 * Gets the hide arc-weight.
	 *
	 * @return the hide arc-weight
	 */
	public HideArcWeight getHideArcWeight() {
		return hideArcWeight;
	}
	

	/**
	 * Adds the hide arc-weight.
	 *
	 * @param hideArcWeight the hide arc-weight
	 */
	public void addHideArcWeight(HideArcWeight hideArcWeight) {
		setString(String.valueOf(hideArcWeight.getValue()));		
		this.hideArcWeight = hideArcWeight;
	}


	/**
	 * Removes the hide arc-weight.
	 *
	 * @return true, if successful
	 */
	public boolean removeHideArcWeight() {
		setString("");
		hideArcWeight = null;
		return true;
	}
}
