/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute;

import org.graffiti.attributes.StringAttribute;

import vanted.petrinetelements.misc.IgnoreNode;

// TODO: Auto-generated Javadoc
/**
 * The Class IgnoreNodeAttribute handles the ignore node attribute.
 *
 * @author Kevin Pucknat
 */
public class IgnoreNodeAttribute extends StringAttribute {
	
	/** The ignore node. */
	private IgnoreNode ignoreNode;
	
	/** The Constant path. */
	public static final String path = "petrinet";
	
	/** The Constant name. */
	public static final String name = "IgnoreNode";
	
	/**
	 * Instantiates a new ignore node attribute.
	 */
	public IgnoreNodeAttribute() {
		super();
	}
	
	/**
	 * Instantiates a new ignore node attribute.
	 *
	 * @param id the id
	 */
	public IgnoreNodeAttribute(String id) {
		super(id);
	}
	
	/**
	 * Instantiates a new ignore node attribute.
	 *
	 * @param id the id
	 * @param value the value
	 */
	public IgnoreNodeAttribute(String id, String value) {
		super(id, value);
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#doSetValue(java.lang.Object)
	 */
	@Override
	protected void doSetValue(Object o) throws IllegalArgumentException {
		assert o != null;
		
		try {
			doSetValue((String) o);
		} catch (ClassCastException cce) {
			throw new IllegalArgumentException("Invalid value type.");
		}
	}
	

	/**
	 * Sets the ignore node value.
	 *
	 * @param value the value
	 * @throws IllegalArgumentException the illegal argument exception
	 */
	protected void doSetValue(String value) throws IllegalArgumentException {
		setString(value);
		ignoreNode = new IgnoreNode(Boolean.valueOf(value));			
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getString()
	 */
	@Override
	public String getString() {		
		String str = super.getString();
		
		if(str==null)
			return "";
			
		return str;
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getValue()
	 */
	@Override
	public Object getValue() {
		return getString();
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#setString(java.lang.String)
	 */
	@Override
	public void setString(String value) {
		super.setString(value);
	}

	/**
	 * Gets the ignore node.
	 *
	 * @return the ignore node
	 */
	public IgnoreNode getIgnoreNode() {
		return ignoreNode;
	}
	

	/**
	 * Adds the ignore node.
	 *
	 * @param ignoreNode the ignore node
	 */
	public void addIgnoreNode(IgnoreNode ignoreNode) {
		setString(String.valueOf(ignoreNode.getValue()));		
		this.ignoreNode = ignoreNode;
	}


	/**
	 * Removes the ignore node.
	 *
	 * @return true, if successful
	 */
	public boolean removeIgnoreNode() {
		setString("");
		ignoreNode = null;
		return true;
	}
}
