/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute;

import java.awt.Point;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;

import org.AttributeHelper;
import org.ErrorMsg;
import org.graffiti.attributes.Attributable;
import org.graffiti.attributes.Attribute;
import org.graffiti.attributes.AttributeNotFoundException;
import org.graffiti.attributes.CollectionAttribute;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;
import org.graffiti.graphics.EdgeLabelPositionAttribute;
import org.graffiti.graphics.GraphicAttributeConstants;
import org.graffiti.graphics.NodeLabelPositionAttribute;
import org.graffiti.plugin.view.GraphElementShape;
import org.graffiti.plugin.view.NodeShape;
import org.graffiti.plugins.attributecomponents.simplelabel.PointPair;
import org.graffiti.plugins.attributecomponents.simplelabel.Quadrant;
import org.graffiti.util.Pair;

//copied from the Add-on-Example, LabelComponentHelper version 1.6
public class LabelComponentHelper implements GraphicAttributeConstants {

	public static void calculateLabelPositionForNode(double labelWidth,
						double labelHeight, String align, GraphElement ge,
						GraphElementShape geShape, Point loc,
						NodeLabelPositionAttribute posAttr) {
		double sizeX = geShape.getBounds2D().getWidth();
		double sizeY = geShape.getBounds2D().getHeight();

		if (geShape instanceof NodeShape) {
			sizeX += ((NodeShape) geShape).shapeWidthCorrection();
			sizeY += ((NodeShape) geShape).shapeHeightCorrection();
		}

		double borderWidth = AttributeHelper.getFrameThickNess(ge);

		// calculate position relative to this component ...
		double coordX = 0; // top left
		double coordY = 0; // bottom right
		double centerX = (sizeX / 2d);
		double centerY = sizeY / 2d;

		if (geShape.getXexcess() > 0) {
			coordX += geShape.getXexcess();
			centerX = (sizeX / 2d);
		}
		if (geShape.getYexcess() > 0) {
			coordY += geShape.getYexcess();
			centerY = (sizeY / 2d + geShape.getYexcess());
		}

		align = processAutoAlignment(align, ge);
		if (CENTERED.equals(align)) {
			loc.setLocation(coordX + centerX - (labelWidth / 2.0d), centerY
								- (labelHeight / 2.0d));
		} else
			if (BELOW.equals(align)) {
				loc.setLocation(coordX + centerX - (labelWidth / 2.0d), coordY
									+ sizeY + LABEL_DISTANCE);
			} else
				if (BELOWRIGHT.equals(align)) {
					loc.setLocation(coordX + sizeX + LABEL_DISTANCE, coordY + sizeY
										+ LABEL_DISTANCE);
				} else
					if (BELOWLEFT.equals(align)) {
						loc.setLocation(coordX - labelWidth - LABEL_DISTANCE, coordY
											+ sizeY + LABEL_DISTANCE);
					} else
						if (BORDER_BOTTOM_CENTER.equals(align)) {
							loc.setLocation(coordX + sizeX / 2d - (labelWidth / 2d), coordY
												+ sizeY - labelHeight / 2d - borderWidth / 2d);
						} else
							if (BORDER_BOTTOM_LEFT.equals(align)) {
								loc.setLocation(coordX + sizeX / 2d - (labelWidth / 2d) - sizeX
													* (1d / 2d - 1d / 4d), coordY + sizeY - labelHeight / 2d
													- borderWidth / 2d);
							} else
								if (BORDER_BOTTOM_RIGHT.equals(align)) {
									loc.setLocation(coordX + sizeX / 2d - (labelWidth / 2d) + sizeX
														* (1d / 2d - 1d / 4d), coordY + sizeY - labelHeight / 2d
														- borderWidth / 2d);
								} else
									if (BORDER_TOP_CENTER.equals(align)) {
										loc.setLocation(coordX + sizeX / 2d - (labelWidth / 2d), coordY
															- labelHeight / 2d + borderWidth / 2d);
									} else
										if (BORDER_TOP_LEFT.equals(align)) {
											loc.setLocation(coordX + sizeX / 2d - (labelWidth / 2d) - sizeX
																* (1d / 2d - 1d / 4d), coordY - labelHeight / 2d
																+ borderWidth / 2d);
										} else
											if (BORDER_TOP_RIGHT.equals(align)) {
												loc.setLocation(coordX + sizeX / 2d - (labelWidth / 2d) + sizeX
																	* (1d / 2d - 1d / 4d), coordY - labelHeight / 2d
																	+ borderWidth / 2d);
											} else
												if (INSIDEBOTTOM.equals(align)) {
													loc.setLocation(coordX + centerX - (labelWidth / 2.0d),
																		(coordY + sizeY) - labelHeight - LABEL_DISTANCE
																							- borderWidth);
												} else
													if (ABOVE.equals(align)) {
														loc.setLocation(coordX + centerX - (labelWidth / 2.0d), coordY
																			- LABEL_DISTANCE - labelHeight);
													} else
														if (ABOVELEFT.equals(align)) {
															loc.setLocation(coordX - labelWidth - LABEL_DISTANCE, coordY
																				- LABEL_DISTANCE - labelHeight);
														} else
															if (LEFT.equals(align)) {
																loc.setLocation(
																					coordX - labelWidth - LABEL_DISTANCE/*-label.offX()*/,
																					centerY - (labelHeight / 2.0d));
															} else
																if (BORDER_LEFT_TOP.equals(align)) {
																	loc.setLocation(coordX - labelWidth / 2d + borderWidth / 2d,
																						centerY - (labelHeight / 2.0d) - sizeY
																											* (1d / 2d - 1d / 4d));
																} else
																	if (BORDER_LEFT_CENTER.equals(align)) {
																		loc.setLocation(coordX - labelWidth / 2d + borderWidth / 2d,
																							centerY - (labelHeight / 2.0d));
																	} else
																		if (BORDER_LEFT_BOTTOM.equals(align)) {
																			loc.setLocation(coordX - labelWidth / 2d + borderWidth / 2d,
																								centerY - (labelHeight / 2.0d) + sizeY
																													* (1d / 2d - 1d / 4d));
																		} else
																			if (BORDER_RIGHT_TOP.equals(align)) {
																				loc.setLocation(
																									coordX + sizeX - labelWidth / 2d - borderWidth / 2d,
																									centerY - (labelHeight / 2.0d) - sizeY
																														* (1d / 2d - 1d / 4d));
																			} else
																				if (BORDER_RIGHT_CENTER.equals(align)) {
																					loc.setLocation(
																										coordX + sizeX - labelWidth / 2d - borderWidth / 2d,
																										centerY - (labelHeight / 2.0d));
																				} else
																					if (BORDER_RIGHT_BOTTOM.equals(align)) {
																						loc.setLocation(
																											coordX + sizeX - labelWidth / 2d - borderWidth / 2d,
																											centerY - (labelHeight / 2.0d) + sizeY
																																* (1d / 2d - 1d / 4d));
																					} else
																						if (ABOVERIGHT.equals(align)) {
																							loc.setLocation(coordX + sizeX + LABEL_DISTANCE, coordY
																												- LABEL_DISTANCE - labelHeight);
																						} else
																							if (RIGHT.equals(align)) {
																								loc.setLocation(coordX + sizeX + LABEL_DISTANCE, centerY
																													- (labelHeight / 2.0d));
																							} else
																								if (INSIDETOP.equals(align)) {
																									loc.setLocation(coordX + centerX - (labelWidth / 2.0d), coordY
																														+ LABEL_DISTANCE + borderWidth);
																								} else {
																									// no
																									// supported
																									// alignment
																									// constant:
																									// use
																									// relative
																									// positions
																									try {
																										loc.setLocation(centerX + ((posAttr.getRelHor() * sizeX) / 2d)
																															+ ((posAttr.getLocalAlign() - 1d) * (labelWidth / 2d)),
																															(centerY + ((posAttr.getRelVert() * sizeY) / 2d))
																																				- (labelHeight / 2d));
																									} catch (Exception err) {
																										loc.setLocation(centerX - (labelWidth / 2.0d), centerY
																															- (labelHeight / 2.0d));
																									}
																								}
	}

	private static String processAutoAlignment(String alignment, Attributable a) {
		if (alignment.equals(AUTO_OUTSIDE)) {
			if (a != null && a instanceof Node) {
				Node n = (Node) a;
				return getBestAutoOutsideSetting(n);
			} else
				return alignment;
		} else
			return alignment;
	}

	public static String getBestAutoOutsideSetting(Node n) {
		if (n == null)
			return ABOVE;
		Quadrant lowestQuadrant = getBestQuadrant(n);
		if (lowestQuadrant == Quadrant.LEFT)
			return LEFT;
		if (lowestQuadrant == Quadrant.RIGHT)
			return RIGHT;
		if (lowestQuadrant == Quadrant.TOP)
			return ABOVE;
		if (lowestQuadrant == Quadrant.BOTTOM)
			return BELOW;
		return BELOW;
	}

	private static Quadrant getBestQuadrant(Node n) {
		int isBadTop = 0;
		int isBadLeft = 0;
		int isBadRight = 0;
		int isBadBottom = 0;
		for (Node neighbour : n.getNeighbors()) {
			if (getQuadrant(neighbour, n) == Quadrant.TOP)
				isBadTop += 1;
			else
				if (getQuadrant(neighbour, n) == Quadrant.LEFT)
					isBadLeft += 1;
				else
					if (getQuadrant(neighbour, n) == Quadrant.RIGHT)
						isBadRight += 1;
					else
						if (getQuadrant(neighbour, n) == Quadrant.BOTTOM)
							isBadBottom += 1;
		}
		Quadrant lowestQuadrant;

		if (isBadTop <= isBadLeft && isBadTop <= isBadRight
							&& isBadTop <= isBadBottom)
			lowestQuadrant = Quadrant.TOP;
		else
			if (isBadLeft <= isBadTop && isBadLeft < isBadRight
								&& isBadTop <= isBadBottom)
				lowestQuadrant = Quadrant.LEFT;
			else
				if (isBadRight <= isBadTop && isBadRight <= isBadLeft
									&& isBadTop <= isBadBottom)
					lowestQuadrant = Quadrant.RIGHT;
				else
					if (isBadBottom <= isBadTop && isBadTop <= isBadLeft
										&& isBadTop <= isBadRight)
						lowestQuadrant = Quadrant.BOTTOM;
					else
						lowestQuadrant = Quadrant.BOTTOM;
		return lowestQuadrant;
	}

	private static Quadrant getQuadrant(Node neighbour, Node refNode) {
		Point2D pRef = AttributeHelper.getPosition(refNode);
		Point2D pNei = AttributeHelper.getPosition(neighbour);
		if (pRef.getX() < pNei.getX()) {
			if (Math.abs(pNei.getY() - pRef.getY()) <= Math.abs(pNei.getX()
								- pRef.getX()))
				return Quadrant.RIGHT;
			else
				if (pNei.getY() >= pRef.getY())
					return Quadrant.BOTTOM;
				else
					return Quadrant.TOP;
		} else {
			if (Math.abs(pNei.getY() - pRef.getY()) <= Math.abs(pNei.getX()
								- pRef.getX()))
				return Quadrant.LEFT;
			else
				if (pNei.getY() >= pRef.getY())
					return Quadrant.BOTTOM;
				else
					return Quadrant.TOP;
		}
	}

	public static void updatePositionForEdgeMapping(Attribute aaa,
						GraphElementShape geShape, String attributeName,
						String attributeFolder, double flatness, int width, int height,
						Point loc) {
		String positionAttributePathAndName = attributeFolder
							+ Attribute.SEPARATOR + attributeName;

		EdgeLabelPositionAttribute posAttr = null;
		try {
			CollectionAttribute ca = (CollectionAttribute) aaa
								.getAttributable().getAttribute(
													positionAttributePathAndName);
			if (ca != null && (ca instanceof EdgeLabelPositionAttribute)) {
				posAttr = (EdgeLabelPositionAttribute) ca;
			} else {
				posAttr = new EdgeLabelPositionAttribute(attributeName);
				posAttr.setCollection(ca.getCollection());
				CollectionAttribute parent = ca.getParent();
				parent.remove(ca);
				parent.add(posAttr, false);
			}
		} catch (Exception e) {
			// ErrorMsg.addErrorMessage(e);
		}
		if (posAttr == null) {
			posAttr = new EdgeLabelPositionAttribute(attributeName);
			try {
				aaa.getAttributable().addAttribute(posAttr, attributeFolder);
			} catch (AttributeNotFoundException err) {
				return;
			}
		}

		// Edge edge = (Edge) ge;
		// EdgeGraphicAttribute edgeAttr = (EdgeGraphicAttribute)
		// edge.getAttribute(GRAPHICS);
		Point2D labelLoc = null;

		if (posAttr.getAlignSegment() <= 0) {
			// calc pos rel to whole edge
			PathIterator pi = geShape.getPathIterator(null, flatness);
			double dist = (iterateTill(pi, null)).getX();

			pi = geShape.getPathIterator(null, flatness);
			labelLoc = iterateTill(pi, new Double(posAttr.getRelAlign() * dist));
		} else {
			// calc pos rel to spec seg
			PathIterator pi = geShape.getPathIterator(null);

			// fst = sum of length of first (alignSegment-1) segments
			// snd = (length of segment number alignSegment)
			PointPair segPos = calculateSegPos(pi, posAttr.getAlignSegment());

			if (segPos == null) {
				pi = geShape.getPathIterator(null, flatness);

				double dist = (iterateTill(pi, null)).getX();

				pi = geShape.getPathIterator(null, flatness);
				labelLoc = iterateTill(pi, new Double(posAttr.getRelAlign()
									* dist));
			} else {
				pi = geShape.getPathIterator(null, flatness);

				Pair dists = calculateDists(pi, segPos.getFst(), segPos
									.getSnd());

				// move along path till correct pos
				pi = geShape.getPathIterator(null, flatness);
				labelLoc = iterateTill(pi, new Double((Double) dists.getFst()
									+ (posAttr.getRelAlign() * (Double) dists.getSnd())));
			}
		}
		loc.setLocation(labelLoc.getX() - (width / 2.0d) + posAttr.getAbsHor(),
							labelLoc.getY() - (height / 2.0d) + posAttr.getAbsVert());
	}

	/**
	 * Calculates a pair of two values: fst = sum of length of first (seg-1)
	 * segments snd = length of segment number seg
	 * 
	 * @param pi
	 *           DOCUMENT ME!
	 * @param segStartPos
	 *           DOCUMENT ME!
	 * @param segEndPos
	 *           DOCUMENT ME!
	 * @return DOCUMENT ME!
	 */
	protected static Pair calculateDists(PathIterator pi, Point2D segStartPos,
						Point2D segEndPos) {
		double[] seg = new double[6];

		double dist = 0;

		// fst
		double firstdist = 0;

		// snd
		double segnrdist = 0;
		double diffdist = 0;
		double lastx = 0;
		double lasty = 0;
		double newx = 0;
		double newy = 0;
		int type;
		int segcnt = 0;
		boolean haveFound = false;
		boolean foundStart = false;

		try {
			type = pi.currentSegment(seg);
			lastx = seg[0];
			lasty = seg[1];

			while (!pi.isDone() && !haveFound) {
				segcnt++;
				pi.next();
				type = pi.currentSegment(seg);

				switch (type) {
					case java.awt.geom.PathIterator.SEG_MOVETO:

						if (!pi.isDone()) {
							diffdist = Point2D.distance(lastx, lasty, seg[0],
												seg[1]);
							firstdist = dist;
							dist += diffdist;
							newx = seg[0];
							newy = seg[1];

							if (!foundStart
												&& ((lastx - segStartPos.getX()) <= Double.MIN_VALUE)
												&& ((lasty - segStartPos.getY()) <= Double.MIN_VALUE)) {
								foundStart = true;
								segnrdist = 0;
							} else
								if (foundStart) {
									segnrdist += diffdist;
								}

							if (((newx - segEndPos.getX()) <= Double.MIN_VALUE)
												&& ((newy - segEndPos.getY()) <= Double.MIN_VALUE)) {
								haveFound = true;
							}
						}

						break;

					case java.awt.geom.PathIterator.SEG_LINETO:
						newx = seg[0];
						newy = seg[1];
						diffdist = Point2D.distance(lastx, lasty, newx, newy);

						// System.out.println("diffStart = (" +
						// (lastx-segStartPos.getX()) + ", " +
						// (lasty-segStartPos.getY()) + ")");
						if (!foundStart
											&& (Math.abs(lastx - segStartPos.getX()) <= Double.MIN_VALUE)
											&& (Math.abs(lasty - segStartPos.getY()) <= Double.MIN_VALUE)) {
							foundStart = true;

							// System.out.println("found start");
							firstdist = dist;
							segnrdist = 0;
						}

						dist += diffdist;

						if (foundStart) {
							segnrdist += diffdist;
						}

						// System.out.println("diffEnd = (" +
						// (newx-segStartPos.getX()) + ", " +
						// (newy-segStartPos.getY()) + ")");
						if ((Math.abs(newx - segEndPos.getX()) <= Double.MIN_VALUE)
											&& (Math.abs(newy - segEndPos.getY()) <= Double.MIN_VALUE)) {
							// assert !foundStart :
							haveFound = true;

							// System.out.println("found end");
						}

						break;
				}

				lastx = newx;
				lasty = newy;
			}
		} catch (java.util.NoSuchElementException e) {
		}

		// System.out.println("returning " + firstdist + "    " + segnrdist);
		return new Pair(firstdist, segnrdist);
	}

	protected static PointPair calculateSegPos(PathIterator pi, int segnr) {
		// assert segnr>=0 :
		double[] seg = new double[6];

		// double firstx = 0;
		// double firsty = 0;
		double lastx = 0;
		double lasty = 0;

		double newx = 0;
		double newy = 0;

		int type;
		int segcnt = 0;

		PointPair pp = new PointPair();

		try {
			type = pi.currentSegment(seg);
			lastx = seg[0];
			lasty = seg[1];

			// firstx = lastx;
			// firsty = lasty;
			while (!pi.isDone() && (segcnt < segnr)) {
				segcnt++;
				pi.next();
				type = pi.currentSegment(seg);

				switch (type) {
					case java.awt.geom.PathIterator.SEG_MOVETO:

						if (!pi.isDone()) {
							newx = seg[0];
							newy = seg[1];
						} else {
							return null;
						}

						break;

					case java.awt.geom.PathIterator.SEG_LINETO:
						newx = seg[0];
						newy = seg[1];

						break;

					case java.awt.geom.PathIterator.SEG_QUADTO:
						newx = seg[2];
						newy = seg[3];

						break;

					case java.awt.geom.PathIterator.SEG_CUBICTO:
						newx = seg[4];
						newy = seg[5];

						break;
				}

				// System.out.println("now found segnr=" + segcnt + "; at (" +
				// newx + ", " + newy + ")");
				if (segcnt == segnr) {
					// System.out.println("found segstartpospoint at (" + lastx
					// + ", " + lasty + ")");
					pp.setFst(new Point2D.Double(lastx, lasty));

					// System.out.println("found segendpospoint at (" + newx +
					// ", " + newy + ")");
					pp.setSnd(new Point2D.Double(newx, newy));
				}

				lastx = newx;
				lasty = newy;
			}
		} catch (java.util.NoSuchElementException e) {
			ErrorMsg.addErrorMessage(e);
		}

		// pp.setSnd(new Point2D.Double(lastx, lasty));
		if (segcnt < segnr) {
			// System.out.println("segnr out of bounds");
			pp = null;
		}

		return pp;
	}

	/**
	 * If d == null then calculates length of path given by pi if d is a value
	 * then calculates a position on the path near the distance given by this
	 * parameter, measured from the start.
	 * 
	 * @param pi
	 *           <code>PathIterator</code> describing the path
	 * @param d
	 *           null or distance
	 * @return distance at first component of <code>Point2D</code> or the
	 *         position wanted as <code>point2D</code> .
	 */
	protected static Point2D iterateTill(PathIterator pi, Double d) {
		double[] seg = new double[6];
		double limitDist;

		if (d == null) {
			limitDist = Double.POSITIVE_INFINITY;
		} else {
			limitDist = d.doubleValue();
		}

		double dist = 0;
		double lastx = 0;
		double lasty = 0;
		int type;

		try {
			type = pi.currentSegment(seg);
			lastx = seg[0];
			lasty = seg[1];

			while (!pi.isDone() && (dist < limitDist)) {
				pi.next();
				type = pi.currentSegment(seg);

				switch (type) {
					case java.awt.geom.PathIterator.SEG_MOVETO:

						if (!pi.isDone()) {
							dist += Point2D.distance(lastx, lasty, seg[0],
												seg[1]);
							lastx = seg[0];
							lasty = seg[1];
						}

						break;

					case java.awt.geom.PathIterator.SEG_LINETO:
						dist += Point2D.distance(lastx, lasty, seg[0], seg[1]);

						if ((d != null) && (dist >= limitDist)) {
							// System.out.println(dist +"    "+ limitDist);
							double diffx = seg[0] - lastx;
							double diffy = seg[1] - lasty;
							double diffsqr = Math.sqrt((diffx * diffx)
												+ (diffy * diffy));

							// System.out.println("diffx
							// lastx += Math.sqrt(diffsq - diffy*diffy);
							// lasty += Math.sqrt(diffsq - diffx*diffx);
							double factor = (diffsqr - dist + limitDist)
												/ diffsqr;
							lastx += (diffx * factor);
							lasty += (diffy * factor);
						} else {
							lastx = seg[0];
							lasty = seg[1];
						}

						break;

					case java.awt.geom.PathIterator.SEG_QUADTO:

						// unnecessary since this approximation uses only lines
						// System.out.println(" quad");
						dist += Point2D.distance(lastx, lasty, seg[2], seg[3]);
						lastx = seg[2];
						lasty = seg[3];

						break;

					case java.awt.geom.PathIterator.SEG_CUBICTO:

						// unnecessary since this approximation uses only lines
						// System.out.println(" cube");
						dist += Point2D.distance(lastx, lasty, seg[4], seg[5]);
						lastx = seg[4];
						lasty = seg[5];

						break;
				}
			}
		} catch (java.util.NoSuchElementException e) {
		}

		if (d == null) {
			return new Point2D.Double(dist, 0);
		} else {
			return new Point2D.Double(lastx, lasty);
		}
	}
}
