/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute;

import org.graffiti.attributes.StringAttribute;

import vanted.analysis.Reachability;

// TODO: Auto-generated Javadoc
/**
 * The Class ReachabilityAttribute handles the reachability attribute.
 *
 * @author Kevin Pucknat
 */
public class ReachabilityAttribute extends StringAttribute {
	
	/** The state. */
	private Reachability state;
	
	/** The Constant path. */
	public static final String path = "petrinet";
	
	/** The Constant name. */
	public static final String name = "reachability_Token";
	
	/** The Constant seperator. */
	public static final String seperator = "@";
	
	/**
	 * Instantiates a new reachability attribute.
	 */
	public ReachabilityAttribute() {
		super();
	}
	
	/**
	 * Instantiates a new reachability attribute.
	 *
	 * @param id the id
	 */
	public ReachabilityAttribute(String id) {
		super(id);
	}
	
	/**
	 * Instantiates a new reachability attribute.
	 *
	 * @param id the id
	 * @param value the value
	 */
	public ReachabilityAttribute(String id, String value) {
		super(id, value);
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#doSetValue(java.lang.Object)
	 */
	@Override
	protected void doSetValue(Object o) throws IllegalArgumentException {
		assert o != null;
		
		try {
			doSetValue((String) o);
		} catch (ClassCastException cce) {
			throw new IllegalArgumentException("Invalid value type.");
		}
	}
	
	/**
	 * Sets the reachability value.
	 *
	 * @param value the value
	 * @throws IllegalArgumentException the illegal argument exception
	 */
	protected void doSetValue(String value) throws IllegalArgumentException {
		setString(value);
		state=new Reachability(value);
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getString()
	 */
	@Override
	public String getString() {		
		String str = super.getString();
		
		if(str==null)
			return "";
			
		return str;
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getValue()
	 */
	@Override
	public Object getValue() {
		return getString();
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#setString(java.lang.String)
	 */
	@Override
	public void setString(String value) {
		super.setString(value);
	}
	

	/**
	 * Gets the state.
	 *
	 * @return the state
	 */
	public Reachability getState() {
		return state;
	}
	
	/**
	 * Adds the state.
	 *
	 * @param state the state
	 */
	public void addState(Reachability state) {
		this.state=state;	
	}
			
	/**
	 * Removes the state.
	 *
	 * @return true, if successful
	 */
	public boolean removeState() {
		setString("");
		state=null;
		return true;
	}	
}
