/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute.cpn.expression;

import info.clearthought.layout.TableLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.graffiti.plugin.Displayable;
import org.graffiti.plugin.editcomponent.AbstractValueEditComponent;

import vanted.petrinetelements.cpn.Expression;

// TODO: Auto-generated Javadoc
/**
 * The Class ExpressionArcAttributeEditor a way to change the arc-expression in the VANTED gui.
 * 
 * @author Kevin Pucknat
 */
public class ExpressionArcAttributeEditor extends AbstractValueEditComponent {
	
	/** The spinner list. */
	private ArrayList<JComponent> spinnerList;
	
	/** The expression lst. */
	private ArrayList<Expression> expressionLst;
	
	/** The change list. */
	private HashMap<Expression, Object> changeList;
	
	/** The delete list. */
	private ArrayList<Expression> deleteList;
	
	/** The btn lst. */
	private ArrayList<JButton> btnLst;
	
	/** The no expression. */
	private final JLabel noExpression = new JLabel("No expression set!");
	
	/** The jp. */
	private final JPanel jp;
	
	/**
	 * Instantiates a new arc-expression attribute editor.
	 *
	 * @param disp the disp
	 */
	public ExpressionArcAttributeEditor(Displayable disp) {
		super(disp);		
		jp = new JPanel();
		jp.setOpaque(false);
		jp.setLayout(TableLayout.getLayout(TableLayout.PREFERRED, TableLayout.PREFERRED));
		jp.add(noExpression, "0,0");
	}	
		
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.editcomponent.ValueEditComponent#getComponent()
	 */
	public JComponent getComponent() {
		return jp;
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.editcomponent.ValueEditComponent#setEditFieldValue()
	 */
	public void setEditFieldValue() {
		jp.removeAll();
		
		JPanel left = new JPanel();
		left.setOpaque(false);
		JPanel right = new JPanel();
		right.setOpaque(false);
		
		
		ExpressionAttribute ea = (ExpressionAttribute) getDisplayable();
		if (ea.getExpressions().size() <= 0) {
			jp.add(noExpression, "0,0");
		} else {
			spinnerList = new ArrayList<JComponent>();	
			expressionLst = new ArrayList<Expression>();
			changeList = new HashMap<Expression, Object>();
			deleteList = new ArrayList<Expression>();
			btnLst = new ArrayList<JButton>();
			
			int i=0;
			for (Expression exp : ea.getExpressions()) {
				JSpinner js = null;
				
				js = getSpinnerInteger(Integer.parseInt(exp.getValue().toString()));		
				
				expressionLst.add(exp);
				js.setName(""+i);
				js.addChangeListener(new ChangeListener() {					
					@Override
					public void stateChanged(ChangeEvent e) {									
						Expression expression = expressionLst.get(Integer.parseInt(((JSpinner)e.getSource()).getName()));						
						changeList.put(expression, ((JSpinner)e.getSource()).getValue());	
					}
				});
				spinnerList.add(js);
				
				JButton btn = new JButton("remove");
				btn.addActionListener(new ActionListener() {					
					@Override
					public void actionPerformed(ActionEvent e) {	
						int i = Integer.parseInt(((JButton)e.getSource()).getName());
						Expression expression = expressionLst.get(i);
						if(!deleteList.contains(expression)){
							deleteList.add(expression);
							spinnerList.get(i).setEnabled(false);
						}else{
							deleteList.remove(expression);
							spinnerList.get(i).setEnabled(true);
						}
					}
				});
				btnLst.add(btn);
				btn.setName(""+i);				
				i++;
			}
						
			left.add(TableLayout.getMultiSplitVertical(spinnerList, 100, TableLayout.MAX_ALIGN), "0,0");
			right.add(TableLayout.getMultiSplitVertical(btnLst, TableLayout.PREFERRED, TableLayout.MIN_ALIGN), "0,0");
			jp.add(TableLayout.getSplit(left, right, TableLayout.PREFERRED, TableLayout.PREFERRED), "0,0");

		}	
	}
	
	/**
	 * Gets the spinner integer.
	 *
	 * @param value the value
	 * @return the spinner integer 
	 */
	private JSpinner getSpinnerInteger(int value){		
		return new JSpinner(new SpinnerNumberModel(value, 1, Integer.MAX_VALUE, 1)); 
	}
			
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.editcomponent.ValueEditComponent#setValue()
	 */
	public void setValue() {
		if(deleteList==null || changeList==null)return;
		
		if (getDisplayable() instanceof ExpressionAttribute) {			
			for(Expression expression : deleteList){
				((ExpressionAttribute) getDisplayable()).removeExpression(expression);
				if(changeList.containsKey(expression))
					changeList.remove(expression);
			}			
			
			Iterator<Expression> it = changeList.keySet().iterator();
			while(it.hasNext()){
				Expression expression = it.next();
				((ExpressionAttribute) getDisplayable()).changeExpressionValue(expression, changeList.get(expression));
			}
		}
	}
}
