/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute.cpn.expression;

import java.util.ArrayList;
import java.util.List;

import org.graffiti.attributes.StringAttribute;

import vanted.petrinetelements.cpn.Expression;

// TODO: Auto-generated Javadoc
/**
 * The Class ExpressionAttribute for expression attribute handling.
 * 
 * @author Kevin Pucknat
 */
public class ExpressionAttribute extends StringAttribute{
	
	/** The Constant path. */
	public static final String path = "petrinet";
	
	/** The Constant name. */
	public static final String name = "expressionArc";
	
	/** The Constant seperator. */
	public static final String seperator = "@";
	
	/** The expressions. */
	private List<Expression> expressions = new ArrayList<Expression>();
	
	/**
	 * Instantiates a new expression attribute.
	 */
	public ExpressionAttribute() {
		super();
	}
	
	/**
	 * Instantiates a new expression attribute.
	 *
	 * @param id the id
	 */
	public ExpressionAttribute(String id) {
		super(id);
	}
	
	/**
	 * Instantiates a new expression attribute.
	 *
	 * @param id the id
	 * @param value the value
	 */
	public ExpressionAttribute(String id, String value) {
		super(id, value);
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#doSetValue(java.lang.Object)
	 */
	@Override
	protected void doSetValue(Object o) throws IllegalArgumentException {
		assert o != null;
		
		try {
			doSetValue((String) o);
		} catch (ClassCastException cce) {
			throw new IllegalArgumentException("Invalid value type.");
		}
	}
	
	/**
	 * Sets expression values.
	 *
	 * @param value the value
	 * @throws IllegalArgumentException the illegal argument exception
	 */
	protected void doSetValue(String value) throws IllegalArgumentException {
		setString(value);		
		for (String expression : value.split(seperator))
			if (expression != null && expression.length() > 0)
				expressions.add(new Expression(expression));			
			
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getString()
	 */
	@Override
	public String getString() {		
		String str = super.getString();
		
		if(str==null)
			return "";
			
		return str;
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getValue()
	 */
	@Override
	public Object getValue() {
		return getString();
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#setString(java.lang.String)
	 */
	@Override
	public void setString(String value) {
		super.setString(value);
	}
	
	/**
	 * Gets the expressions.
	 *
	 * @return the expressions
	 */
	public List<Expression> getExpressions() {
		return expressions;
	}
	
	/**
	 * Adds the expression.
	 *
	 * @param expression the expression
	 * @return true, if successful
	 */
	public boolean addExpression(Expression expression) {		
		if(getString().equals(""))
			setString("stat='"+expression.getStatement().toString()+"' value='"+expression.getValue()+"'");
		else
			setString(getString()+seperator+"stat='"+expression.getStatement().toString()+"' value='"+expression.getValue()+"'");

		
		return expressions.add(expression);
	}
	
	/**
	 * Removes the expression.
	 *
	 * @param expression the expression
	 * @return true, if successful
	 */
	public boolean removeExpression(Expression expression) {
		if(getString().contains("stat='"+expression.getStatement().toString()+"' value='"+expression.getValue()+"'")){			
			setString(getString().replaceFirst("stat='"+expression.getStatement().toString()+"' value='"+expression.getValue()+"'", ""));
			if(getString().contains(seperator+seperator))
				setString(getString().replaceFirst(seperator+seperator, seperator));
			if(getString().startsWith(seperator))
				setString(getString().replaceFirst(seperator, ""));
			return expressions.remove(expression);
		}
		
		return false;
	}
	
	/**
	 * Removes all expressions.
	 */
	public void removeAllExpressions() {
		setString("");
		expressions.clear();
	}
	
	/**
	 * Changes the expression value.
	 *
	 * @param expression the expression
	 * @param value the value
	 * @return true, if successful
	 */
	public boolean changeExpressionValue(Expression expression, Object value){
		if(expressions.contains(expression)){
			removeExpression(expression);	
			return addExpression(new Expression(expression.getStatement(), value));			
		}		
		return false;
	}
}
