/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute.cpn.expression;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

import org.graffiti.attributes.Attributable;
import org.graffiti.attributes.Attribute;
import org.graffiti.graph.Edge;
import org.graffiti.graph.Node;
import org.graffiti.plugin.attributecomponent.AbstractAttributeComponent;
import org.graffiti.plugin.view.ShapeNotFoundException;

import vanted.petrinetelements.cpn.Expression;

// TODO: Auto-generated Javadoc
/**
 * The Class ExpressionAttributeComponent visualizes the expressions.
 *
 * @author Kevin Pucknat
 */
public class ExpressionAttributeComponent extends AbstractAttributeComponent {
	
	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;
	
	/** The point. */
	private Point point = new Point();
	
	/** The exp size. */
	private int expSize = 20;	
	
	/** The expressions. */
	private List<Expression> expressions;

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#attributeChanged(org.graffiti.attributes.Attribute)
	 */
	@Override
	public void attributeChanged(Attribute attr) throws ShapeNotFoundException {
		if (attr.getParent().getName().equals(attr.getParent().getName()))
			recreate();
	}


	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#recreate()
	 */
	@Override
	public void recreate() throws ShapeNotFoundException {
		expressions = null;
		if(attr.getValue()!=null&&!attr.getValue().toString().equals("")){
			expressions = getExpressionsFromString(attr.getValue().toString());				
			setSize(expSize * expressions.size() , expSize );
		}
		validate();
		repaint();
	}

	/**
	 * Update the expression position.
	 *
	 * @param attributable the attributable
	 */
	protected void updatePosition(Attributable attributable) {
		if (attributable instanceof Node) {		
			setLocation((int) (point.getX() + shift.getX()), (int) (point.getY() + shift.getY()));

		}
		if (attributable instanceof Edge) {
			setLocation((int) (point.getX() + shift.getX()), -1 + (int) (point.getY() + shift.getY()));
		}
	}

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#setShift(java.awt.Point)
	 */
	@Override
	public void setShift(Point shift) {
		this.shift = shift;
		if (attr != null && attr.getAttributable() != null)
			updatePosition(attr.getAttributable());
	}

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.view.AttributeComponent#adjustComponentSize()
	 */
	@Override
	public void adjustComponentSize() {
		if (attr != null && attr.getAttributable() != null)
			updatePosition(attr.getAttributable());
	}

	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g) {
		super.paintComponent(g);
				
		if(attr.getValue()!=null&&!attr.getValue().toString().equals(""))
			expressions = getExpressionsFromString(attr.getValue().toString());
		
		g.setColor(Color.BLUE);
		g.setFont(new Font("arial", Font.BOLD, expSize));
		if(expressions!=null && expressions.size()>0)
			if(Double.parseDouble(expressions.get(0).getValue().toString())>1.0 ){
				g.drawString(expressions.get(0).getValue()+"", 7, 23);			
			}
	}

	/**
	 * Gets the expressions from a String.
	 *
	 * @param expressionStr the expression str
	 * @return the expressions from string
	 */
	private List<Expression> getExpressionsFromString(String expressionStr){
		List<Expression> expessions = new ArrayList<Expression>();
		
		for (String expression : expressionStr.split(ExpressionAttribute.seperator)){
			expessions.add(new Expression(expression));
		}
		
		return expessions;
	}	
}
