/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute.cpn.guard;

import org.graffiti.graph.Edge;
import org.graffiti.plugin.algorithm.AbstractEditorAlgorithm;
import org.graffiti.plugin.algorithm.PreconditionException;
import org.graffiti.plugin.parameter.IntegerParameter;
import org.graffiti.plugin.parameter.Parameter;
import org.graffiti.plugin.view.View;

import petri_jfern.cpn.StatementGuard;
import vanted.petrinetelements.AbstractArc;
import vanted.petrinetelements.InArc;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.cpn.Guard;

// TODO: Auto-generated Javadoc
/**
 * The Class AddGuardsToArc another way to add arc-guards to input arcs in the menu bar of VANTED.
 * 
 * @author Kevin Pucknat
 */
public class AddGuardsToArc extends AbstractEditorAlgorithm {
	
	/** The count. */
	private int count;
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.algorithm.EditorAlgorithm#activeForView(org.graffiti.plugin.view.View)
	 */
	public boolean activeForView(View v) {
		return v != null;
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.algorithm.Algorithm#execute()
	 */
	public void execute() {	
		PetriNet pn = new PetriNet();
		if(!pn.createPetriNet(graph, true))return;
		for (Edge edge : selection.getEdges()){			
			for(AbstractArc arc : pn.getAllArcs()){
				if(arc instanceof InArc){
					if(arc.getEdge().equals(edge)){
						InArc inarc = (InArc)arc;
						for (int i = 0; i < count; i++) {	
							inarc.addGuard(new Guard(StatementGuard.ELEMENT_COUNT, ">=1"));			
						}
						graph.setModified(true);
					}
				}
			}			
		}		
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.algorithm.AbstractAlgorithm#getParameters()
	 */
	@Override
	public Parameter[] getParameters() {
		return new Parameter[] { new IntegerParameter(1, "How many?", null) };
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.algorithm.AbstractAlgorithm#setParameters(org.graffiti.plugin.parameter.Parameter[])
	 */
	@Override
	public void setParameters(Parameter[] params) {
		int idx = 0;
		count = (Integer) params[idx++].getValue();
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.algorithm.Algorithm#getName()
	 */
	public String getName() {
		return "Add Arc Guards";
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.algorithm.AbstractAlgorithm#check()
	 */
	@Override
	public void check() throws PreconditionException {
		super.check();
		if (selection == null || selection.getElements().size() < 1)
			throw new PreconditionException(
								"Please select at least one place");
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.algorithm.AbstractAlgorithm#getCategory()
	 */
	@Override
	public String getCategory() {
		return "Petri-Net";
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.algorithm.AbstractAlgorithm#getDescription()
	 */
	@Override
	public String getDescription() {
		return "<html>" + "Adds a guard to selected arc.<br>"
							+ "Guards can be switched on and off using<br>"
							+ "the Network->Edge and Network->Edge tabs.";
	}
}
