/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute.cpn.guard;

import java.util.ArrayList;
import java.util.List;

import org.graffiti.attributes.StringAttribute;

import vanted.petrinetelements.cpn.Guard;

// TODO: Auto-generated Javadoc
/**
 * The Class GuardArcAttribute for arc-guard handling.
 * 
 * @author Kevin Pucknat
 */
public class GuardArcAttribute extends StringAttribute{
	
	/** The Constant path. */
	public static final String path = "petrinet";
	
	/** The Constant name. */
	public static final String name = "guardArc";
	
	/** The Constant seperator. */
	public static final String seperator = "@";
	
	/** The guards. */
	private List<Guard> guards = new ArrayList<Guard>();
	
	/**
	 * Instantiates a new arc-guard attribute.
	 */
	public GuardArcAttribute() {
		super();
	}
	
	/**
	 * Instantiates a new arc-guard attribute.
	 *
	 * @param id the id
	 */
	public GuardArcAttribute(String id) {
		super(id);
	}
	
	/**
	 * Instantiates a new arc-guard attribute.
	 *
	 * @param id the id
	 * @param value the value
	 */
	public GuardArcAttribute(String id, String value) {
		super(id, value);
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#doSetValue(java.lang.Object)
	 */
	@Override
	protected void doSetValue(Object o) throws IllegalArgumentException {
		assert o != null;
		
		try {
			doSetValue((String) o);
		} catch (ClassCastException cce) {
			throw new IllegalArgumentException("Invalid value type.");
		}
	}
	
	/**
	 * Sets the arc-guard value.
	 *
	 * @param value the value
	 * @throws IllegalArgumentException the illegal argument exception
	 */
	protected void doSetValue(String value) throws IllegalArgumentException {
		setString(value);
		for (String guard : value.split(seperator))
			if (guard != null && guard.length() > 0)
				guards.add(new Guard(guard));			
			
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getString()
	 */
	@Override
	public String getString() {		
		String str = super.getString();
		
		if(str==null)
			return "";
			
		return str;
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getValue()
	 */
	@Override
	public Object getValue() {
		return getString();
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#setString(java.lang.String)
	 */
	@Override
	public void setString(String value) {
		super.setString(value);
	}
	
	/**
	 * Gets the guards.
	 *
	 * @return the guards
	 */
	public List<Guard> getGuards() {
		return guards;
	}
	
	/**
	 * Adds the guard.
	 *
	 * @param guard the guard
	 * @return true, if successful
	 */
	public boolean addGuard(Guard guard) {		
		if(getString().equals("")){
			setString("stat='"+guard.getStatement().toString()+"' value='"+guard.getValue()+"'");
		}else{
			setString(getString()+seperator+"stat='"+guard.getStatement().toString()+"' value='"+guard.getValue()+"'");
		}
		
		return guards.add(guard);
	}
	
	/**
	 * Removes the guard.
	 *
	 * @param guard the guard
	 * @return true, if successful
	 */
	public boolean removeGuard(Guard guard) {
		if(getString().contains("stat='"+guard.getStatement().toString()+"' value='"+guard.getValue()+"'")){			
			setString(getString().replaceFirst("stat='"+guard.getStatement().toString()+"' value='"+guard.getValue()+"'", ""));
			if(getString().contains(seperator+seperator))
				setString(getString().replaceFirst(seperator+seperator, seperator));
			if(getString().startsWith(seperator))
				setString(getString().replaceFirst(seperator, ""));
			return guards.remove(guard);
		}
		
		return false;
	}
	
	/**
	 * Removes all guards.
	 */
	public void removeAllGuards() {
		setString("");
		guards.clear();
	}
	
	/**
	 * Changes the guard value.
	 *
	 * @param guard the guard
	 * @param value the value
	 * @return true, if successful
	 */
	public boolean changeGuardValue(Guard guard, Object value){
		if(guards.contains(guard)){
			removeGuard(guard);	
			return addGuard(new Guard(guard.getStatement(), value.toString()));			
		}		
		return false;
	}
}
