/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute.cpn.guard;

import info.clearthought.layout.TableLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.graffiti.plugin.Displayable;
import org.graffiti.plugin.editcomponent.AbstractValueEditComponent;

import vanted.petrinetelements.cpn.Guard;

// TODO: Auto-generated Javadoc
/**
 * The Class GuardArcAttributeEditor a way to change the arc-guard in the VANTED gui.
 * 
 * @author Kevin Pucknat
 */
public class GuardArcAttributeEditor extends AbstractValueEditComponent {
	
	/** The txt list. */
	private ArrayList<JComponent> txtList;
	
	/** The guard lst. */
	private ArrayList<Guard> guardLst;
	
	/** The change list. */
	private HashMap<Guard, Object> changeList;
	
	/** The delete list. */
	private ArrayList<Guard> deleteList;
	
	/** The btn lst. */
	private ArrayList<JButton> btnLst;
	
	/** The no guard. */
	private final JLabel noGuard = new JLabel("No guard set!");
	
	/** The jp. */
	private final JPanel jp;
	
	/**
	 * Instantiates a new guard arc attribute editor.
	 *
	 * @param disp the disp
	 */
	public GuardArcAttributeEditor(Displayable disp) {
		super(disp);		
		jp = new JPanel();
		jp.setOpaque(false);
		jp.setLayout(TableLayout.getLayout(TableLayout.PREFERRED, TableLayout.PREFERRED));
		jp.add(noGuard, "0,0");
	}	
		
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.editcomponent.ValueEditComponent#getComponent()
	 */
	public JComponent getComponent() {
		return jp;
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.editcomponent.ValueEditComponent#setEditFieldValue()
	 */
	public void setEditFieldValue() {
		jp.removeAll();
		
		JPanel left = new JPanel();
		left.setOpaque(false);
		JPanel right = new JPanel();
		right.setOpaque(false);
		
		
		GuardArcAttribute ta = (GuardArcAttribute) getDisplayable();
		if (ta.getGuards().size() <= 0) {
			jp.add(noGuard, "0,0");
		} else {
			txtList = new ArrayList<JComponent>();	
			guardLst = new ArrayList<Guard>();
			changeList = new HashMap<Guard, Object>();
			deleteList = new ArrayList<Guard>();
			btnLst = new ArrayList<JButton>();
			
			int i=0;
			for (Guard g : ta.getGuards()) {
				JTextField jt = null;			
				jt = new JTextField(g.getValue().toString(), 100);		
				
				guardLst.add(g);
				jt.setName(""+i);
				jt.addActionListener(new ActionListener() {
					
					@Override
					public void actionPerformed(ActionEvent e) {
						Guard guard = guardLst.get(Integer.parseInt(((JTextField)e.getSource()).getName()));						
						changeList.put(guard, ((JTextField)e.getSource()).getText());							
					}
				});
				jt.addPropertyChangeListener(new PropertyChangeListener() {					
					@Override
					public void propertyChange(PropertyChangeEvent evt) {
						Guard guard = guardLst.get(Integer.parseInt(((JTextField)evt.getSource()).getName()));						
						changeList.put(guard, ((JTextField)evt.getSource()).getText());							
					}
				});
				
				txtList.add(jt);
				
				JButton btn = new JButton("remove");
				btn.addActionListener(new ActionListener() {					
					@Override
					public void actionPerformed(ActionEvent e) {	
						int i = Integer.parseInt(((JButton)e.getSource()).getName());
						Guard guard = guardLst.get(i);
						if(!deleteList.contains(guard)){
							deleteList.add(guard);
							txtList.get(i).setEnabled(false);
						}else{
							deleteList.remove(guard);
							txtList.get(i).setEnabled(true);
						}
					}
				});
				btnLst.add(btn);
				btn.setName(""+i);				
				i++;
			}
			
			
			left.add(TableLayout.getMultiSplitVertical(txtList, 150, TableLayout.MAX_ALIGN), "0,0");
			right.add(TableLayout.getMultiSplitVertical(btnLst, TableLayout.PREFERRED, TableLayout.MIN_ALIGN), "0,0");
			jp.add(TableLayout.getSplit(left, right, TableLayout.PREFERRED, TableLayout.PREFERRED), "0,0");
		}	
	}
	
	
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.editcomponent.ValueEditComponent#setValue()
	 */
	public void setValue() {
		if(deleteList==null || changeList==null)return;
		if (getDisplayable() instanceof GuardArcAttribute) {			
			for(Guard guard : deleteList){
				((GuardArcAttribute) getDisplayable()).removeGuard(guard);
				if(changeList.containsKey(guard))
					changeList.remove(guard);
			}			
			
			Iterator<Guard> it = changeList.keySet().iterator();
			while(it.hasNext()){
				Guard guard = it.next();
				((GuardArcAttribute) getDisplayable()).changeGuardValue(guard, changeList.get(guard));
			}
		}
	}
}
