/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute.token;

import java.util.ArrayList;
import java.util.List;

import org.graffiti.attributes.StringAttribute;

import vanted.petrinetelements.AbstractToken;
import vanted.petrinetelements.TokenDiscrete;

// TODO: Auto-generated Javadoc
/**
 * The Class AbstractTokenAttribute handles the abstract token attribute.
 *
 * @author Kevin Pucknat
 */
public abstract class AbstractTokenAttribute extends StringAttribute {	
	
	/** The tokens. */
	protected final ArrayList<AbstractToken> tokens = new ArrayList<AbstractToken>();	
	
	/** The Constant path. */
	public static final String path = "petrinet";
	
	/** The Constant seperator. */
	public static final String seperator = "@";
	
	/**
	 * Instantiates a new abstract token attribute.
	 */
	public AbstractTokenAttribute() {
		super();
	}
	
	/**
	 * Instantiates a new abstract token attribute.
	 *
	 * @param id the id
	 */
	public AbstractTokenAttribute(String id) {
		super(id);
	}
	
	/**
	 * Instantiates a new abstract token attribute.
	 *
	 * @param id the id
	 * @param value the value
	 */
	public AbstractTokenAttribute(String id, String value) {
		super(id, value);
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#doSetValue(java.lang.Object)
	 */
	@Override
	protected void doSetValue(Object o) throws IllegalArgumentException {
		assert o != null;
		
		try {
			doSetValue((String) o);
		} catch (ClassCastException cce) {
			throw new IllegalArgumentException("Invalid value type.");
		}
	}
	
	/**
	 * Sets token values.
	 *
	 * @param value the value
	 * @throws IllegalArgumentException the illegal argument exception
	 */
	protected void doSetValue(String value) throws IllegalArgumentException {
		setString(value);
		for (String token : value.split(seperator))
			tokens.add(new TokenDiscrete(Integer.parseInt(token)));
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getString()
	 */
	@Override
	public String getString() {
		String str = super.getString();
		
		if (str == null)
			return "";
		
		return str;
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#getValue()
	 */
	@Override
	public Object getValue() {
		return getString();
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#setString(java.lang.String)
	 */
	@Override
	public void setString(String value) {
		super.setString(value);
	}
	
	/**
	 * Gets the tokens.
	 *
	 * @return the tokens
	 */
	public List<AbstractToken> getTokens() {
		return tokens;
	}
	
	/**
	 * Adds the token.
	 *
	 * @param token the token
	 * @return true, if successful
	 */
	public boolean addToken(AbstractToken token) {
		if (getString().equals(""))
			setString(token.getValue() + "");
		else
			setString(getString() + seperator + token.getValue());
		
		return tokens.add(token);
	}
	
	/**
	 * Removes the token.
	 *
	 * @param token the token
	 * @return true, if successful
	 */
	public boolean removeToken(AbstractToken token) {
		if (getString().contains(token.getValue().toString())) {
			setString(getString().replaceFirst(token.getValue().toString(), ""));
			if (getString().contains(seperator + seperator))
				setString(getString().replaceFirst(seperator + seperator, seperator));
			if (getString().startsWith(seperator))
				setString(getString().replaceFirst(seperator, ""));
			
			if (getString().isEmpty()) {
				removeAllTokens();
			}
			
			return tokens.remove(token);
		}
		
		return false;
	}

	/**
	 * Removes all tokens.
	 */
	public void removeAllTokens() {
		setString("");
		tokens.clear();
	}

	/* (non-Javadoc)
	 * @see org.graffiti.attributes.StringAttribute#setDefaultValue()
	 */
	@Override
	public void setDefaultValue() {		
		super.setDefaultValue();
		if(tokens!=null)
			tokens.clear();
	}	
}
