/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute.token;

import org.AttributeHelper;
import org.graffiti.graph.Node;

import vanted.petrinetelements.AbstractToken;
import vanted.petrinetelements.TokenContinuous;

// TODO: Auto-generated Javadoc
/**
 * The Class TokenAttributeContinuous handles a continuous token attribute.
 *
 * @author Kevin Pucknat
 */
public class TokenAttributeContinuous extends AbstractTokenAttribute {	
	
	/** The Constant name. */
	public static final String name = "continuous_Token";	
	
	/** The Constant decimal. */
	public static final String decimal = "#.######"; //if you change this, you must also change the method round
	
	/** The Constant positionAttributeName. */
	public static final String positionAttributeName = "tokendoubleposition";
	
	/** The Constant sizeAttributeName. */
	public static final String sizeAttributeName = "tokendoublesize";
	
	/** The Constant colorAttributeName. */
	public static final String colorAttributeName = "tokendoublecolor";
	
	/**
	 * Instantiates a new token attribute continuous.
	 */
	public TokenAttributeContinuous() {
		super();
	}
	
	/**
	 * Instantiates a new token attribute continuous.
	 *
	 * @param id the id
	 */
	public TokenAttributeContinuous(String id) {
		super(id);
	}
	
	/**
	 * Instantiates a new token attribute continuous.
	 *
	 * @param id the id
	 * @param value the value
	 */
	public TokenAttributeContinuous(String id, String value) {
		super(id, value);
	}
	
	/* (non-Javadoc)
	 * @see vanted.attribute.token.AbstractTokenAttribute#doSetValue(java.lang.String)
	 */
	protected void doSetValue(String value) throws IllegalArgumentException {
		setString(value);
		for (String token : value.split(seperator))
			tokens.add(new TokenContinuous(Double.parseDouble(token)));
	}

	/* (non-Javadoc)
	 * @see vanted.attribute.token.AbstractTokenAttribute#removeAllTokens()
	 */
	public void removeAllTokens() {
		super.removeAllTokens();
		if (getAttributable() instanceof Node) {
			Node node = (Node) getAttributable();
			AttributeHelper.deleteAttribute(node, TokenAttributeContinuous.path, TokenAttributeContinuous.name);
			AttributeHelper.deleteAttribute(node, TokenAttributeContinuous.path, TokenAttributeContinuous.positionAttributeName);
			AttributeHelper.deleteAttribute(node, TokenAttributeContinuous.path, TokenAttributeContinuous.colorAttributeName);
			AttributeHelper.deleteAttribute(node, TokenAttributeContinuous.path, TokenAttributeContinuous.sizeAttributeName);
		}
	}
	
	/**
	 * Changes token value.
	 *
	 * @param token the token
	 * @param value the value
	 * @return true, if successful
	 */
	public boolean changeTokenValue(AbstractToken token, double value) {
		if (tokens.contains(token)) {
			if (getString().contains(token.getValue().toString())) {
				setString(getString().replaceFirst(token.getValue().toString(), ""));
				if (getString().contains(seperator + seperator))
					setString(getString().replaceFirst(seperator + seperator, seperator));
				if (getString().startsWith(seperator))
					setString(getString().replaceFirst(seperator, ""));
				
				tokens.remove(token);
			}		
			
			return addToken(new TokenContinuous(value));
		}
		return false;
	}
	
	
	/**
	 * Rounds a double value.
	 *
	 * @param value the value
	 * @return the double
	 */
	public static double round(double value) {
		return Math.round(value * 1000000d) / 1000000d; // rounded to 6 decimal places
	}	
}
