/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute.token;


import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.text.DecimalFormat;
import java.text.NumberFormat;

import org.AttributeHelper;
import org.color.ColorUtil;
import org.graffiti.attributes.Attributable;
import org.graffiti.attributes.Attribute;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;
import org.graffiti.graphics.GraphicAttributeConstants;
import org.graffiti.plugin.attributecomponent.AbstractAttributeComponent;
import org.graffiti.plugin.view.ShapeNotFoundException;

import vanted.attribute.LabelComponentHelper;
import vanted.petrinetelements.PetriNet;

// TODO: Auto-generated Javadoc
/**
 * The Class TokenAttributeContinuousComponent visualizes the continuous token.
 *
 * @author Kevin Pucknat
 */
public class TokenAttributeContinuousComponent extends AbstractAttributeComponent {
	
	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** The point. */
	private Point point = new Point();
	
	/** The strlength. */
	private int strlength=10;
	
	/** The tokensize. */
	private int tokensize = 18;
	
	/** The color. */
	private Color color = Color.RED;

	/** The align. */
	private String align;

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#attributeChanged(org.graffiti.attributes.Attribute)
	 */
	@Override
	public void attributeChanged(Attribute attr) throws ShapeNotFoundException {
		if (attr.getParent().getName().equals(attr.getParent().getName()))
			recreate();
	}


	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#recreate()
	 */
	@Override
	public void recreate() throws ShapeNotFoundException {
		GraphElement ge = (GraphElement) this.attr.getAttributable();
			
		if(ge instanceof Node){
//			if(AttributeHelper.hasAttribute((Node)ge, TokenAttributeContinuous.path, TokenAttributeContinuous.sizeAttributeName))
//				tokensize = (Integer) AttributeHelper.getAttributeValue(ge, TokenAttributeContinuous.path, TokenAttributeContinuous.sizeAttributeName, 18, 18);
			tokensize = (int) (geShape.getBounds().width * 0.7);
			if(AttributeHelper.hasAttribute((Node)ge, TokenAttributeContinuous.path, TokenAttributeContinuous.sizeAttributeName))
				color = ColorUtil.getColorFromHex((String) AttributeHelper.getAttributeValue(ge, TokenAttributeContinuous.path, TokenAttributeContinuous.colorAttributeName, ColorUtil
						.getHexFromColor(color), "", true));
		}
		
		updatePosition(ge);		
			
		setSize(strlength, tokensize);
		validate();
		repaint();
	}

	/**
	 * Update the token position.
	 *
	 * @param attributable the attributable
	 */
	protected void updatePosition(Attributable attributable) {
		if (attributable instanceof Node) {
			Node n = (Node) attributable;			

			if(AttributeHelper.hasAttribute(attributable, TokenAttributeContinuous.path, TokenAttributeContinuous.positionAttributeName))
				align = (String) AttributeHelper.getAttributeValue(attributable, TokenAttributeContinuous.path, TokenAttributeContinuous.positionAttributeName,
								GraphicAttributeConstants.CENTERED, "", true);

			LabelComponentHelper.calculateLabelPositionForNode(getWidth(), getHeight(), align, n, geShape, point, null);

			setLocation((int) (point.getX() + shift.getX()), (int) (point.getY() + shift.getY()));

		}
	}

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#setShift(java.awt.Point)
	 */
	@Override
	public void setShift(Point shift) {
		this.shift = shift;
		if (attr != null && attr.getAttributable() != null)
			updatePosition(attr.getAttributable());
	}

	/* (non-Javadoc)
	 * @see org.graffiti.plugin.view.AttributeComponent#adjustComponentSize()
	 */
	@Override
	public void adjustComponentSize() {
		if (attr != null && attr.getAttributable() != null)
			updatePosition(attr.getAttributable());
	}

	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g) {
		super.paintComponent(g);
				
		
		if(new PetriNet(((Node)attr.getAttributable()).getGraph()).getHideToken().getValue())
			return;
		
		double tokens = 0;	
		if(attr.getValue()!=null&&!attr.getValue().toString().equals(""))
			for (String token : attr.getValue().toString().split(TokenAttributeContinuous.seperator)){
				tokens += Double.parseDouble(token);			
			}	

		if (tokens > 0){
			NumberFormat formatter = new DecimalFormat(TokenAttributeContinuous.decimal);
			String txt = formatter.format(tokens);		
			g.setColor(color);
			g.setFont(new Font("arial", Font.BOLD, tokensize));	
			
			strlength = g.getFontMetrics().stringWidth(txt);		
			setSize(strlength, g.getFontMetrics().getAscent());
			adjustComponentSize();
					
			g.drawString(txt, 0, g.getFontMetrics().getAscent()-g.getFontMetrics().getDescent());				
		}			
	}	
}
