/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute.token;

import org.AttributeHelper;
import org.graffiti.graph.Node;

import vanted.petrinetelements.TokenDiscrete;

// TODO: Auto-generated Javadoc
/**
 * The Class TokenAttributeDiscrete handles a discrete token attribute.
 *
 * @author Kevin Pucknat
 */
public class TokenAttributeDiscrete extends AbstractTokenAttribute {	
	
	/** The Constant name. */
	public static final String name = "discrete_Token";	
	
	/** The Constant positionAttributeName. */
	public static final String positionAttributeName = "tokenintposition";
	
	/** The Constant sizeAttributeName. */
	public static final String sizeAttributeName = "tokenintsize";
	
	/** The Constant colorAttributeName. */
	public static final String colorAttributeName = "tokenintcolor";
	
	/**
	 * Instantiates a new token attribute discrete.
	 */
	public TokenAttributeDiscrete() {
		super();
	}
	
	/**
	 * Instantiates a new token attribute discrete.
	 *
	 * @param id the id
	 */
	public TokenAttributeDiscrete(String id) {
		super(id);
	}
	
	/**
	 * Instantiates a new token attribute discrete.
	 *
	 * @param id the id
	 * @param value the value
	 */
	public TokenAttributeDiscrete(String id, String value) {
		super(id, value);
	}
	
	
	/* (non-Javadoc)
	 * @see vanted.attribute.token.AbstractTokenAttribute#doSetValue(java.lang.String)
	 */
	protected void doSetValue(String value) throws IllegalArgumentException {
		setString(value);
		for (String token : value.split(seperator))
			tokens.add(new TokenDiscrete(Integer.parseInt(token)));
	}
	
	
	/**
	 * Removes any token.
	 *
	 * @return true, if successful
	 */
	public boolean removeAnyToken() {
		if (getString().contains(seperator))
			setString(getString().replaceFirst("1@", ""));
		else {
			setString("");
			removeAttributes();
		}
		if (tokens.size() > 0)
			tokens.remove(0);
		return true;
	}
	
	/* (non-Javadoc)
	 * @see vanted.attribute.token.AbstractTokenAttribute#removeAllTokens()
	 */
	public void removeAllTokens() {
		super.removeAllTokens();
		removeAttributes();
	}
	
	/**
	 * Removes the node attributes: name, position, color and size.
	 */
	private void removeAttributes() {
		if (getAttributable() instanceof Node) {
			Node node = (Node) getAttributable();
			AttributeHelper.deleteAttribute(node, TokenAttributeDiscrete.path, TokenAttributeDiscrete.name);
			AttributeHelper.deleteAttribute(node, TokenAttributeDiscrete.path, TokenAttributeDiscrete.positionAttributeName);
			AttributeHelper.deleteAttribute(node, TokenAttributeDiscrete.path, TokenAttributeDiscrete.colorAttributeName);
			AttributeHelper.deleteAttribute(node, TokenAttributeDiscrete.path, TokenAttributeDiscrete.sizeAttributeName);
		}
	}	
}
