/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.attribute.token;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;

import org.AttributeHelper;
import org.color.ColorUtil;
import org.graffiti.attributes.Attributable;
import org.graffiti.attributes.Attribute;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;
import org.graffiti.graphics.GraphicAttributeConstants;
import org.graffiti.plugin.attributecomponent.AbstractAttributeComponent;
import org.graffiti.plugin.view.ShapeNotFoundException;

import vanted.attribute.LabelComponentHelper;
import vanted.petrinetelements.PetriNet;

// TODO: Auto-generated Javadoc
/**
 * The Class TokenAttributeDiscreteComponent visualizes the discrete token.
 *
 * @author Kevin Pucknat
 */
public class TokenAttributeDiscreteComponent extends AbstractAttributeComponent {
	
	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;
	
	/** The point. */
	private final Point point = new Point();
	
	/** The strlength. */
	private int strlength = 10;
	
	/** The tokensize. */
	private int tokensize = 18;
	
	/** The color. */
	private Color color = Color.RED;
	
	/** The align. */
	private String align = GraphicAttributeConstants.CENTERED;
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#attributeChanged(org.graffiti.attributes.Attribute)
	 */
	@Override
	public void attributeChanged(Attribute attr) throws ShapeNotFoundException {
		if (attr.getParent().getName().equals(attr.getParent().getName()))
			recreate();
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#recreate()
	 */
	@Override
	public void recreate() throws ShapeNotFoundException {
		GraphElement ge = (GraphElement) this.attr.getAttributable();
		
		if (ge instanceof Node) {
//			if (AttributeHelper.hasAttribute(ge, TokenAttributeDiscrete.path, TokenAttributeDiscrete.sizeAttributeName))
//				tokensize = (Integer) AttributeHelper.getAttributeValue(ge, TokenAttributeDiscrete.path, TokenAttributeDiscrete.sizeAttributeName, 18, 18);
			if (AttributeHelper.hasAttribute(ge, TokenAttributeDiscrete.path, TokenAttributeDiscrete.colorAttributeName))
				color = ColorUtil.getColorFromHex((String) AttributeHelper.getAttributeValue(ge, TokenAttributeDiscrete.path,
						TokenAttributeDiscrete.colorAttributeName, ColorUtil
								.getHexFromColor(color), "", true));
		}
		tokensize = (int) (geShape.getBounds().width * 0.7);
		updatePosition(ge);
		
		setSize(strlength, tokensize);
		validate();
		repaint();
	}
	
	/**
	 * Update the token position.
	 *
	 * @param attributable the attributable
	 */
	protected void updatePosition(Attributable attributable) {
		if (attributable instanceof Node) {
			Node n = (Node) attributable;
			if (AttributeHelper.hasAttribute(attributable, TokenAttributeDiscrete.path, TokenAttributeDiscrete.positionAttributeName))
				align = (String) AttributeHelper.getAttributeValue(attributable, TokenAttributeDiscrete.path, TokenAttributeDiscrete.positionAttributeName,
								GraphicAttributeConstants.CENTERED, "", true);
			
			LabelComponentHelper.calculateLabelPositionForNode(getWidth(), getHeight(), align, n, geShape, point, null);
			
			setLocation((int) (point.getX() + shift.getX()), (int) (point.getY() + shift.getY()));
			
		}
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.attributecomponent.AbstractAttributeComponent#setShift(java.awt.Point)
	 */
	@Override
	public void setShift(Point shift) {
		this.shift = shift;
		if (attr != null && attr.getAttributable() != null)
			updatePosition(attr.getAttributable());
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.plugin.view.AttributeComponent#adjustComponentSize()
	 */
	@Override
	public void adjustComponentSize() {
		if (attr != null && attr.getAttributable() != null)
			updatePosition(attr.getAttributable());			
	}
	
	/**
	 * Gets the number of tokens.
	 *
	 * @return the token count
	 */
	private int getTokenCount() {
		int tokens = 0;
		if (attr.getValue() != null && !attr.getValue().toString().equals(""))
			for (String token : attr.getValue().toString().split(TokenAttributeDiscrete.seperator)) {
				tokens++;
			}
		return tokens;
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paint(java.awt.Graphics)
	 */
	@Override
	public void paint(Graphics g) {
		super.paintComponent(g);
		if (!(attr.getAttributable() instanceof Node))
			return;
		
		if (new PetriNet(((Node) attr.getAttributable()).getGraph()).getHideToken().getValue())
			return;
				
		if (getTokenCount() > 0) {
			String str = Integer.toString(getTokenCount());
			
			g.setFont(new Font("arial", Font.BOLD, tokensize));
			strlength = g.getFontMetrics().stringWidth(str);
			setSize(strlength, g.getFontMetrics().getAscent());
			adjustComponentSize();
			
			g.setColor(color);
			if (getTokenCount() > 1) {
				g.setClip(-strlength, -g.getFontMetrics().getAscent(), strlength * 2, g.getFontMetrics().getAscent() * 2);
				g.drawString(str, 0, g.getFontMetrics().getAscent() - g.getFontMetrics().getDescent() + g.getFontMetrics().getLeading() + 1);
				
//				g.drawRect(-strlength, -g.getFontMetrics().getAscent(), strlength * 2, g.getFontMetrics().getAscent() * 2);
//				g.fillRect(-strlength, -g.getFontMetrics().getAscent(), strlength * 2, g.getFontMetrics().getAscent() * 2);
			} else{
				g.fillOval(0, g.getFontMetrics().getDescent() - g.getFontMetrics().getLeading(), strlength, strlength);
			
//				g.drawRect(-strlength, -g.getFontMetrics().getAscent(), strlength * 2, g.getFontMetrics().getAscent() * 2);
//				g.fillRect(-strlength, -g.getFontMetrics().getAscent(), strlength * 2, g.getFontMetrics().getAscent() * 2);
			}
		}
	}
}
