/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.gui;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import org.graffiti.graph.Graph;

import vanted.gui.button.JSpinnerDouble;
import vanted.petrinetelements.AbstractArc;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.Transition;
import vanted.petrinetelements.misc.PetriNetType;


public abstract class AbstractGUIComp extends JComponent{	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/** The spinner. */
	protected JSpinnerDouble spinner;
	
	/** The change Button. */
	protected JButton changeBtn;
	
	/** The remove Button. */
	protected JButton removeBtn;

	/** The graph. */
	protected Graph graph;
	
	/** The pn. */
	protected PetriNet pn;

	/** The attribute listener. */
	protected List<AttributeChanged> listener = new ArrayList<AttributeChanged>();
		
	public AbstractGUIComp() {		
		setLayout(new TableLayout(new double[][] {
				{ TableLayoutConstants.FILL, TableLayoutConstants.FILL, TableLayoutConstants.FILL },
				{ TableLayoutConstants.PREFERRED } }));	
		
		add(spinner = new JSpinnerDouble(new SpinnerNumberModel(), ""), "0,0");
		addKeyListener();
		
		add(changeBtn = new JButton(), "1,0");		
		changeBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {	
				if(!((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().getText().equals("~"))
					changeValue();
			}
		});
		
		add(removeBtn = new JButton(), "2,0");		
	}

	public abstract void changeValue();

	public abstract void removeElements(Place place);
	
	public abstract void removeElements(Transition transition);
	
	public abstract void removeElements(AbstractArc arc);	
	

	protected void notifyAttributeListener(){
		for(AttributeChanged ac : listener){
			ac.attributeChanged();
		}
	}
	
	/**
	 * Creates the petri net.
	 *
	 * @return the petri net
	 */
	protected PetriNet createPetriNet(){
		if (graph == null)
			return null;
		
		PetriNet pn = new PetriNet();
		if (!pn.createPetriNet(graph, true)) {
			pn = null;
			return null;
		}
		
		return pn;
	}
	
	/**
	 * Graph changed.
	 *
	 * @param graph the graph
	 */
	public void graphChanged(Graph graph) {
		this.graph = graph;
		if(graph==null)return;
	}
	
	public JSpinnerDouble getSpinner() {
		if(spinner!=null)
			return spinner;
		else
			return spinner = (JSpinnerDouble)new JSpinner();
	}


	public void setSpinner(JSpinnerDouble spinner) {
		this.spinner = spinner;
		add(spinner, "0,0");
//		addKeyListener();
	}


	public JButton getChangeBtn() {
		if(changeBtn!=null)
			return changeBtn;
		else
			return changeBtn = new JButton();
	}


	public void setChangeBtn(JButton changeBtn) {
		this.changeBtn = changeBtn;
		add(changeBtn, "1,0");	
	}


	public JButton getRemoveBtn() {
		if(removeBtn!=null)
			return removeBtn;
		else
			return removeBtn = new JButton();
	}


	public void setRemoveBtn(JButton removeBtn) {
		this.removeBtn = removeBtn;
		add(removeBtn, "2,0");	
	}

	@Override
	public void setEnabled(boolean enabled) {
		if(spinner!=null)spinner.setEnabled(enabled);
		if(changeBtn!=null)changeBtn.setEnabled(enabled);
		if(removeBtn!=null)removeBtn.setEnabled(enabled);		
	}

	public List<AttributeChanged> getAttributeListener() {
		return new ArrayList<AttributeChanged>(listener);
	}

	public void addAttributeListener(AttributeChanged listener) {
		if(!this.listener.contains(listener))
			this.listener.add(listener);
	}
	
	public void removeAttributeListener(AttributeChanged listener) {
		this.listener.remove(listener);
	}
	
	public void addKeyListener(){		
		((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().addKeyListener(new KeyListener() {	
			@Override
			public void keyTyped(KeyEvent arg0) {}
			
			@Override
			public void keyReleased(KeyEvent arg0) {
				if(arg0.getKeyCode()==KeyEvent.VK_ENTER){
					if(!((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().getText().equals("~"))					
						changeValue();					
				}
			}
			
			@Override
			public void keyPressed(KeyEvent arg0) {}
		});	
	}

	
	public abstract void showAttr(PetriNetType type, Object value);	
}
