/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.gui;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import misc.InvariantHelper;
import misc.MatrixOperation;

import org.AttributeHelper;
import org.ReleaseInfo;
import org.graffiti.editor.MainFrame;
import org.graffiti.graph.Edge;
import org.graffiti.graph.Graph;
import org.graffiti.graph.Node;
import org.graffiti.graphics.NodeLabelAttribute;
import org.graffiti.plugin.algorithm.ThreadSafeOptions;
import org.graffiti.selection.Selection;
import org.graffiti.selection.SelectionModel;
import org.graffiti.session.Session;
import org.graffiti.session.SessionListenerExt;

import pipe.analysis.invariants.InvariantAnalysis;
import pipe.analysis.reachability.ReachabilityGraphGenerator;
import vanted.GraphHelper;
import vanted.ThreadSafeClass;
import vanted.addon.PetriAddonTab;
import vanted.analysis.Reachability;
import vanted.attribute.DrawAttributeComponent;
import vanted.petrinetelements.AbstractArc;
import vanted.petrinetelements.NodePN;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.Transition;
import vanted.petrinetelements.misc.DrawingEdge;
import vanted.petrinetelements.misc.DrawingNode;
import vanted.petrinetelements.misc.HideArcWeight;
import vanted.petrinetelements.misc.HideCapacity;
import vanted.petrinetelements.misc.HideLabel;
import vanted.petrinetelements.misc.HideToken;
import de.ipk_gatersleben.ag_nw.graffiti.MyInputHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.layouters.pattern_springembedder.clusterCommands.IntroduceParallelEdgeBends;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.layouters.rt_tree.RTTreeLayout;
import de.ipk_gatersleben.ag_nw.graffiti.services.task.BackgroundTaskHelper;
import de.ipk_gatersleben.ag_nw.graffiti.services.task.BackgroundTaskStatusProviderSupportingExternalCallImpl;

// TODO: Auto-generated Javadoc
/**
 * The Class AnalysisGUI create the whole analysis tab.
 *
 * @author Kevin Pucknat
 */
public class AnalysisGUI extends JComponent implements
		ActionListener,
		SessionListenerExt {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/** The graph. */
	private Graph graph;
	
	/** The referenced graph names. */
	private List<String> referencedGraphNames = new ArrayList<String>();
	
	/** The hide token chk box. */
	private JCheckBox hideTokenChkBox;
	
	/** The hide capacity chk box. */
	private JCheckBox hideCapacityChkBox;
	
	/** The hide arc weight chk box. */
	private JCheckBox hideArcWeightChkBox;
	
	/** The hide label chk box. */
	private JCheckBox hideLabelChkBox;
	
	/** The Constant seperator. */
	public final static String seperator = "@@";
	
	/** The spinner reachability. */
	private JSpinner spinnerReachability;
	
	/**
	 * Instantiates a new analysis gui.
	 */
	public AnalysisGUI() {
		setLayout(new TableLayout(new double[][] {
				{ TableLayoutConstants.FILL }, // Columns
				{ TableLayoutConstants.PREFERRED,
						TableLayoutConstants.PREFERRED,
						TableLayoutConstants.PREFERRED
				} })); // Rows
		
		int pos = 0;
		add(createInvariantsComponent(), "0," + pos++);
		add(createReachabilityComponent(), "0," + pos++);
		add(createDrawingOptionComponent(), "0," + pos++);
		
		MainFrame.getInstance().addSessionListener(this);
	}
	
	/**
	 * Creates the invariants component.
	 *
	 * @return the component
	 */
	private JComponent createInvariantsComponent() {
		JPanel jp = new JPanel();
		jp.setLayout(new TableLayout(new double[][] {
				{ TableLayoutConstants.FILL, TableLayoutConstants.FILL }, { TableLayoutConstants.PREFERRED } }));
		jp.setBorder(BorderFactory.createTitledBorder("Place/Transition Invariants"));
		jp.setOpaque(false);
		
		jp.add(createInvariantsBtn(), "0,0");
		return jp;
	}
	
	/**
	 * Creates the reachability component.
	 *
	 * @return the component
	 */
	private JComponent createReachabilityComponent() {
		JPanel jpReachability = new JPanel();
		jpReachability.setLayout(new TableLayout(new double[][] {
				{ TableLayoutConstants.FILL, TableLayoutConstants.FILL }, { TableLayoutConstants.PREFERRED, TableLayoutConstants.PREFERRED } }));
		jpReachability.setBorder(BorderFactory.createTitledBorder("Reachability/Coverbility Tree"));
		jpReachability.setOpaque(false);
		jpReachability.setToolTipText("After the maximum is reached, there will be build the coverbility graph.");
		
		JLabel label = new JLabel("Maximum fired transitions");
		spinnerReachability = new JSpinner(new SpinnerNumberModel(400, 1, Integer.MAX_VALUE, 10));
		
		jpReachability.add(createReachabilityBtn(), "0,0");
		jpReachability.add(label, "0,1");
		jpReachability.add(spinnerReachability, "1,1");
		return jpReachability;
	}
	
	/**
	 * Creates the drawing option component.
	 *
	 * @return the component
	 */
	private JComponent createDrawingOptionComponent() {
		JPanel jpDrawing = new JPanel();
		jpDrawing.setLayout(new TableLayout(new double[][] {
				{ TableLayoutConstants.FILL, TableLayoutConstants.FILL }, { TableLayoutConstants.PREFERRED,
						TableLayoutConstants.PREFERRED,
						TableLayoutConstants.PREFERRED,
						TableLayoutConstants.PREFERRED } }));
		jpDrawing.setBorder(BorderFactory.createTitledBorder("Draw options"));
		jpDrawing.setOpaque(false);
		
		jpDrawing.add(createHideTokenCheckBox(), "0,0");
		jpDrawing.add(createHideCapacityCheckBox(), "0,1");
		jpDrawing.add(createHideArcWeightCheckBox(), "0,2");
		jpDrawing.add(createHideLabelCheckBox(), "0,3");
		return jpDrawing;
	}
	
	/**
	 * Creates the invariants button.
	 *
	 * @return the button
	 */
	private JButton createInvariantsBtn() {
		JButton btn = new JButton("Invariants");
		btn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (graph == null)
					return;
				
				if (PetriAddonTab.isReachabilityGraph(graph))
					return;
				
				final BackgroundTaskStatusProviderSupportingExternalCallImpl status = new BackgroundTaskStatusProviderSupportingExternalCallImpl("", "");
				final JTabbedPane tabbedPane = new JTabbedPane();
				
				PetriNet pn = new PetriNet();
				if (!pn.createPetriNet(graph, true)) {
					pn = null;
					return;
				}
				
				final ThreadSafeClass tsc = new ThreadSafeClass();
				tsc.setObj(pn);
				
				BackgroundTaskHelper.issueSimpleTask("Analysis", "Please wait...", new Runnable() {
					@Override
					public void run() {
						InvariantAnalysis ia = new InvariantAnalysis();
//						TrigilaDAnna ta = new TrigilaDAnna();
						status.setCurrentStatusText1("Invariant analysis is running");
						MatrixOperation mo = new MatrixOperation((PetriNet) tsc.getObj());
						PetriNet pn = (PetriNet) tsc.getObj();		
						
						List<String> names = new ArrayList<String>();
						for(Place place : pn.getAllPlaces()){
							names.add(place.getName());
						}

						ia.startAnalyse(PetriNet.getMarking(pn),  mo.getIncidenceMatrix().getArrayCopy(), status, names);
						
						List<HashMap<NodePN, Double>> iplist = 
								InvariantHelper.getAllInvariantNodes((pn).getAllPlaces(), ia.get_PInvariants());

						JPanel mainPlace = new JPanel(new BorderLayout());
						mainPlace.setOpaque(false);
						JPanel infoPlaceJP = new JPanel();
						infoPlaceJP.setLayout(new BorderLayout());
						infoPlaceJP.setBorder(BorderFactory.createTitledBorder("Place-Invariants information"));
						infoPlaceJP.setOpaque(false);
						JScrollPane placeJSP = new JScrollPane(new JEditorPane("text/html", ia.getPInvariantStr()));
						infoPlaceJP.add(placeJSP, BorderLayout.CENTER);						

						mainPlace.add(infoPlaceJP, BorderLayout.NORTH);
						
						InvariantTableComponent itp = new InvariantTableComponent();
						JComponent jp = itp.createTableComponent(iplist, "Place-Invariants table");
						mainPlace.add(jp, BorderLayout.CENTER);
						tabbedPane.addTab("Place", mainPlace);

						
						List<HashMap<NodePN, Double>> itlist = InvariantHelper.getAllInvariantNodes(((PetriNet) tsc.getObj()).getAllTransitions(), ia.get_TInvariants());
						JPanel mainTransition = new JPanel(new BorderLayout());
						mainTransition.setOpaque(false);
						JPanel infoTransitionJP = new JPanel(new BorderLayout());
						infoTransitionJP.setBorder(BorderFactory.createTitledBorder("Transition-Invariants information"));
						infoTransitionJP.setOpaque(false);								
						JScrollPane transitionJSP = new JScrollPane(new JEditorPane("text/html", ia.getTInvariantStr()));		
						transitionJSP.setSize(600, 400);
						infoTransitionJP.add(transitionJSP, BorderLayout.CENTER);
						mainTransition.add(infoTransitionJP, BorderLayout.NORTH);
						
						InvariantTableComponent itt = new InvariantTableComponent();
						JComponent jt = itt.createTableComponent(itlist, "Transition-Invariants table");
						mainTransition.add(jt, BorderLayout.CENTER);
						tabbedPane.addTab("Transition", mainTransition);
						
						tabbedPane.setPreferredSize(new Dimension(600, 400));			
						
						tabbedPane.addChangeListener(new ChangeListener() {							
							@Override
							public void stateChanged(ChangeEvent e) {
								removeDrawing(((PetriNet) tsc.getObj()), true);
							}
						});
					}
				}, new Runnable() {
					@Override
					public void run() {						
						Object[] objs = new Object[] { "", tabbedPane };
						MyInputHelper.getInput("Invariants", "Invariants", objs);
						
						removeDrawing(((PetriNet) tsc.getObj()), true);
					}
				}, status);
				
			}
		});
		return btn;
	}
	
	
	
	
	
	/**
	 * Creates the reachability button and starts the creation of the reachability-/coverbilitygraph in a separate thread. 
	 *
	 * @return the j button
	 */
	private JButton createReachabilityBtn() {
		JButton btn = new JButton("Reachability");
		btn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (graph == null)
					return;
				
				if (PetriAddonTab.isReachabilityGraph(graph))
					return;
				
				PetriNet pn = new PetriNet();
				if (!pn.createPetriNet(graph, true)) {
					pn = null;
					return;
				}
				
				String graphName = graph.getName();
				if (graphName.contains(seperator))
					graphName = graphName.substring(0, graphName.indexOf(seperator));
				graph.setName(graphName + seperator
						+ (referencedGraphNames.size()));
				referencedGraphNames.add(graph.getName());
				
				setIgnoreDrawing(new PetriNet(graph));
				final ReachabilityGraphGenerator r = new ReachabilityGraphGenerator();
				
				final ThreadSafeOptions tso = new ThreadSafeOptions();
				
				final BackgroundTaskStatusProviderSupportingExternalCallImpl status = new BackgroundTaskStatusProviderSupportingExternalCallImpl(
						"", "");
				BackgroundTaskHelper.issueSimpleTask("Analysis", "Please wait...", new Runnable() {
					@Override
					public void run() {
						status.setCurrentStatusText1("Petri net creation is running");
						PetriNet sourcePN = new PetriNet();
						if (!sourcePN.createPetriNet(graph, true))
							return;
						status.setCurrentStatusText1("Reachability graph creation is running");
						
						MatrixOperation mo = new MatrixOperation(sourcePN);
						
						List<String[]> labelidentity = new ArrayList<String[]>();
						for(Transition transition : sourcePN.getAllTransitions()){
							labelidentity.add(new String[]{transition.getName(), transition.getIdentity().getValue()+""});
						}
						File reachabilityGraph = new File(ReleaseInfo.getAppSubdirFolderWithFinalSep("addons", "PetriNet") + "results1.rg");
						reachabilityGraph.deleteOnExit();
						File intermediate = new File(ReleaseInfo.getAppSubdirFolderWithFinalSep("addons", "PetriNet") + "graph1.irg");
						intermediate.deleteOnExit();
						
						tso.setGraphInstance(r.startAnalyse(mo.getIncidenceMatrix(), mo.getFMatrix(), mo.getBMatrix(), sourcePN.getCapacityVector(), PetriNet.getMarking(sourcePN), status,
								Integer.parseInt(spinnerReachability.getValue().toString()), reachabilityGraph, 
								intermediate, labelidentity));
					}
				}, new Runnable() {
					@Override
					public void run() {
						if (tso.getGraphInstance() == null)
							return;
						
						status.setCurrentStatusText1("Graph layout is running");
						
						//set the marking as graffiti graph attribute, this is used for the interaction technique
						if(r.getNodeList().size()==r.getMarkingList().size())
							for(int i=0; i<r.getNodeList().size(); i++){
								for(Node node : tso.getGraphInstance().getNodes()){
									if(node.equals(r.getNodeList().get(i))){
										NodePN nodePN = new NodePN(node);
										nodePN.addReachability(new Reachability(r.getMarkingList().get(i)));
									}
								}								
							}						
						
						// create self-loops
						GraphHelper.selfLoop(tso.getGraphInstance());
						
						
						// layout graph
						RTTreeLayout rt = new RTTreeLayout();
						rt.attach(tso.getGraphInstance(), null);
						rt.setParameters(rt.getParameters());
						
						// set root
						for (Node node : tso.getGraphInstance().getNodes()) {
							if (AttributeHelper.getLabel(node, "").equals("S0")) {
								rt.rootedTree(node);
							}
						}
						
						rt.execute();
						
						
						// IntroduceParallelEdgeBends creates new unique ids, but the old ids are needed to find the references
						List<Edge> edges = new ArrayList<Edge>();
						List<Long> ids = new ArrayList<Long>();
						for (Edge edge : tso.getGraphInstance().getEdges()) {
							edges.add(edge);
							ids.add(edge.getID());
						}
						
						//create parallel edges
						IntroduceParallelEdgeBends intro = new IntroduceParallelEdgeBends();
						intro.attach(tso.getGraphInstance(), null);
						intro.execute();
						
						int i = 0;
						for (Edge edge : edges) {
							edge.setID(ids.get(i++));
						}
						// end ids
						
						
						// select nothing
						SelectionModel sm = MainFrame.getInstance().getActiveEditorSession().getSelectionModel();
						sm.setActiveSelection(new Selection("nodes"));
						sm.selectionChanged();
						
						tso.getGraphInstance()
								.setName(
										ReachabilityGraphGenerator.reachabilityGraphName
												+ seperator
												+ (referencedGraphNames
														.size() - 1));
						
						setIgnoreDrawing(new PetriNet(tso
								.getGraphInstance()));
						
						for (Node node : tso.getGraphInstance().getNodes()) {
							new NodePN(node).addDraw(new DrawingNode(DrawAttributeComponent.nodeNumber));
						}
						
						for (Edge edge : tso.getGraphInstance().getEdges()) {
							new AbstractArc(edge) {}.addDraw(new DrawingEdge(AttributeHelper.getLabel(edge, "")));
						}
						
						MainFrame.getInstance().showGraph(
								tso.getGraphInstance(), null);
					}
				}, status);
			}
		});
		return btn;
	}
	
	/**
	 * Gets the graph position in the set of all opened graphs.
	 *
	 * @return the graph position
	 */
	private int getGraphPos() {
		String name = MainFrame.getInstance().getActiveEditorSession()
				.getGraph().getName();
		if (!name.contains(seperator))
			return -1;
		int i = name.lastIndexOf(seperator);
		String end = name.substring(i);
		
		int pos = -1;
		int counter = 0;
		for (String str : referencedGraphNames) {
			if (str.endsWith(end))
				pos = counter;
			counter++;
		}
		
		return pos;
	}
	
	/**
	 * Sets the ignore drawing attribute.
	 *
	 * @param pn the new ignore drawing
	 */
	private void setIgnoreDrawing(PetriNet pn) {
		pn.addHideToken(new HideToken(true));
		// pn.addHideLabel(new HideLabel(true));
		hideTokenChkBox.setSelected(pn.getHideToken().getValue());
		hideArcWeightChkBox.setSelected(pn.getHideArcWeight().getValue());
		hideCapacityChkBox.setSelected(pn.getHideCapacity().getValue());
		hideLabelChkBox.setSelected(pn.getHideLabel().getValue());
	}
	
	/**
	 * Removes the drawing attribute.
	 *
	 * @param pn the pn
	 * @param resetCheckBox the reset check box
	 */
	private void removeDrawing(PetriNet pn, boolean resetCheckBox) {
		pn.removeHideArcWeight();
		pn.removeHideCapacity();
		pn.removeHideToken();
		
		if (resetCheckBox) {
			hideTokenChkBox.setSelected(false);
			hideArcWeightChkBox.setSelected(false);
			hideCapacityChkBox.setSelected(false);
			hideLabelChkBox.setSelected(false);
		}
		
		List<Place> places = pn.getAllPlaces();
		for (Place place : places) {
			place.removeDraw();
		}
		List<Transition> transitions = pn.getAllTransitions();
		for (Transition transition : transitions) {
			transition.removeDraw();
		}
	}
	
	/**
	 * Graph changed.
	 *
	 * @param graph the graph
	 */
	public void graphChanged(Graph graph) {
		this.graph = graph;
		if (referencedGraphNames.isEmpty()
				&& !graph.getName().contains(
						ReachabilityGraphGenerator.reachabilityGraphName))
			referencedGraphNames.add(graph.getName());
		
		if (this.isShowing())
			setStatus(graph);
	}
	
	/**
	 * Sets the status of the 4 buttons: Hide token, Hide capacity, Hide arc-weight and Hide node label.
	 *
	 * @param graph the new status
	 */
	private void setStatus(Graph graph) {
		PetriNet pn = new PetriNet(graph);
		hideTokenChkBox.setSelected(pn.getHideToken().getValue());
		hideArcWeightChkBox.setSelected(pn.getHideArcWeight().getValue());
		hideCapacityChkBox.setSelected(pn.getHideCapacity().getValue());
		hideLabelChkBox.setSelected(pn.getHideLabel().getValue());
	}
	
	/**
	 * Checks the status of the 4 buttons: Hide token, Hide capacity, Hide arc-weight and Hide node label.
	 */
	private void checkStatus() {
		Graph graphTemp = MainFrame.getInstance().getActiveEditorSession()
				.getGraph();
		
		setAttributes(new PetriNet(graphTemp));
		
		Graph refGraph = getRefGraph(graphTemp);
		
		if (refGraph == null)
			return;
		
		setAttributes(new PetriNet(refGraph));
	}
	
	/**
	 * Sets the attributes of the 4 buttons: Hide token, Hide capacity, Hide arc-weight and Hide node label.
	 *
	 * @param pn the new attributes
	 */
	private void setAttributes(PetriNet pn) {
		if (hideTokenChkBox.isSelected()) {
			pn.addHideToken(new HideToken(true));
		} else {
			pn.removeHideToken();
		}
		if (hideArcWeightChkBox.isSelected()) {
			pn.addHideArcWeight(new HideArcWeight(true));
		} else {
			pn.removeHideArcWeight();
		}
		if (hideCapacityChkBox.isSelected()) {
			pn.addHideCapacity(new HideCapacity(true));
		} else {
			pn.removeHideCapacity();
		}
		if (hideLabelChkBox.isSelected()) {
			pn.addHideLabel(new HideLabel(true));
		} else {
			pn.removeHideLabel();
		}
	}
	
	/**
	 * Gets the referenced graph. The Petri net and the reachability graph are referenced with the separator and the number in the graph names (e.g. ...@@1).
	 *
	 * @param graph the graph
	 * @return the ref graph
	 */
	private Graph getRefGraph(Graph graph) {
		int pos = getGraphPos();
		if (pos == -1)
			return null;
		Graph refGraph = null;
		if (graph.getName().contains(
				ReachabilityGraphGenerator.reachabilityGraphName)) {
			for (Session session : MainFrame.getEditorSessions()) {
				if (session.getGraph().getName()
						.equals(referencedGraphNames.get(pos))) {
					refGraph = session.getGraph();
				}
			}
		} else {
			for (Session session : MainFrame.getEditorSessions()) {
				if (session
						.getGraph()
						.getName()
						.equals(ReachabilityGraphGenerator.reachabilityGraphName
								+ seperator + pos)) {
					refGraph = session.getGraph();
				}
			}
		}
		return refGraph;
	}
	
	/**
	 * Creates the hide token check box.
	 *
	 * @return the component
	 */
	private JComponent createHideTokenCheckBox() {
		hideTokenChkBox = new JCheckBox("Hide token");
		hideTokenChkBox.setOpaque(false);
		hideTokenChkBox.setSelected(false);
		hideTokenChkBox.addActionListener(this);
		return hideTokenChkBox;
	}
	
	/**
	 * Creates the hide capacity check box.
	 *
	 * @return the component
	 */
	private JComponent createHideCapacityCheckBox() {
		hideCapacityChkBox = new JCheckBox("Hide capacity");
		hideCapacityChkBox.setOpaque(false);
		hideCapacityChkBox.setSelected(false);
		hideCapacityChkBox.addActionListener(this);
		return hideCapacityChkBox;
	}
	
	/**
	 * Creates the hide arc-weight check box.
	 *
	 * @return the component
	 */
	private JComponent createHideArcWeightCheckBox() {
		hideArcWeightChkBox = new JCheckBox("Hide arc-weight");
		hideArcWeightChkBox.setOpaque(false);
		hideArcWeightChkBox.setSelected(false);
		hideArcWeightChkBox.addActionListener(this);
		return hideArcWeightChkBox;
	}
	
	/**
	 * Creates the hide label check box.
	 *
	 * @return the component
	 */
	private JComponent createHideLabelCheckBox() {
		hideLabelChkBox = new JCheckBox("Hide node label");
		hideLabelChkBox.setOpaque(false);
		hideLabelChkBox.setSelected(false);
		hideLabelChkBox.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (graph != null)					 
					if (hideLabelChkBox.isSelected()) {
						new PetriNet(graph).addHideLabel(new HideLabel(true));
						new PetriNet(getRefGraph(graph)).addHideLabel(new HideLabel(true));
						setHideLabel();
					} else {
						new PetriNet(graph).removeHideLabel();
						new PetriNet(getRefGraph(graph)).removeHideLabel();
						setHideLabel();
					}
				
			}
		});
		return hideLabelChkBox;
	}
	
	/** The font size. */
	private int fontSize = -1;
	
	/**
	 * Sets the attribute HideLabel.
	 */
	private void setHideLabel() {
		Graph tempGraph = null;
		if (graph.getName().contains(ReachabilityGraphGenerator.reachabilityGraphName))
			tempGraph = getRefGraph(graph);
		else
			tempGraph = graph;
		for (Node node : tempGraph.getNodes()) {
			if (hideLabelChkBox.isSelected()) {
				NodeLabelAttribute nl = AttributeHelper.getLabel(-1, node);
				if (nl.getFontSize() > 0) {
					fontSize = nl.getFontSize();
					nl.setFontSize(0);
				}
			} else {
				NodeLabelAttribute nl = AttributeHelper.getLabel(-1, node);
				if (fontSize > 0)
					nl.setFontSize(fontSize);
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		checkStatus();
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.session.SessionListener#sessionChanged(org.graffiti.session.Session)
	 */
	@Override
	public void sessionChanged(Session s) {
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.session.SessionListener#sessionDataChanged(org.graffiti.session.Session)
	 */
	@Override
	public void sessionDataChanged(Session s) {
	}
	
	/* (non-Javadoc)
	 * @see org.graffiti.session.SessionListenerExt#sessionClosed(org.graffiti.session.Session)
	 */
	@Override
	public void sessionClosed(Session session) {
		if (session.getGraph().getName().contains(ReachabilityGraphGenerator.reachabilityGraphName)) {
			Graph temp = getRefGraph(session.getGraph());
			if (temp != null) {
				PetriNet pn = new PetriNet();
				pn.createPetriNet(temp, true);
				removeDrawing(pn, true);
			}
		}
	}
}
