/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JSpinner;

import org.graffiti.editor.MainFrame;
import org.graffiti.graph.Edge;

import vanted.attribute.token.TokenAttributeContinuous;
import vanted.petrinetelements.AbstractArc;
import vanted.petrinetelements.ArcWeight;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.Transition;
import vanted.petrinetelements.misc.PetriNetType;

public class GUICompArcWeight extends AbstractGUIComp{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public GUICompArcWeight() {
		changeBtn.setText("<html>Change Arc-Weight");
		removeBtn.setText("<html>Remove ArcWeight");
		removeBtn.setToolTipText("Removes the Arc-Weight from a arc or from all selected arcs (all arcs must have the same Arc-Weight)");
		
		removeBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				pn = createPetriNet();
				if(pn==null)return;
				
				for(Edge edge : MainFrame.getInstance().getActiveEditorSession().getSelectionModel().getActiveSelection().getEdges())
					if(edge.isDirected())
						removeElements(new AbstractArc(edge) {});
			}
		});
	}

	@Override
	public void changeValue() {
		PetriNet pn = createPetriNet();
		if(pn==null)return;
		
		double weight = Double.valueOf(spinner.getValue(
				TokenAttributeContinuous.decimal).toString());
		
		if(MainFrame.getInstance().getActiveEditorSession()
				.getSelectionModel().getActiveSelection()==null)return;
		
		for (Edge edge : MainFrame.getInstance().getActiveEditorSession()
				.getSelectionModel().getActiveSelection().getEdges()) {
			
			AbstractArc arc = null;
			for (AbstractArc a : pn.getAllArcs()) {
				for (Edge e : a.getEdges())
					if (e.equals(edge)){
						arc = a;
						break;
					}
			}
		
			if (weight == 1 || weight==0){				
				arc.removeArcWeight();
			}
			else{				
				if(PetriNetType.getType2(graph).equals(PetriNetType.DISCRETE)){
					arc.addArcWeight(new ArcWeight((int)weight));
				}else{
					arc.addArcWeight(new ArcWeight(weight));
				}
				notifyAttributeListener();
			}
		}
	}

	@Override
	public void removeElements(Place place) {}
	@Override
	public void removeElements(Transition transition) {}

	@Override
	public void removeElements(AbstractArc arc) {
		arc.removeArcWeight();
		notifyAttributeListener();
	}
	
	@Override
	public void showAttr(PetriNetType type, Object value) {
		spinner.setValue(value);		
		if(((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().getText().equals("~"))
			((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().setValue(((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().getValue());
	}
}
