/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import org.graffiti.editor.MainFrame;
import org.graffiti.graph.Node;

import vanted.attribute.token.TokenAttributeContinuous;
import vanted.petrinetelements.AbstractArc;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.TokenContinuous;
import vanted.petrinetelements.Transition;
import vanted.petrinetelements.misc.PetriNetType;

public class GUICompContinuousToken extends AbstractGUIComp{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public GUICompContinuousToken() {
		super();		
		spinner.setModel(new SpinnerNumberModel(5d, 0d, Double.MAX_VALUE, 0.1d));
		addKeyListener();
		
		changeBtn.setText("<html>Change Token <small>(continuous)");
		removeBtn.setText("<html>Remove Token");
		removeBtn.setToolTipText("Removes the tokens from all selected places");
		
		removeBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
//				if(!((JSpinner.DefaultEditor) spinnerTokenDiscrete.getEditor()).getTextField().getText().equals(emptyString)){  
					pn = createPetriNet();
					if(pn==null)return;
					
					for(Node nd : MainFrame.getInstance().getActiveEditorSession().getSelectionModel().getActiveSelection().getNodes()){					
						Place place = null;
						for (Place p : pn.getAllPlaces()) {
							for (Node node : p.getNodes())
								if (node.equals(nd))
									place = p;
						}
						if(place!=null)
							removeElements(place);
					}
//				}
			}
		});	
	}

	@Override
	public void changeValue() {
		//Pn wurde gebaut und type wurde bestimmt
		
		PetriNet pn = createPetriNet();
		if(pn==null)return;
		
		PetriNetType type = PetriNetType.getTokenType(pn.getAllPlaces());
		if (!(type.equals(PetriNetType.CONTINUOUS) || type.equals(PetriNetType.EMPTY))) {
			MainFrame.showMessageDialog(
					"<html>The Petri-Net already contains non-continous tokens.",
					"Continuous Token Error");
			return;
		}

		double value = Double.valueOf(spinner.getValue(TokenAttributeContinuous.decimal).toString());
						
		if(MainFrame.getInstance().getActiveEditorSession()
				.getSelectionModel().getActiveSelection()==null)return;
		
		for (Node nd : MainFrame.getInstance().getActiveEditorSession()
				.getSelectionModel().getActiveSelection().getNodes()) {
			
			Place place = null;
			for (Place p : pn.getAllPlaces()) {
				for (Node node : p.getNodes())
					if (node.equals(nd))
						place = p;
			}
			
			if(place==null)continue;
			double capacity = Double.parseDouble(place.getCapacity().getValue()
					.toString());
			
			place.removeAllToken();
			
			if (value == 0)
				continue;
			
			if (value <= capacity) {
				place.addToken(new TokenContinuous(value));
				notifyAttributeListener();
			} else
				MainFrame
						.showMessageDialog(
								"<html>The token weight is higher than the place capacity. Increase the place capacity or remove token from the place",
								"Continuous Token Error");
		}		
	}

	@Override
	public void removeElements(Place place) {
		place.removeAllToken();		
		
		notifyAttributeListener();
	}

	@Override
	public void removeElements(Transition transition) {}
	@Override
	public void removeElements(AbstractArc arc) {}
	
	@Override
	public void showAttr(PetriNetType type, Object value) {
		spinner.setValue(value);
		if(((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().getText().equals("~"))
			((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().setValue(((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().getValue());
	}
}
