/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import org.graffiti.editor.MainFrame;
import org.graffiti.graph.Node;

import vanted.petrinetelements.AbstractArc;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.TokenDiscrete;
import vanted.petrinetelements.Transition;
import vanted.petrinetelements.misc.PetriNetType;

public class GUICompDiscreteToken extends AbstractGUIComp{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public GUICompDiscreteToken() {
		super();
		spinner.setModel(new SpinnerNumberModel(1, 0, Integer.MAX_VALUE, 1));
		addKeyListener();
		
		changeBtn.setText("<html>Change Token <small>(discrete)");
		removeBtn.setText("<html>Remove Token");
		removeBtn.setToolTipText("Removes the tokens from all selected places");
		
		removeBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) { 
				pn = createPetriNet();
				if(pn==null)return;
				
				for(Node nd : MainFrame.getInstance().getActiveEditorSession().getSelectionModel().getActiveSelection().getNodes()){					
					Place place = null;
					for (Place p : pn.getAllPlaces()) {
						for (Node node : p.getNodes())
							if (node.equals(nd))
								place = p;
					}
					if(place!=null)
						removeElements(place);
				}
			}
		});	
	}


	@Override
	public void changeValue() {	
		PetriNet pn = createPetriNet();
		if(pn==null)return;
		
		PetriNetType type = PetriNetType.getTokenType(pn.getAllPlaces());
		if (!(type.equals(PetriNetType.DISCRETE) || type.equals(PetriNetType.EMPTY))) {
			MainFrame.showMessageDialog(
					"<html>The Petri-Net already contains non-discrete tokens.",
					"Continuous Token Error");
			return;
		}

		int count = Integer.valueOf(spinner.getValue().toString());
			
		for (Node nd : MainFrame.getInstance().getActiveEditorSession()
				.getSelectionModel().getActiveSelection().getNodes()) {
			Place place = null;
			for (Place p : pn.getAllPlaces()) {
				for (Node node : p.getNodes())
					if (node.equals(nd))
						place = p;
			}
			
			if(place==null)continue;
			double capacity = Double.parseDouble(place.getCapacity().getValue()
					.toString());
			
			place.removeAllToken();
			if (count == 0)
				continue;
			
			if (count <= capacity) {
				for (int i = 0; i < count; i++) {
					place.addToken(new TokenDiscrete(1));
				}
				notifyAttributeListener();
			} else
				MainFrame
						.showMessageDialog(
								"<html>The token weight is higher than the place capacity. Increase the place capacity or remove token from the place",
								"Discrete Token Error");
		}	
	}

	@Override
	public void removeElements(Place place) {
		place.removeAllToken();				
		notifyAttributeListener();
	}

	@Override
	public void removeElements(Transition transition) {}
	@Override
	public void removeElements(AbstractArc arc) {}	
	
	@Override
	public void showAttr(PetriNetType type, Object value) {
		spinner.setValue(value);
		if(((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().getText().equals("~"))
			((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().setValue(((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().getValue());
	}
}
