/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JSpinner;

import org.graffiti.editor.MainFrame;
import org.graffiti.graph.Node;

import vanted.attribute.token.TokenAttributeContinuous;
import vanted.petrinetelements.AbstractArc;
import vanted.petrinetelements.AbstractToken;
import vanted.petrinetelements.Capacity;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.TokenContinuous;
import vanted.petrinetelements.TokenDiscrete;
import vanted.petrinetelements.Transition;
import vanted.petrinetelements.misc.PetriNetType;

public class GUICompPlaceCapacity extends AbstractGUIComp{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;



	public GUICompPlaceCapacity() {		
		super();		
		changeBtn.setText("<html>Change Place-Capacity");
		removeBtn.setText("<html>Remove Capacity");
		removeBtn.setToolTipText("Removes the capacity from a place or from all selected places (all places must have the same capacity)");
		
		removeBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
//				if(!((JSpinner.DefaultEditor) spinnerTokenDiscrete.getEditor()).getTextField().getText().equals(emptyString)){  
					pn = createPetriNet();
					if(pn==null)return;
					
					for(Node nd : MainFrame.getInstance().getActiveEditorSession().getSelectionModel().getActiveSelection().getNodes()){					
						Place place = null;
						for (Place p : pn.getAllPlaces()) {
							for (Node node : p.getNodes())
								if (node.equals(nd))
									place = p;
						}
						if(place!=null)
							removeElements(place);
					}
//				}
			}
		});	
	}

	@Override
	public void changeValue() {
		//Pn wurde gebaut und type wurde bestimmt
		
		PetriNet pn = createPetriNet();
		if(pn==null)return;
		
		double capacity = Double.valueOf(spinner.getValue(
				TokenAttributeContinuous.decimal).toString());
		
		if(MainFrame.getInstance().getActiveEditorSession()
				.getSelectionModel().getActiveSelection()==null)return;
		
		for (Node nd : MainFrame.getInstance().getActiveEditorSession()
				.getSelectionModel().getActiveSelection().getNodes()) {
			
			Place place = null;
			for (Place p : pn.getAllPlaces()) {
				for (Node node : p.getNodes())
					if (node.equals(nd))
						place = p;
			}
			
			if(place==null)continue;
			double tokenWeight = 0;
			for (AbstractToken token : place.getTokens()) {
				if (token instanceof TokenDiscrete)
					tokenWeight += ((TokenDiscrete) token).getValue();
				else
					if (token instanceof TokenDiscrete)
						tokenWeight += ((TokenContinuous) token).getValue();
			}
			
			if (capacity == 0) {
				place.removeCapacity();
				continue;
			}
			
			if (tokenWeight <= capacity){
				if(PetriNetType.getType2(graph).equals(PetriNetType.DISCRETE)){
					place.addCapacity(new Capacity((int)capacity));
				}else
					place.addCapacity(new Capacity(capacity));
				
				notifyAttributeListener();		
			}
				
			else
				MainFrame
						.showMessageDialog(
								"The token weight is height as the place capacity. Increase the place capacity or remove token from the place",
								"Capacity Error");
		}		
	}

	@Override
	public void removeElements(Place place) {
		place.removeCapacity();		
		
		notifyAttributeListener();		
	}

	@Override
	public void removeElements(Transition transition) {}
	@Override
	public void removeElements(AbstractArc arc) {}

	
	
	@Override
	public void showAttr(PetriNetType type, Object value) {
		spinner.setValue(value);
		if(((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().getText().equals("~"))
			((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().setValue(((JSpinner.DefaultEditor) spinner.getEditor()).getTextField().getValue());
	}	
}
