/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.gui;

import java.awt.BorderLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import org.graffiti.graph.Graph;

import vanted.attribute.token.TokenAttributeContinuous;
import vanted.petrinetelements.NodePN;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Transition;
import vanted.petrinetelements.misc.DrawingNode;
import vanted.petrinetelements.misc.HideToken;

public class InvariantTableComponent {
	
	private Graph graph;
	/**
	 * Creates the table component for the p- and t-invariant.
	 *
	 * @param invList the invariant list
	 * @return the component
	 */
	public JComponent createTableComponent(final List<HashMap<NodePN, Double>> invList, String borderText) {
		if(invList != null && invList.size() > 0)
				if(invList.get(0).keySet().iterator().hasNext())
					graph = invList.get(0).keySet().iterator().next().getNode().getGraph();
				
		
		
		if (invList == null || invList.size()==0)
			return new JLabel();
		
		// create table header
		List<Object> columns = createTableHeader(invList.get(0));		
		
		Object[][] data = new Object[invList.size()][];
			
		NumberFormat format1 = new DecimalFormat(
				TokenAttributeContinuous.decimal);
		
		ArrayList<Object> row = new ArrayList<Object>(); 
		int i=0;
		// add data
		for (HashMap<NodePN, Double> map : invList) {	
			row = new ArrayList<Object>();
			for (NodePN node : map.keySet())
				if(map.get(node) != 0)
					row.add(format1.format(map.get(node)));			
				else{
					row.add("");
				}

			data[i++] = row.toArray();
		}


		JTable table = new JTable(data, columns.toArray()){
			public boolean isCellEditable(int rowIndex, int colIndex) {
				  return false;
			}
		};	
		table.addMouseMotionListener(new MouseMotionListener() {
			int last=-1;
			@Override
			public void mouseMoved(MouseEvent e) {
				JTable table = ((JTable)e.getSource());	
				
				int row = table.rowAtPoint(e.getPoint());			
				if(row!=-1){									
					table.setRowSelectionInterval(row, row);
					setNewDrawing(invList.get(row), invList.get(last==-1?0:last));					
					last = row;
				}
			}
			
			@Override
			public void mouseDragged(MouseEvent e) {}
		});
			
		JPanel jp = new JPanel(new BorderLayout());
		jp.setBorder(BorderFactory.createTitledBorder(borderText));
		JScrollPane scp = new JScrollPane(table);
		jp.add(scp, BorderLayout.CENTER);
		jp.setOpaque(false);
		return jp;
	}
	
	/**
	 * Creates a new table header.
	 *
	 * @param header the header data
	 * 
	 * @return the header list
	 */
	private List<Object> createTableHeader(HashMap<NodePN, Double> header){
		List<Object> columns = new ArrayList<Object>();
		for (NodePN node : header.keySet()) 	
			columns.add(rev(node));			
		
		return columns;
	}
	
	private String rev(NodePN node){
		if(node instanceof Transition)
			if(((Transition)node).isRevTransition())
				return node.getName()+"_rev";			
		
		return node.getName();
	}
	
	/**
	 * Sets the new drawing attribute.
	 *
	 * @param nodes the nodes
	 * @param lastNodes the last nodes
	 */
	private void setNewDrawing(HashMap<NodePN, Double> nodes,
			HashMap<NodePN, Double> lastNodes) {
		PetriNet pn = new PetriNet(graph);
		pn.addHideToken(new HideToken(true));
		
		// restore original
		if(lastNodes == null)return;
		for (NodePN node : lastNodes.keySet()) 
			node.removeDraw();			
		
		// set new drawing
		if(nodes == null)return;
		for (NodePN node : nodes.keySet()) 
			node.addDraw(new DrawingNode(nodes.get(node)));			
	}
}
