/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.gui;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import org.graffiti.editor.MainFrame;
import org.graffiti.graph.Edge;
import org.graffiti.graph.Graph;
import org.graffiti.graph.Node;
import org.graffiti.selection.SelectionEvent;
import org.graffiti.selection.SelectionListener;

import vanted.gui.button.JSpinnerDouble;
import vanted.petrinetelements.AbstractArc;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.TokenContinuous;
import vanted.petrinetelements.misc.PetriNetType;

// TODO: Auto-generated Javadoc
/**
 * The Class PetriNetAttributeGUI creates the whole Petri net attribute component in the Reconstruction tab.
 * 
 * @author Kevin Pucknat
 */
public class PetriNetAttributeGUI extends JComponent implements SelectionListener, AttributeChanged{	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/** The graph. */
	private Graph graph;
	
	/** The discrete token components. */
	private AbstractGUIComp discreteTokenComp;
	
	/** The continuous token components. */
	private AbstractGUIComp continuousTokenComp;
	
	/** The place capacity components. */
	private AbstractGUIComp placeCapacityComp;
	
	/** The arc weigt comp. */
	private AbstractGUIComp arcWeigtComp;
	
	/** The type. */
	private PetriNetType type = PetriNetType.EMPTY;
	
	/** The empty string. */
	private String emptyString = "~"; 
		
	
	/**
	 * Instantiates a new petri net attribute gui.
	 */
	public PetriNetAttributeGUI() {
		buildGUI();
	}
	
	/**
	 * Builds the gui for all attributes.
	 */
	private void buildGUI() {
		removeAll();
		setLayout(new TableLayout(new double[][] {
				{ TableLayoutConstants.FILL }, // Columns
				{ TableLayoutConstants.PREFERRED,
						TableLayoutConstants.PREFERRED } })); // Rows
		
		JPanel jp = new JPanel();
		jp.setLayout(new TableLayout(new double[][] {
				{ TableLayoutConstants.FILL },
				{ TableLayoutConstants.PREFERRED,
						TableLayoutConstants.PREFERRED,
						TableLayoutConstants.PREFERRED,
						TableLayoutConstants.PREFERRED } }));
		jp.setBorder(BorderFactory.createTitledBorder("Petri-Net attributes"));
		jp.setOpaque(false);

		
		jp.add(discreteTokenComp = new GUICompDiscreteToken(), "0,0");		
		discreteTokenComp.addAttributeListener(this);
		jp.add(continuousTokenComp = new GUICompContinuousToken(), "0,1");
		continuousTokenComp.addAttributeListener(this);
		jp.add(placeCapacityComp = new GUICompPlaceCapacity(), "0,2");	
		placeCapacityComp.addAttributeListener(this);
		jp.add(arcWeigtComp = new GUICompArcWeight(), "0,3");
		arcWeigtComp.addAttributeListener(this);

		add(jp, "0,0");
		MainFrame.getInstance().addSelectionListener(this);		
	}
			
	/**
	 * Graph changed.
	 *
	 * @param graph the graph
	 */
	public void graphChanged(Graph graph) {
		//TODO: call GUIComp the changes
		discreteTokenComp.graphChanged(graph);
		continuousTokenComp.graphChanged(graph);
		placeCapacityComp.graphChanged(graph);
		arcWeigtComp.graphChanged(graph);
		this.graph = graph;
		if(graph==null)return;
		attributeChanged();
	}
		
	/* (non-Javadoc)
	 * @see org.graffiti.selection.SelectionListener#selectionChanged(org.graffiti.selection.SelectionEvent)
	 */
	@Override
	public void selectionChanged(SelectionEvent e) {		
		refreshPlaces();
		refreshArcs();
	}
	
	
	/**
	 * Gets the selected places.
	 *
	 * @return the selected places
	 */
	private List<Place> getSelectedPlaces(){
		List<Place> places = new ArrayList<Place>();
		
		if(MainFrame.getInstance().getActiveEditorSession()
				.getSelectionModel().getActiveSelection()==null)return null;
		
		Collection<Node> nodes = MainFrame.getInstance().getActiveEditorSession()
				.getSelectionModel().getActiveSelection().getNodes();
		
		for(Node node : nodes){
			if(Place.isPlace(node)){
				places.add(new Place(node));				
			}
		}
		if(places.size()==0)return null;
		
		return places;
	}
	
	/**
	 * Gets the selected arcs.
	 *
	 * @return the selected arcs
	 */
	private List<AbstractArc> getSelectedArcs(){
		List<AbstractArc> arcs = new ArrayList<AbstractArc>();
		
		if(MainFrame.getInstance().getActiveEditorSession()
				.getSelectionModel().getActiveSelection()==null)return null;
		
		for(Edge edge : MainFrame.getInstance().getActiveEditorSession()
				.getSelectionModel().getActiveSelection().getEdges()){
			if(edge.isDirected()){
				arcs.add(new AbstractArc(edge){});				
			}
		}
		if(arcs.size()==0)return null;
		
		return arcs;
	}
	
	
	/**
	 * Change gui attributes.
	 */
	private void changeGuiAttributes(){
		if(type.equals(PetriNetType.EMPTY)){
			continuousTokenComp.setEnabled(true);
			discreteTokenComp.setEnabled(true);
			if(placeCapacityComp!=null){				
				if(placeCapacityComp.getSpinner().getModel().getValue() instanceof Integer || Double.valueOf(placeCapacityComp.getSpinner().getModel().getValue().toString()) == 0){
					placeCapacityComp.getSpinner().setModel(new SpinnerNumberModel(Double.parseDouble(((JSpinnerDouble)placeCapacityComp.getSpinner()).getValue().toString()), 0d, Double.MAX_VALUE, 0.1));
					placeCapacityComp.addKeyListener();				
				}
			}
			if(arcWeigtComp!=null){
				if(arcWeigtComp.getSpinner().getModel().getValue() instanceof Integer || Double.valueOf(placeCapacityComp.getSpinner().getModel().getValue().toString()) == 0){
					arcWeigtComp.getSpinner().setModel(new SpinnerNumberModel(Double.parseDouble(arcWeigtComp.getSpinner().getValue().toString()), 0d, Double.MAX_VALUE, 0.1));
					arcWeigtComp.addKeyListener();
				}
			}
			discreteTokenComp.showAttr(null, 0);
			continuousTokenComp.showAttr(null, 0);
			placeCapacityComp.showAttr(null, 0);
			arcWeigtComp.showAttr(null, 1);
		}else if(type.equals(PetriNetType.DISCRETE)){
			discreteTokenComp.setEnabled(true);
			continuousTokenComp.setEnabled(false);
			if(placeCapacityComp!=null){				
				if(placeCapacityComp.getSpinner().getModel().getValue() instanceof Double || Double.valueOf(placeCapacityComp.getSpinner().getModel().getValue().toString()) == 0){
					placeCapacityComp.getSpinner().setModel(new SpinnerNumberModel((int)Double.parseDouble(((JSpinnerDouble)placeCapacityComp.getSpinner()).getValue().toString()), 0, Integer.MAX_VALUE, 1));
					placeCapacityComp.addKeyListener();
				}
			}
			if(arcWeigtComp!=null){
				if(arcWeigtComp.getSpinner().getModel().getValue() instanceof Double || Double.valueOf(placeCapacityComp.getSpinner().getModel().getValue().toString()) == 0){			
					arcWeigtComp.getSpinner().setModel(new SpinnerNumberModel((int)Double.parseDouble(arcWeigtComp.getSpinner().getValue().toString()), 0, Integer.MAX_VALUE, 1));
					arcWeigtComp.addKeyListener();
				}
			}
		}else if(type.equals(PetriNetType.CONTINUOUS)){
			continuousTokenComp.setEnabled(true);
			discreteTokenComp.setEnabled(false);
			if(placeCapacityComp!=null){				
				if(placeCapacityComp.getSpinner().getModel().getValue() instanceof Integer || Double.valueOf(placeCapacityComp.getSpinner().getModel().getValue().toString()) == 0){
					placeCapacityComp.getSpinner().setModel(new SpinnerNumberModel(Double.parseDouble(((JSpinnerDouble)placeCapacityComp.getSpinner()).getValue().toString()), 0d, Double.MAX_VALUE, 0.1));
					placeCapacityComp.addKeyListener();
				}
			}
			if(arcWeigtComp!=null){
				if(arcWeigtComp.getSpinner().getModel().getValue() instanceof Integer || Double.valueOf(placeCapacityComp.getSpinner().getModel().getValue().toString()) == 0){
					arcWeigtComp.getSpinner().setModel(new SpinnerNumberModel(Double.parseDouble(arcWeigtComp.getSpinner().getValue().toString()), 0d, Double.MAX_VALUE, 0.1));
					arcWeigtComp.addKeyListener();
				}
			}
		}
	}
		

	/* (non-Javadoc)
	 * @see org.graffiti.selection.SelectionListener#selectionListChanged(org.graffiti.selection.SelectionEvent)
	 */
	@Override
	public void selectionListChanged(SelectionEvent e) {		
	}

	@Override
	public void attributeChanged() {
		type=PetriNetType.getType2(graph);
		changeGuiAttributes();
	}
	
	
	private void refreshPlaces(){
		List<Place> places = getSelectedPlaces();
		if(places==null){
			discreteTokenComp.showAttr(null, 0);
			continuousTokenComp.showAttr(null, 0);
			placeCapacityComp.showAttr(null, 0);
			return;
		}
		
		for(Place place : places){
			switch(type){
				case DISCRETE: 					
					if(!isDifferentTokenDiscreteValues(places))
						discreteTokenComp.showAttr(type, place.getTokensDiscrete().size());
					else					
						((JSpinner.DefaultEditor) discreteTokenComp.getSpinner().getEditor()).getTextField().setText(emptyString);					
					if(!isDifferentCapacityDiscreteValues(places)){
						int capa = (int)Double.parseDouble(place.getCapacity().getValue().toString());						
						placeCapacityComp.showAttr(type, capa==Integer.MAX_VALUE?0:capa);
					}
					else
						((JSpinner.DefaultEditor) placeCapacityComp.getSpinner().getEditor()).getTextField().setText(emptyString);
					break;					
				case CONTINUOUS: 
					if(!isDifferentTokenContinuousValues(places))
						continuousTokenComp.showAttr(type, getTokenContinuousValue(place));
					else
						((JSpinner.DefaultEditor) continuousTokenComp.getSpinner().getEditor()).getTextField().setText(emptyString);
					
					if(!isDifferentCapacityContinuousValues(places)){
						double value = Double.parseDouble(place.getCapacity().getValue().toString());
						placeCapacityComp.showAttr(type, value==Integer.MAX_VALUE?0:value);
					}else
						((JSpinner.DefaultEditor) placeCapacityComp.getSpinner().getEditor()).getTextField().setText(emptyString);
					break;
					
				case EMPTY:
					if(!isDifferentCapacityDiscreteValues(places)){
						double capa = Double.parseDouble(place.getCapacity().getValue().toString());						
						placeCapacityComp.showAttr(type, capa==Integer.MAX_VALUE?0:capa);
					}else
						((JSpinner.DefaultEditor) placeCapacityComp.getSpinner().getEditor()).getTextField().setText(emptyString);
					break;
			}
		}	
	}
	
	private boolean isDifferentTokenDiscreteValues(List<Place> places){
		if(places.size()<=1)return false;
		
		int count=places.get(0).getTokensDiscrete().size();
		for(Place place : places){
			if(count!=place.getTokensDiscrete().size())
				return true;
		}
			
		return false;
	}
	
	private boolean isDifferentTokenContinuousValues(List<Place> places){
		if(places.size()<=1)return false;
		
		double value=-1;
		double sum=0;
		for(Place place : places){
			sum=getTokenContinuousValue(place);				
			if(value!=sum && value!=-1)return true;
			value=sum;
		}
			
		return false;
	}
	
	private boolean isDifferentCapacityDiscreteValues(List<Place> places){
		if(places.size()<=1)return false;
						
		int value=(int)Double.parseDouble(places.get(0).getCapacity().getValue().toString());
		for(int i=1; i<places.size(); i++){
			if(value!=(int)Double.parseDouble(places.get(i).getCapacity().getValue().toString()))
				return true;
		}
			
		return false;
	}
	
	private boolean isDifferentCapacityContinuousValues(List<Place> places){
		if(places.size()<=1)return false;
						
		double value=Double.parseDouble(places.get(0).getCapacity().getValue().toString());
		for(int i=1; i<places.size(); i++){
			if(value!=Double.parseDouble(places.get(i).getCapacity().getValue().toString()))
				return true;
		}
			
		return false;
	}
	
	private double getTokenContinuousValue(Place place){
		double value=0;
		for(TokenContinuous token : place.getTokensContinuous())
			value+=token.getValue();
		return value;
	}
	
	private void refreshArcs(){
		List<AbstractArc> arcs = getSelectedArcs();		
		if(arcs==null){
			arcWeigtComp.showAttr(null, 1);
			return;
		}
		
		for(AbstractArc arc : arcs){
			switch(type){
				case DISCRETE: 					
					if(!isDifferentArcWeightDiscreteValues(arcs))		
						arcWeigtComp.showAttr(type, (int)Double.parseDouble(arc.getArcWeight().getValue().toString()));
					else
						((JSpinner.DefaultEditor) arcWeigtComp.getSpinner().getEditor()).getTextField().setText(emptyString);
					
					break;					
				case CONTINUOUS: 
					if(!isDifferentArcWeightContinuousValues(arcs))		
						arcWeigtComp.showAttr(type, Double.parseDouble(arc.getArcWeight().getValue().toString()));
					else
						((JSpinner.DefaultEditor) arcWeigtComp.getSpinner().getEditor()).getTextField().setText(emptyString);					
					break;					
				case EMPTY:
					if(!isDifferentArcWeightDiscreteValues(arcs))		
						arcWeigtComp.showAttr(type, Double.parseDouble(arc.getArcWeight().getValue().toString()));
					else
						((JSpinner.DefaultEditor) arcWeigtComp.getSpinner().getEditor()).getTextField().setText(emptyString);					
					break;
			}
		}	
	}
	
	private boolean isDifferentArcWeightDiscreteValues(List<AbstractArc> arcs){
		if(arcs.size()<=1)return false;
						
		int value=(int)Double.parseDouble(arcs.get(0).getArcWeight().getValue().toString());
		for(int i=1; i<arcs.size(); i++){
			if(value!=(int)Double.parseDouble(arcs.get(i).getArcWeight().getValue().toString()))
				return true;
		}
			
		return false;
	}
	
	private boolean isDifferentArcWeightContinuousValues(List<AbstractArc> arcs){
		if(arcs.size()<=1)return false;
						
		double value=Double.parseDouble(arcs.get(0).getArcWeight().getValue().toString());
		for(int i=1; i<arcs.size(); i++){
			if(value!=Double.parseDouble(arcs.get(i).getArcWeight().getValue().toString()))
				return true;
		}
			
		return false;
	}	
}
