/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.gui;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.AttributeHelper;
import org.BackgroundTaskStatusProviderSupportingExternalCall;
import org.graffiti.editor.MainFrame;
import org.graffiti.event.GraphEvent;
import org.graffiti.event.GraphListener;
import org.graffiti.event.TransactionEvent;
import org.graffiti.graph.Graph;
import org.graffiti.graph.Node;
import org.graffiti.selection.SelectionEvent;
import org.graffiti.selection.SelectionListener;

import petri_jfern.Translation;
import vanted.gui.button.PNButton;
import vanted.petrinetelements.NodePN;
import vanted.petrinetelements.NodeShape;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.Transition;
import vanted.petrinetelements.misc.IgnoreNode;
import vanted.validation.Validator;

// TODO: Auto-generated Javadoc
/**
 * The Class ReconstructionGUI creates the whole reconstruction tab.
 *
 * @author Kevin Pucknat
 */
public class ReconstructionGUI extends JComponent implements GraphListener, SelectionListener{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/** The graph. */
	private Graph graph;
	
	/** The trans. */
	private PNButton trans = new PNButton(NodeShape.Transition);
	
	/** The place. */
	private PNButton place = new PNButton(NodeShape.Place);	
	
	/** The pn gui. */
	private PetriNetAttributeGUI pnGUI = new PetriNetAttributeGUI();
	
	/**
	 * Instantiates a new reconstruction gui.
	 */
	public ReconstructionGUI(){
		setLayout(new TableLayout(new double[][] { 
				{TableLayoutConstants.FILL}, 		// Columns
				{TableLayoutConstants.PREFERRED, TableLayoutConstants.PREFERRED, TableLayoutConstants.PREFERRED, 
					TableLayoutConstants.PREFERRED, TableLayoutConstants.PREFERRED, TableLayoutConstants.PREFERRED, 
					TableLayoutConstants.PREFERRED, TableLayoutConstants.PREFERRED } })); 	// Rows
				
		JPanel manualJP = new JPanel();
		manualJP.setLayout(new TableLayout(new double[][] { 
				{TableLayoutConstants.FILL}, {TableLayoutConstants.PREFERRED, TableLayoutConstants.PREFERRED, TableLayoutConstants.PREFERRED, 
					TableLayoutConstants.PREFERRED} }));
		manualJP.setBorder(BorderFactory.createTitledBorder("Manual changes"));
		manualJP.setOpaque(false);	
		manualJP.add(createManualComponent(), "0,0");
		
		add(manualJP, "0,0");		
		add(new JLabel("Validation:"),"0,1");
		add(createValidationBtn(), "0,2");	
		add(new JLabel(" "),"0,3");		
		add(pnGUI, "0,4");		
		
		JPanel completeJP = new JPanel();
		completeJP.setLayout(new TableLayout(new double[][] { 
				{TableLayoutConstants.FILL}, {TableLayoutConstants.PREFERRED, TableLayoutConstants.PREFERRED, TableLayoutConstants.PREFERRED, 
					TableLayoutConstants.PREFERRED} }));
		completeJP.setBorder(BorderFactory.createTitledBorder("Changes complete Petri-Net"));
		completeJP.setOpaque(false);
		completeJP.add(new JLabel(""),"0,0");
		completeJP.add(createSemiAutoComponent(),"0,1");
		
		completeJP.add(new JLabel(" "),"0,2");
		completeJP.add(createAutoComponent(), "0,3");
		add(completeJP,"0,5");				
	}
	
	/**
	 * Creates the semi-automatic transformation component.
	 *
	 * @return the component
	 */
	private JComponent createSemiAutoComponent() {
		JComponent j = new JPanel();
		j.setLayout(new TableLayout(new double[][] { 
				{TableLayoutConstants.FILL}, 		// Columns
				{TableLayoutConstants.PREFERRED, TableLayoutConstants.PREFERRED } })); 	// Rows
		
		
		JButton pBtn = new JButton("<html>Selected nodes are transformed into places<br><small><center>(all unselected into transitions)");
		pBtn.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {
				transformSelection(NodeShape.Place);
			}
		});
		j.add(pBtn, "0,0");

		JButton tBtn = new JButton("<html>Selected nodes are transformed into transitions<br><small><center>(all unselected into places)");
		tBtn.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {
				transformSelection(NodeShape.Transition);						
			}
		});
		j.add(tBtn, "0,1");
				
		MainFrame.getInstance().addSelectionListener(this);
		
		return j;
	}

	/**
	 * Transform selection semi-automatic (either selected nodes into transitions and unselected nodes into places or vice versa).
	 *
	 * @param shape the shape
	 */
	private void transformSelection(NodeShape shape){
		List<Node> allNodes = graph.getNodes();				
		Collection<Node> selectedNodes = MainFrame.getInstance().getActiveEditorSession().getSelectionModel().getActiveSelection().getNodes();
		
		List<Node> unselectedNodes = new ArrayList<Node>();
		for(Node node : allNodes){
			if(!selectedNodes.contains(node))unselectedNodes.add(node);
		}
		
		for(Node node : selectedNodes){
			if(new NodePN(node).getIgnoreNode().getValue())continue;
			AttributeHelper.setShape(node, shape.equals(NodeShape.Place)? NodeShape.Place.getTxt() : NodeShape.Transition.getTxt());
		}
		for(Node node : unselectedNodes){
			if(new NodePN(node).getIgnoreNode().getValue())continue;
			AttributeHelper.setShape(node, shape.equals(NodeShape.Place)? NodeShape.Transition.getTxt(): NodeShape.Place.getTxt());
		}		
	}
	
	/**
	 * Creates the editing component.
	 *
	 * @return the component
	 */
	private JComponent createManualComponent(){
		JComponent j = new JPanel();
		j.setOpaque(false);
		j.setLayout(new TableLayout(new double[][] { 
				{TableLayoutConstants.FILL,TableLayoutConstants.FILL}, 		// Columns
				{TableLayoutConstants.PREFERRED, TableLayoutConstants.PREFERRED } })); 	// Rows
		
		trans.addListener(place);
		j.add(trans, "0,0");
		trans.setSelected(true);
		
		place.addListener(trans);
		j.add(place, "1,0");		
		
		JButton hideBtn = new JButton("<html>Hide node<br>");
		hideBtn.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {				
				Collection<Node> nodes = MainFrame.getInstance().getActiveEditorSession().getSelectionModel().getActiveSelection().getNodes();				
				for(Node node : nodes){
					NodePN nodePN = new NodePN(node);			
					nodePN.addIgnoreNode(new IgnoreNode(true));
				}				
			}
		});
		j.add(hideBtn, "0,1");
		JButton dignoreBtn = new JButton("<html>Remove \"Hide node\"");
		dignoreBtn.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {				
				Collection<Node> nodes = MainFrame.getInstance().getActiveEditorSession().getSelectionModel().getActiveSelection().getNodes();				
				for(Node node : nodes){
					NodePN nodePN = new NodePN(node);					
					nodePN.removeIgnoreNode();				
				}				
			}
		});
		j.add(dignoreBtn, "1,1");
				
		return j;
	}
	
	/**
	 * Creates the validation button.
	 *
	 * @return the button
	 */
	private JButton createValidationBtn(){		
		JButton btn = new JButton("Validation");
		final Validator val = new Validator();		
		btn.addActionListener(new ActionListener() {							
			@Override
			public void actionPerformed(ActionEvent e) {
				if(graph==null)return;
				val.isPNCorrect(graph);		
			}
		});
		return btn;
	}
	
	/**
	 * Creates the automatic transformation component.
	 *
	 * @return the button
	 */
	private JButton createAutoComponent(){
		JButton btn = new JButton("Transforms all Nodes in Places and inserts transitions between the changed places");
		
		btn.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {
				if(graph==null)return;
				Translation translation = new Translation(graph);
				translation.start();			
			}
		});
		
		return btn;
	}
		
	
	/* (non-Javadoc)
	 * @see org.graffiti.event.GraphListener#preNodeAdded(org.graffiti.event.GraphEvent)
	 */
	@Override
	public void preNodeAdded(GraphEvent e) {		
		if(trans.isSelected()){
			if(!Place.isPlace(e.getNode()))
				AttributeHelper.setShape(((Node)e.getNode()), NodeShape.Transition.getTxt());			
		}
		else if(place.isSelected()){			
				AttributeHelper.setShape(((Node)e.getNode()), NodeShape.Place.getTxt());
		}		
	}

	/* (non-Javadoc)
	 * @see org.graffiti.event.GraphListener#postEdgeAdded(org.graffiti.event.GraphEvent)
	 */
	@Override
	public void postEdgeAdded(GraphEvent e) {}

	/* (non-Javadoc)
	 * @see org.graffiti.event.GraphListener#postEdgeRemoved(org.graffiti.event.GraphEvent)
	 */
	@Override
	public void postEdgeRemoved(GraphEvent e) {}

	/* (non-Javadoc)
	 * @see org.graffiti.event.GraphListener#postGraphCleared(org.graffiti.event.GraphEvent)
	 */
	@Override
	public void postGraphCleared(GraphEvent e) {}

	/* (non-Javadoc)
	 * @see org.graffiti.event.GraphListener#postNodeAdded(org.graffiti.event.GraphEvent)
	 */
	@Override
	public void postNodeAdded(GraphEvent e) {}

	/* (non-Javadoc)
	 * @see org.graffiti.event.GraphListener#postNodeRemoved(org.graffiti.event.GraphEvent)
	 */
	@Override
	public void postNodeRemoved(GraphEvent e) {}

	/* (non-Javadoc)
	 * @see org.graffiti.event.GraphListener#preEdgeAdded(org.graffiti.event.GraphEvent)
	 */
	@Override
	public void preEdgeAdded(GraphEvent e) {}

	/* (non-Javadoc)
	 * @see org.graffiti.event.GraphListener#preEdgeRemoved(org.graffiti.event.GraphEvent)
	 */
	@Override
	public void preEdgeRemoved(GraphEvent e) {}

	/* (non-Javadoc)
	 * @see org.graffiti.event.GraphListener#preGraphCleared(org.graffiti.event.GraphEvent)
	 */
	@Override
	public void preGraphCleared(GraphEvent e) {}

	/* (non-Javadoc)
	 * @see org.graffiti.event.GraphListener#preNodeRemoved(org.graffiti.event.GraphEvent)
	 */
	@Override
	public void preNodeRemoved(GraphEvent e) {}

	/* (non-Javadoc)
	 * @see org.graffiti.event.TransactionListener#transactionFinished(org.graffiti.event.TransactionEvent, org.BackgroundTaskStatusProviderSupportingExternalCall)
	 */
	@Override
	public void transactionFinished(TransactionEvent e,
			BackgroundTaskStatusProviderSupportingExternalCall status) {}

	/* (non-Javadoc)
	 * @see org.graffiti.event.TransactionListener#transactionStarted(org.graffiti.event.TransactionEvent)
	 */
	@Override
	public void transactionStarted(TransactionEvent e) {}

	/**
	 * Graph changed.
	 *
	 * @param graph the graph
	 */
	public void graphChanged(Graph graph) {
		this.graph = graph;
		pnGUI.graphChanged(graph);
	}

	/* (non-Javadoc)
	 * @see org.graffiti.selection.SelectionListener#selectionChanged(org.graffiti.selection.SelectionEvent)
	 */
	@Override
	public void selectionChanged(SelectionEvent e) {
		boolean transitionsSelected=false;
		boolean placesSelected=false;
		for(Node node : MainFrame.getInstance().getActiveEditorSession().getSelectionModel().getActiveSelection().getNodes()){
			if(Place.isPlace(node))
				placesSelected=true;
			else if(Transition.isTransition(node))
				transitionsSelected=true;
		}
		if(placesSelected&&transitionsSelected){
			place.setSelected(true);
			trans.setSelected(true);
			return;
		}
		else if(placesSelected){
			place.setSelected(true);
			trans.setSelected(false);
		}
		else if(transitionsSelected){
			trans.setSelected(true);
			place.setSelected(false);
		}
			
	}

	/* (non-Javadoc)
	 * @see org.graffiti.selection.SelectionListener#selectionListChanged(org.graffiti.selection.SelectionEvent)
	 */
	@Override
	public void selectionListChanged(SelectionEvent e) {}
}
