/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.gui;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import org.graffiti.editor.MainFrame;
import org.graffiti.graph.Graph;
import org.graffiti.graph.Node;
import org.graffiti.selection.Selection;
import org.graffiti.selection.SelectionModel;

import simulation.SimulatorListener;
import vanted.ThreadSafeClass;
import vanted.addon.PetriAddonTab;
import vanted.petrinetelements.Library;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Transition;

// TODO: Auto-generated Javadoc
/**
 * The Class SimulationGUI creates the whole simulation tab.
 *
 * @author Kevin Pucknat
 */
public class SimulationGUI extends JComponent implements SimulatorListener{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/** The graph. */
	private Graph graph;
	
	/** The state. */
	private boolean state = true;
	
	/** The run btn. */
	private JButton runBtn;
	
	/** The step btn. */
	private JButton stepBtn;
	
	/** The step js. */
	private JSpinner stepJS;
	
	/** The delay sp. */
	private JSpinner delaySP;
	
	/** The show token flow. */
	private JCheckBox showTokenFlow;
	
	/** The write2 file. */
	private JCheckBox write2File;
	
	/** The fire btn. */
	private JButton fireBtn;
	
	/**
	 * Instantiates a new simulation gui.
	 */
	public SimulationGUI() {
		state = true;
		setLayout(new TableLayout(new double[][] {
				{ TableLayoutConstants.FILL }, // Columns
				{ TableLayoutConstants.PREFERRED,
						TableLayoutConstants.PREFERRED,
						TableLayoutConstants.PREFERRED,
						TableLayoutConstants.PREFERRED } })); // Rows
				
		add(createEnabledComponent(), "0,0");
		add(createTransitionComponent(), "0,1");
		add(createStepComponent(), "0,2");
		add(createRunComponent(), "0,3");
	}
	
	/**
	 * Creates the show all enabled transitions component.
	 *
	 * @return the component
	 */
	private JComponent createEnabledComponent(){
		JPanel enabledJP = new JPanel();
		enabledJP.setLayout(new TableLayout(new double[][] {
				{ TableLayoutConstants.FILL, TableLayoutConstants.FILL }, { TableLayoutConstants.PREFERRED } }));
		enabledJP.setBorder(BorderFactory.createTitledBorder("Enabled Transitions"));
		enabledJP.setOpaque(false);
		enabledJP.add(createSelectAllEnabledTrBtn(), "0,0");
		return enabledJP;
	}
	
	/**
	 * Creates the all selected transition simulation component.
	 *
	 * @return the component
	 */
	private JComponent createTransitionComponent(){
		JPanel simTransitionJP = new JPanel();
		simTransitionJP.setLayout(new TableLayout(new double[][] {
				{ TableLayoutConstants.FILL, TableLayoutConstants.FILL }, { TableLayoutConstants.PREFERRED } }));
		simTransitionJP.setBorder(BorderFactory.createTitledBorder("Simulation Transition"));
		simTransitionJP.setOpaque(false);
		simTransitionJP.add(createFireBtn(), "0,0");
		return simTransitionJP;
	}
	
	/**
	 * Creates the stepwise simulation component.
	 *
	 * @return the component
	 */
	private JComponent createStepComponent(){
		JPanel simStepJP = new JPanel();
		simStepJP.setLayout(new TableLayout(new double[][] {
				{ TableLayoutConstants.FILL, TableLayoutConstants.FILL }, { TableLayoutConstants.PREFERRED, TableLayoutConstants.PREFERRED,
						TableLayoutConstants.PREFERRED, TableLayoutConstants.PREFERRED } }));
		simStepJP.setBorder(BorderFactory.createTitledBorder("Simulation Step"));
		simStepJP.setOpaque(false);
		
		stepBtn = new JButton("Fire all for ... steps");
		

		final ThreadSafeClass tsc = new ThreadSafeClass();
		stepBtn.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {
				if (graph == null)
					return;
				if (state) {
					if (PetriAddonTab.isReachabilityGraph(graph))
						return;
					
					tsc.setObj(new PetriNet());
					if (! ((PetriNet)tsc.getObj()).createPetriNet(graph, true)) {
						tsc.setObj(null);
						return;
					}
					write2File.setEnabled(false);
					runBtn.setEnabled(false);
					fireBtn.setEnabled(false);
					stepBtn.setText("Stop");					
					Library lib = perform(((PetriNet)tsc.getObj()));
					lib.showTokenFlow(showTokenFlow.isSelected());
					lib.write2File(write2File.isSelected());
					((PetriNet)tsc.getObj()).run((Integer) stepJS.getValue());
				} else {
					stepBtn.setText("Fire all for ... steps");
					state = true;
					write2File.setEnabled(true);
					runBtn.setEnabled(true);
					fireBtn.setEnabled(true);
					if (((PetriNet)tsc.getObj()) != null)
						((PetriNet)tsc.getObj()).stop();
				}
			}
		});
		
		simStepJP.add(stepBtn, "0,1");
		stepJS = new JSpinner(new SpinnerNumberModel(10, 1,
				Integer.MAX_VALUE, 1));
		simStepJP.add(stepJS, "1,1");
		final JLabel delay = new JLabel("Highlight firing transitions (ms)");
		delaySP = new JSpinner(new SpinnerNumberModel(0, 0, 10000, 500));
		showTokenFlow = new JCheckBox("Show token flow");
		showTokenFlow.setSelected(true);
		showTokenFlow.setOpaque(false);
		showTokenFlow.addActionListener(new ActionListener() {		
			@Override
			public void actionPerformed(ActionEvent e) {
				if(((PetriNet)tsc.getObj())!=null)
					if(((PetriNet)tsc.getObj()).getLibrary()!=null)
						((PetriNet)tsc.getObj()).getLibrary().showTokenFlow(showTokenFlow.isSelected());
				
				delay.setEnabled(showTokenFlow.isSelected());
				delaySP.setEnabled(showTokenFlow.isSelected());
			}
		});
		
		write2File = new JCheckBox("Write Marking to file");
		write2File.setSelected(false);
		write2File.setOpaque(false);
				
		simStepJP.add(showTokenFlow, "0,2");
		simStepJP.add(write2File, "1,2");
		simStepJP.add(delay, "0,3");
		simStepJP.add(delaySP, "1,3");
		return simStepJP;
	}
	
	/**
	 * Creates the infinity run component.
	 *
	 * @return the component
	 */
	private JComponent createRunComponent(){
		JPanel simRunJP = new JPanel();
		simRunJP.setLayout(new TableLayout(new double[][] {
				{ TableLayoutConstants.FILL, TableLayoutConstants.FILL }, { TableLayoutConstants.PREFERRED } }));
		simRunJP.setBorder(BorderFactory.createTitledBorder("Simulation Run"));
		simRunJP.setOpaque(false);
		simRunJP.add(createRunBtn(), "0,0");
		return simRunJP;
	}
	
	/**
	 * Creates the select all enabled transitions button.
	 *
	 * @return the button
	 */
	private JButton createSelectAllEnabledTrBtn() {
		JButton btn = new JButton("Select all enabled Transitions");
		btn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {				
				PetriNet pn = createPetriNet(true); 
				if(pn==null)return;				
				
				Library lib = pn.parsePetriNet2Lib();
				
				boolean enabledTr[] = new boolean[pn.getAllTransitions().size()];
				enabledTr = pn.getEnabledTransitionsFromMarking(
						lib.getMarking(), PetriNet.TRANSITION_RULE);
				
				Collection<Node> col = new ArrayList<Node>();
				for (int i = 0; i < pn.getAllTransitions().size(); i++) {
					if (enabledTr[i])
						col.add(pn.getAllTransitions().get(i).getNode());
				}
				
				SelectionModel sm = MainFrame.getInstance()
						.getActiveEditorSession().getSelectionModel();
				sm.setActiveSelection(new Selection("enabledtransitions", col));
				sm.selectionChanged();
			}
		});
		return btn;
	}

	/**
	 * Creates the Petri net.
	 *
	 * @param checkReachabilitygraph, true when the graph is a reachabilityGraph
	 * @return the Petri net
	 */
	private PetriNet createPetriNet(boolean checkReachabilitygraph){
		if (graph == null)
			return null;		

		if(checkReachabilitygraph)
			if (PetriAddonTab.isReachabilityGraph(graph))
				return null;
		
		PetriNet pn = new PetriNet();									
		if (!pn.createPetriNet(graph, true)) {
			pn = null;
			return null;
		}					
		return pn;
	}
		
	/**
	 * Creates the fire all selected transitions button.
	 *
	 * @return the button
	 */
	private JButton createFireBtn() {
		fireBtn = new JButton("<html>Fire all selected Transitions<br>");
		fireBtn.setToolTipText("only enabled transitions are fired");
		fireBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				PetriNet pn = createPetriNet(true); 
				if(pn==null)return;
				
				runBtn.setEnabled(false);
				stepBtn.setEnabled(false);
				write2File.setEnabled(false);
				
				boolean transitionSelected = false;
				Collection<Node> nodes = MainFrame.getInstance()
						.getActiveEditorSession().getSelectionModel()
						.getActiveSelection().getNodes();
				if (nodes.isEmpty())
					nodes = graph.getNodes();
				
				for (Node node : nodes) {
					if (Transition.isTransition(node)) {
						transitionSelected = true;
						break;
					}
				}
				if (!transitionSelected) 
					nodes = graph.getNodes();
				
				
				List<Transition> transitions = pn
						.getAllSelectedTransitions(nodes);
				
				Library lib = pn.parsePetriNet2Lib();
				lib.fireTransitions(transitions);
				
				runBtn.setEnabled(true);
				stepBtn.setEnabled(true);
				write2File.setEnabled(true);
			}
		});
		return fireBtn;
	}
		
	/**
	 * Creates the infinity run button.
	 *
	 * @return the button
	 */
	private JButton createRunBtn() {
		runBtn = new JButton("Infinity run");
		runBtn.addActionListener(new ActionListener() {
			PetriNet pn = null;			
			@Override
			public void actionPerformed(ActionEvent e) {
				if (state) {
					pn = createPetriNet(true); 
					if(pn==null)return;
					
					write2File.setEnabled(false);
					stepBtn.setEnabled(false);
					
					runBtn.setText("Stop");
					Library lib = perform(pn);
					lib.showTokenFlow(false);
					lib.write2File(false);
					pn.run();
				} else {
					runBtn.setText("Infinity run");
					state = true;
					write2File.setEnabled(true);
					stepBtn.setEnabled(true);
					if (pn != null)
						pn.stop();
				}
			}
		});
		return runBtn;
	}
	
	/**
	 * Parses the Petri net to a library to perform the simulation.
	 *
	 * @param pn the pn
	 * @return the library
	 */
	private Library perform(PetriNet pn) {
		state = false;
		Library lib = pn.parsePetriNet2Lib();
		lib.getSimulator().addListener(this);
		lib.highlightEnabledTransitionsDelay(Integer.parseInt(delaySP.getValue().toString()));		
		return lib;		
	}
	
	/* (non-Javadoc)
	 * @see simulation.SimulatorListener#step(boolean)
	 */
	@Override
	public void step(boolean lastStep) {
		if (lastStep) {
			state = true;
			if (runBtn.getText().equals("Stop")){
				runBtn.setText("Infinity run");
				write2File.setEnabled(true);
				stepBtn.setEnabled(true);
				fireBtn.setEnabled(true);
			}
			if (stepBtn.getText().equals("Stop")){
				stepBtn.setText("Fire all for ... steps");
				write2File.setEnabled(true);
				runBtn.setEnabled(true);
				fireBtn.setEnabled(true);
			}
		}
	}
	
	/**
	 * Graph changed.
	 *
	 * @param graph the graph
	 */
	public void graphChanged(Graph graph) {
		this.graph = graph;
	}	
}
