/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.petrinetelements;

import java.util.List;

import org.AttributeHelper;
import org.graffiti.graph.Edge;

import vanted.attribute.cpn.guard.GuardArcAttribute;
import vanted.attribute.token.TokenAttributeDiscrete;
import vanted.petrinetelements.cpn.Guard;

// TODO: Auto-generated Javadoc
/**
 * The Class InArc for arc handling, where the arc is an ingoing arc of a transition.
 *
 * @author Kevin Pucknat
 */
public class InArc extends AbstractArc{

	/**
	 * Instantiates a new in arc.
	 *
	 * @param place the place
	 * @param transition the transition
	 * @param edge the edge
	 */
	public InArc(Place place, Transition transition, Edge edge) {
		super(place, transition, edge);
		place.addOutArc(this);
		transition.addInArc(this);
	}
	
	/**
	 * Checks if the arc is an ingoing arc.
	 *
	 * @return true, if the arc is an ingoing arc
	 */
	public boolean isInArc(){
		return true;
	}
	
	
	/**
	 * Gets the guard arc attribute of the ingoing arc.
	 *
	 * @return the guard arc attribute
	 */
	private GuardArcAttribute getGuardArcAttribute(){		
		GuardArcAttribute ga;
		if(!AttributeHelper.hasAttribute(edges.get(0), GuardArcAttribute.path, GuardArcAttribute.name)){
			ga = (GuardArcAttribute) AttributeHelper.getAttributeValue(edges.get(0), GuardArcAttribute.path, GuardArcAttribute.name, new GuardArcAttribute(
					GuardArcAttribute.name), new GuardArcAttribute(TokenAttributeDiscrete.name), true);
		}else{			
			ga = (GuardArcAttribute)AttributeHelper.getAttribute(edges.get(0), GuardArcAttribute.path+"."+GuardArcAttribute.name);	
		}		
		return ga;
	}
	
	/**
	 * Adds the guard to the ingoing arc.
	 *
	 * @param guard the guard
	 */
	public void addGuard(Guard guard) {		
		GuardArcAttribute ga = getGuardArcAttribute();		
		ga.addGuard(guard);
	}
	
	/**
	 * Removes the guard from the ingoing arc.
	 *
	 * @param guard the guard
	 */
	public void removeGuard(Guard guard) {		
		GuardArcAttribute ga = getGuardArcAttribute();		
		ga.removeGuard(guard);
	}
	
	/**
	 * Removes the all guards from the ingoing arc.
	 */
	public void removeAllGuards(){
		GuardArcAttribute ga = getGuardArcAttribute();
		ga.removeAllGuards();
	}
	
	/**
	 * Gets the guards of the ingoing arc.
	 *
	 * @return the guards
	 */
	public List<Guard> getGuards() {
		GuardArcAttribute ga = getGuardArcAttribute();
		return ga.getGuards();
	}	
}
