/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.petrinetelements;

import java.util.ArrayList;
import java.util.List;

import org.AttributeHelper;
import org.graffiti.graph.Node;

import vanted.analysis.Reachability;
import vanted.attribute.DrawingNodeAttribute;
import vanted.attribute.IdentityAttribute;
import vanted.attribute.IgnoreNodeAttribute;
import vanted.attribute.ReachabilityAttribute;
import vanted.petrinetelements.misc.DrawingNode;
import vanted.petrinetelements.misc.Identity;
import vanted.petrinetelements.misc.IgnoreNode;


// TODO: Auto-generated Javadoc
/**
 * The Class NodePN for node handling.
 *
 * @author Kevin Pucknat
 */
public class NodePN {
	
	/** The nodes. */
	protected List<Node> nodes = new ArrayList<Node>();
	
	/**
	 * Instantiates a new node pn.
	 *
	 * @param node the node
	 */
	public NodePN(Node node) {
		nodes.add(node);
	}

	/**
	 * Gets the node.
	 *
	 * @return the node
	 */
	public Node getNode(){
		return nodes.get(0);
	}
	
	/**
	 * Gets the nodes.
	 *
	 * @return the nodes
	 */
	public List<Node> getNodes(){
		return new ArrayList<Node>(nodes);
	}
	
	/**
	 * Adds the node and the attributes: Draw, IgnoreNode and Reachability.
	 *
	 * @param node the node
	 */
	public void addNode(Node node){
		nodes.add(node);
		NodePN temp = new NodePN(node);
		
		if(getDrawingNodeAttribute()!=null)
			temp.addDraw(getDrawingNodeAttribute().getDraw());
		if(getIgnoreNodeAttribute()!=null)
			temp.addIgnoreNode(getIgnoreNodeAttribute().getIgnoreNode());
		if(getReachabilityAttribute()!=null)
			temp.addReachability(getReachabilityAttribute().getState());
	}
	
	/**
	 * Removes the node and the attributes: Draw, IgnoreNode and Reachability.
	 *
	 * @param node the node
	 * @return true, if successful
	 */
	public boolean removeNode(Node node){
		NodePN temp = new NodePN(node);
		temp.removeDraw();
		temp.removeIgnoreNode();
		temp.removeReachability();
		
		return nodes.remove(node);
	}
	
	/**
	 * Gets the node id.
	 *
	 * @return the id
	 */
	public long getID() {
		return nodes.get(0).getID();
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return nodes.get(0).toString();
	}
	
	/**
	 * Gets the node label or the node id.
	 *
	 * @return the name
	 */
	public String getName(){
		if(AttributeHelper.getLabel(nodes.get(0), "").isEmpty())
			return nodes.get(0).getID()+"";
		else
			return AttributeHelper.getLabel(nodes.get(0), "");
	}
	
////ignore node
	/**
 * Gets the IgnoreNodeAttribute which contains the object IgnoreNode.
 *
 * @return the ignore node attribute
 */
private IgnoreNodeAttribute getIgnoreNodeAttribute(){
		if(!AttributeHelper.hasAttribute(nodes.get(0), IgnoreNodeAttribute.path, IgnoreNodeAttribute.name))
			return null;
		else		
		
			return (IgnoreNodeAttribute)AttributeHelper.getAttribute(nodes.get(0), IgnoreNodeAttribute.path+"."+IgnoreNodeAttribute.name);
	}
		
	/**
	 * Adds the object IgnoreNode to the IgnoreNodeAttribute.
	 *
	 * @param ignoreNode the ignore node
	 */
	public void addIgnoreNode(IgnoreNode ignoreNode) {		
		IgnoreNodeAttribute ica = getIgnoreNodeAttribute();
		if(ica==null)			
			ica = createIgnoreNodeAttribute();
		ica.addIgnoreNode(ignoreNode);
		
		if(nodes.size()>1){
			for(int i=1; i<nodes.size(); i++){
				Place temp = new Place(nodes.get(i));					
				temp.addIgnoreNode(ignoreNode);
			}
		}
	}
	
	/**
	 * Creates the IgnoreNodeAttribute which contains the object IgnoreNode.
	 *
	 * @return the ignore node attribute
	 */
	private IgnoreNodeAttribute createIgnoreNodeAttribute() {
		return (IgnoreNodeAttribute) AttributeHelper.getAttributeValue(nodes.get(0), IgnoreNodeAttribute.path, IgnoreNodeAttribute.name, new IgnoreNodeAttribute(
				IgnoreNodeAttribute.name), new IgnoreNodeAttribute(IgnoreNodeAttribute.name), true);
	}
	
	/**
	 * Removes the object IgnoreNode which contains true (hiding the node), or false and the IgnoreNodeAttribute.
	 */
	public void removeIgnoreNode() {		
		IgnoreNodeAttribute ica = getIgnoreNodeAttribute();		
		if(ica!=null){
			ica.removeIgnoreNode();
			AttributeHelper.deleteAttribute(nodes.get(0), IgnoreNodeAttribute.path, IgnoreNodeAttribute.name);
			
			if(nodes.size()>1){
				for(int i=1; i<nodes.size(); i++){
					Place temp = new Place(nodes.get(i));					
					temp.removeIgnoreNode();
				}
			}
		}
	}
	
	/**
	 * Gets the object IgnoreNode which contains true (hiding the node), or false.
	 *
	 * @return the ignore node
	 */
	public IgnoreNode getIgnoreNode(){
		IgnoreNodeAttribute ica = getIgnoreNodeAttribute();
		if(ica==null)
			return new IgnoreNode(false);
		return ica.getIgnoreNode();
	}
	
////drawing attribute
	/**
 * Gets the DrawingNodeAttribute which contains the object DrawingNode.
 *
 * @return the drawing node attribute
 */
private DrawingNodeAttribute getDrawingNodeAttribute(){	
		if(!AttributeHelper.hasAttribute(nodes.get(0), DrawingNodeAttribute.path, DrawingNodeAttribute.name))
			return null;
		else	
			return (DrawingNodeAttribute)AttributeHelper.getAttribute(nodes.get(0), DrawingNodeAttribute.path+"."+DrawingNodeAttribute.name);	
	}
		
	/**
	 * Adds the object DrawingNode to the DrawingNodeAttribute.
	 *
	 * @param draw the draw
	 */
	public void addDraw(DrawingNode draw) {		
		DrawingNodeAttribute da = getDrawingNodeAttribute();
		if(da==null)			
			da = createDrawingNodeAttribute();
		da.addDraw(draw);
		
		if(nodes.size()>1){
			for(int i=1; i<nodes.size(); i++){
				Place temp = new Place(nodes.get(i));					
				temp.addDraw(draw);
			}
		}
	}
	
	/**
	 * Creates the DrawingNodeAttribute which contains the object DrawingNode.
	 *
	 * @return the drawing node attribute
	 */
	private DrawingNodeAttribute createDrawingNodeAttribute() {
		return (DrawingNodeAttribute) AttributeHelper.getAttributeValue(nodes.get(0), DrawingNodeAttribute.path, DrawingNodeAttribute.name, new DrawingNodeAttribute(
				DrawingNodeAttribute.name), new DrawingNodeAttribute(DrawingNodeAttribute.name), true);
	}
	
	/**
	 * Removes the object DrawingNode which contains a double with its marking or -1 for omega (coverbilityGraph) or a constant (check for Petri net drawing) and the DrawingNodeAttribute.
	 */
	public void removeDraw() {		
		DrawingNodeAttribute da = getDrawingNodeAttribute();		
		if(da!=null){
			da.removeDraw();
			AttributeHelper.deleteAttribute(nodes.get(0), DrawingNodeAttribute.path, DrawingNodeAttribute.name);
			
			if(nodes.size()>1){
				for(int i=1; i<nodes.size(); i++){
					Place temp = new Place(nodes.get(i));					
					temp.removeDraw();
				}
			}
		}
	}
	
	/**
	 * Gets the object DrawingNode which contains a double with its marking or -1 for omega (coverbilityGraph) or a constant (check for Petri net drawing).
	 *
	 * @return the draw
	 */
	public DrawingNode getDraw(){
		DrawingNodeAttribute da = getDrawingNodeAttribute();
		if(da==null)
			return new DrawingNode(0);
		return da.getDraw();		
	}
	
////reachability attribute
	/**
 * Gets the ReachabilityAttribute which contains the object Reachability.
 *
 * @return the reachability attribute
 */
private ReachabilityAttribute getReachabilityAttribute(){	
		if(!AttributeHelper.hasAttribute(nodes.get(0), ReachabilityAttribute.path, ReachabilityAttribute.name))
			return null;
		else		
			return (ReachabilityAttribute)AttributeHelper.getAttribute(nodes.get(0), ReachabilityAttribute.path+"."+ReachabilityAttribute.name);
	}
		
	/**
	 * Adds the object Reachability to the ReachabilityAttribute.
	 *
	 * @param reachability the reachability
	 */
	public void addReachability(Reachability reachability) {		
		ReachabilityAttribute da = getReachabilityAttribute();
		if(da==null)			
			da = createReachabilityAttribute();
		da.addState(reachability);
		
		if(nodes.size()>1){
			for(int i=1; i<nodes.size(); i++){
				Place temp = new Place(nodes.get(i));					
				temp.addReachability(reachability);
			}
		}
	}
	
	/**
	 * Creates the ReachabilityAttribute which contains the object Reachability.
	 *
	 * @return the reachability attribute
	 */
	private ReachabilityAttribute createReachabilityAttribute() {
		return (ReachabilityAttribute) AttributeHelper.getAttributeValue(nodes.get(0), ReachabilityAttribute.path, ReachabilityAttribute.name, new ReachabilityAttribute(
				ReachabilityAttribute.name), new ReachabilityAttribute(ReachabilityAttribute.name), true);
	}
	
	/**
	 * Removes the object Reachability which contains a String with its marking and the ReachabilityAttribute.
	 */
	public void removeReachability() {		
		ReachabilityAttribute da = getReachabilityAttribute();		
		if(da!=null){
			da.removeState();
			AttributeHelper.deleteAttribute(nodes.get(0), ReachabilityAttribute.path, ReachabilityAttribute.name);
			
			if(nodes.size()>1){
				for(int i=1; i<nodes.size(); i++){
					Place temp = new Place(nodes.get(i));					
					temp.removeReachability();
				}
			}
		}
	}
	
	/**
	 * Gets the object Reachability which contains a String with its marking.
	 *
	 * @return the reachability
	 */
	public Reachability getReachability(){
		ReachabilityAttribute da = getReachabilityAttribute();
		if(da==null)
			return new Reachability("");
		return da.getState();
	}
	
////identity attribute
	/**
	* Gets the IdentityAttribute which contains the object Identity.
 	*
 	* @return the identity attribute
 	*/
	private IdentityAttribute getIdentityAttribute(){	
		if(!AttributeHelper.hasAttribute(nodes.get(0), IdentityAttribute.path, IdentityAttribute.name))
			return null;
		else
			return (IdentityAttribute)AttributeHelper.getAttribute(nodes.get(0), IdentityAttribute.path+"."+IdentityAttribute.name);		
	}
		
	/**
	 * Adds the object Identity to the IdentityAttribute.
	 *
	 * @param identity the identity
	 */
	public void addIdentity(Identity identity) {		
		IdentityAttribute a = getIdentityAttribute();
		if(a==null)			
			a = createIdentityAttribute();
		a.setIdentity(identity);
	}
	
	/**
	 * Creates the IdentityAttribute which contains the object Identity.
	 *
	 * @return the identity attribute
	 */
	private IdentityAttribute createIdentityAttribute() {
		return (IdentityAttribute) AttributeHelper.getAttributeValue(nodes.get(0), IdentityAttribute.path, IdentityAttribute.name, new IdentityAttribute(
				IdentityAttribute.name), new IdentityAttribute(IdentityAttribute.name), true);
	}
	
	/**
	 * Removes the object Identity which contains a random value for linking between nodes (Petri net) and edges (reachability graph) and the IdentityAttribute.
	 */
	public void removeIdentity() {		
		IdentityAttribute a = getIdentityAttribute();		
		if(a!=null){
			a.removeIdentity();
			AttributeHelper.deleteAttribute(nodes.get(0), IdentityAttribute.path, IdentityAttribute.name);
		}
	}
	
	/**
	 * Gets the object Identity which contains a random value for linking between nodes (Petri net) and edges (reachability graph). 
	 *
	 * @return the identity
	 */
	public Identity getIdentity(){
		IdentityAttribute a = getIdentityAttribute();
		if(a==null)
			return new Identity(0);
		return a.getIdentity();
	}


}


