/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.petrinetelements;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

import org.AttributeHelper;
import org.ErrorMsg;
import org.color.ColorUtil;
import org.graffiti.attributes.Attribute;
import org.graffiti.graph.Node;
import org.graffiti.graphics.GraphicAttributeConstants;

import vanted.attribute.CapacityAttribute;
import vanted.attribute.token.TokenAttributeContinuous;
import vanted.attribute.token.TokenAttributeDiscrete;

// TODO: Auto-generated Javadoc
/**
 * The Class Place for place handling.
 *
 * @author Kevin Pucknat
 */
public class Place extends NodePN {
	
	/** The out arcs. */
	private List<InArc> outArcs = new ArrayList<InArc>();
	
	/** The in arcs. */
	private List<OutArc> inArcs = new ArrayList<OutArc>();
	
	/**
	 * Instantiates a new place.
	 *
	 * @param node the node
	 */
	public Place(Node node) {
		super(node);
		if (!isPlace(node))
			ErrorMsg.addErrorMessage("Node is no place");
	}
	
	/* (non-Javadoc)
	 * @see vanted.petrinetelements.NodePN#addNode(org.graffiti.graph.Node)
	 */
	@Override
	public void addNode(Node node) {
		super.addNode(node);
	}
	
	/* (non-Javadoc)
	 * @see vanted.petrinetelements.NodePN#removeNode(org.graffiti.graph.Node)
	 */
	@Override
	public boolean removeNode(Node node) {
		Place temp = new Place(node);
		temp.removeAllToken();
		temp.removeCapacity();
		return super.removeNode(node);
	}
	
	/**
	 * Gets the token attribute discrete of a place.
	 *
	 * @return the token attribute discrete
	 */
	private TokenAttributeDiscrete getTokenAttributeDiscrete() {
		if (!AttributeHelper.hasAttribute(nodes.get(0), TokenAttributeDiscrete.path,
				TokenAttributeDiscrete.name))
			return null;
		else
			return (TokenAttributeDiscrete) AttributeHelper.getAttribute(nodes.get(0),
					TokenAttributeDiscrete.path + "."
							+ TokenAttributeDiscrete.name);
	}
	
	/**
	 * Creates the token attribute discrete of a place.
	 *
	 * @return the token attribute discrete
	 */
	private TokenAttributeDiscrete createTokenAttributeDiscrete() {
		TokenAttributeDiscrete a = null;
		int i = 0;
		for (Node node : nodes) {
			AttributeHelper.getAttributeValue(node, TokenAttributeDiscrete.path,
					TokenAttributeDiscrete.sizeAttributeName, 18, 18, true);
			AttributeHelper.getAttributeValue(node, TokenAttributeDiscrete.path,
					TokenAttributeDiscrete.colorAttributeName,
					ColorUtil.getHexFromColor(Color.RED), "", true);
			AttributeHelper.getAttributeValue(node, TokenAttributeDiscrete.path,
					TokenAttributeDiscrete.positionAttributeName,
					GraphicAttributeConstants.CENTERED, "", true);
			if (i == 0)
				a = (TokenAttributeDiscrete) AttributeHelper.getAttributeValue(node,
						TokenAttributeDiscrete.path, TokenAttributeDiscrete.name,
						new TokenAttributeDiscrete(TokenAttributeDiscrete.name),
						new TokenAttributeDiscrete(TokenAttributeDiscrete.name), true);
			i++;
		}
		return a;
	}
	
	/**
	 * Gets the token attribute continuous of a place.
	 *
	 * @return the token attribute continuous
	 */
	private TokenAttributeContinuous getTokenAttributeContinuous() {
		if (!AttributeHelper.hasAttribute(nodes.get(0), TokenAttributeContinuous.path,
				TokenAttributeContinuous.name))
			return null;
		else
			return (TokenAttributeContinuous) AttributeHelper.getAttribute(
					nodes.get(0), TokenAttributeContinuous.path + "."
							+ TokenAttributeContinuous.name);
	}
	
	/**
	 * Creates the token attribute continuous of a place.
	 *
	 * @return the token attribute continuous
	 */
	private TokenAttributeContinuous createTokenAttributeContinuous() {
		TokenAttributeContinuous a = null;
		int i = 0;
		for (Node node : nodes) {
			AttributeHelper.getAttributeValue(node, TokenAttributeContinuous.path,
					TokenAttributeContinuous.sizeAttributeName, 18, 18, true);
			AttributeHelper.getAttributeValue(node, TokenAttributeContinuous.path,
					TokenAttributeContinuous.colorAttributeName,
					ColorUtil.getHexFromColor(Color.RED), "", true);
			AttributeHelper.getAttributeValue(node, TokenAttributeContinuous.path,
					TokenAttributeContinuous.positionAttributeName,
					GraphicAttributeConstants.CENTERED, "", true);
			if (i == 0)
				a = (TokenAttributeContinuous) AttributeHelper.getAttributeValue(
						node, TokenAttributeContinuous.path,
						TokenAttributeContinuous.name, new TokenAttributeContinuous(
								TokenAttributeContinuous.name),
						new TokenAttributeContinuous(TokenAttributeContinuous.name),
						true);
			i++;
		}
		return a;
	}
	
	/**
	 * Adds the token of a place (if the place is a logical place then this token of the associated places is also added).
	 *
	 * @param token the token
	 */
	public void addToken(AbstractToken token) {
		if (token instanceof TokenDiscrete) {
			TokenAttributeDiscrete ta = getTokenAttributeDiscrete();
			if (ta == null)
				ta = createTokenAttributeDiscrete();
			ta.addToken(token);
			
			if (nodes.size() > 1) {
				for (int i = 1; i < nodes.size(); i++) {
					Place temp = new Place(nodes.get(i));
					temp.addToken(token);
				}
			}
		}
		if (token instanceof TokenContinuous) {
			TokenAttributeContinuous ta = getTokenAttributeContinuous();
			if (ta == null)
				ta = createTokenAttributeContinuous();
			ta.addToken(token);
			
			if (nodes.size() > 1) {
				for (int i = 1; i < nodes.size(); i++) {
					Place temp = new Place(nodes.get(i));
					temp.addToken(token);
				}
			}
		}
	}
	
	/**
	 * Removes the token of a place (if the place is a logical place then this token of the associated places is also removed).
	 *
	 * @param token the token
	 */
	public void removeToken(AbstractToken token) {
		if (token instanceof TokenDiscrete) {
			TokenAttributeDiscrete ta = getTokenAttributeDiscrete();
			if (ta == null)
				ta = createTokenAttributeDiscrete();
			ta.removeToken(token);
			
			if (nodes.size() > 1) {
				for (int i = 1; i < nodes.size(); i++) {
					Place temp = new Place(nodes.get(i));
					temp.removeToken(token);
				}
			}
		}
		if (token instanceof TokenContinuous) {
			TokenAttributeContinuous ta = getTokenAttributeContinuous();
			if (ta == null)
				ta = createTokenAttributeContinuous();
			ta.removeToken(token);
			
			if (nodes.size() > 1) {
				for (int i = 1; i < nodes.size(); i++) {
					Place temp = new Place(nodes.get(i));
					temp.removeToken(token);
				}
			}
		}
	}
	
	/**
	 * Removes the all token of a place (if the place is a logical place then all tokens of the associated places are also removed).
	 */
	public void removeAllToken() {
		TokenAttributeDiscrete tai = getTokenAttributeDiscrete();
		if (tai == null)
			tai = createTokenAttributeDiscrete();
		
		tai.removeAllTokens();
		for (Node node : nodes)
			AttributeHelper.deleteAttribute(node, TokenAttributeDiscrete.path,
					TokenAttributeDiscrete.name);
		
		TokenAttributeContinuous tad = getTokenAttributeContinuous();
		if (tad == null)
			tad = createTokenAttributeContinuous();
		tad.removeAllTokens();
		
		if (nodes.size() > 1) {
			for (int i = 1; i < nodes.size(); i++) {
				Place temp = new Place(nodes.get(i));
				temp.removeAllToken();
			}
		}
	}
	
	/**
	 * Gets the tokens of a place.
	 *
	 * @return the tokens
	 */
	public List<AbstractToken> getTokens() {
		List<AbstractToken> allToken = new ArrayList<AbstractToken>();
		TokenAttributeDiscrete tai = getTokenAttributeDiscrete();
		if (tai != null) {
			if (tai.getTokens() != null && tai.getTokens().size() > 0)
				allToken.addAll(tai.getTokens());
		}
		
		TokenAttributeContinuous tad = getTokenAttributeContinuous();
		if (tad != null)
			if (tad.getTokens() != null && tad.getTokens().size() > 0)
				allToken.addAll(tad.getTokens());
		
		return allToken;
	}
	
	/**
	 * Gets the discrete tokens of a place.
	 *
	 * @return the tokens
	 */
	public List<AbstractToken> getTokensDiscrete() {
		List<AbstractToken> tokenDisc = new ArrayList<AbstractToken>();
		TokenAttributeDiscrete tai = getTokenAttributeDiscrete();
		if (tai != null) 
			if (tai.getTokens() != null && tai.getTokens().size() > 0)
				for(AbstractToken token : tai.getTokens())
					if(token instanceof TokenDiscrete)tokenDisc.add((TokenDiscrete)token);	
		
		return tokenDisc;
	}
	
	/**
	 * Gets the continuous tokens of a place.
	 *
	 * @return the tokens
	 */
	public List<TokenContinuous> getTokensContinuous() {
		List<TokenContinuous> tokenConti = new ArrayList<TokenContinuous>();
		TokenAttributeContinuous tad = getTokenAttributeContinuous();
		if (tad != null)
			if (tad.getTokens() != null && tad.getTokens().size() > 0)
				for(AbstractToken token : tad.getTokens())
					if(token instanceof TokenContinuous) tokenConti.add((TokenContinuous)token);			
		
		return tokenConti;
	}
	
	/**
	 * Checks if the node is a place.
	 *
	 * @param nd the nd
	 * @return true, if is place
	 */
	public static boolean isPlace(Node nd) {
		return NodeShape.isPlace(AttributeHelper.getShape(nd).toLowerCase());
	}
	
	/**
	 * Adds the out arc of a place.
	 *
	 * @param arc the arc
	 */
	public void addOutArc(InArc arc) {
		outArcs.add(arc);
	}
	
	/**
	 * Sets the in arcs of a place.
	 *
	 * @param inArcs the new in arcs
	 */
	public void setInArcs(List<InArc> inArcs) {
		this.outArcs = inArcs;
	}
	
	/**
	 * Gets the out arcs of a place.
	 *
	 * @return the out arcs
	 */
	public List<InArc> getOutArcs() {
		return outArcs;
	}
	
	/**
	 * Adds the in arc of a place.
	 *
	 * @param arc the arc
	 */
	public void addInArc(OutArc arc) {
		inArcs.add(arc);
	}
	
	/**
	 * Sets the out arcs of a place.
	 *
	 * @param outArcs the new out arcs
	 */
	public void setOutArcs(List<OutArc> outArcs) {
		this.inArcs = outArcs;
	}
	
	/**
	 * Gets the in arcs of a place.
	 *
	 * @return the in arcs
	 */
	public List<OutArc> getInArcs() {
		return inArcs;
	}
	
	/**
	 * Gets the arcs of a place.
	 *
	 * @return the arcs
	 */
	public List<AbstractArc> getArcs() {
		List<AbstractArc> temp = new ArrayList<AbstractArc>();
		temp.addAll(outArcs);
		temp.addAll(inArcs);
		return temp;
	}
	
	/**
	 * Gets the capacity attribute of a place.
	 *
	 * @return the capacity attribute
	 */
	private CapacityAttribute getCapacityAttribute() {
		if (!AttributeHelper.hasAttribute(nodes.get(0), CapacityAttribute.path,
				CapacityAttribute.name))
			return null;
		else
			return (CapacityAttribute) AttributeHelper.getAttribute(nodes.get(0),
					CapacityAttribute.path + Attribute.SEPARATOR
							+ CapacityAttribute.name);
	}
	
	/**
	 * Creates the capacity attribute of a place.
	 *
	 * @return the capacity attribute
	 */
	private CapacityAttribute createCapacityAttribute() {
		CapacityAttribute a = null;
		int i = 0;
		for (Node node : nodes) {
			AttributeHelper.getAttributeValue(node, CapacityAttribute.path,
					CapacityAttribute.sizeAttributeName, 18, 18, true);
			AttributeHelper.getAttributeValue(node, CapacityAttribute.path,
					CapacityAttribute.colorAttributeName,
					ColorUtil.getHexFromColor(Color.GREEN), "", true);
			AttributeHelper.getAttributeValue(node, CapacityAttribute.path,
					CapacityAttribute.positionAttributeName,
					GraphicAttributeConstants.BORDER_TOP_RIGHT, "", true);
			if (i == 0)
				a = (CapacityAttribute) AttributeHelper.getAttributeValue(node,
						CapacityAttribute.path, CapacityAttribute.name,
						new CapacityAttribute(CapacityAttribute.name),
						new CapacityAttribute(CapacityAttribute.name), true);
			i++;
		}
		return a;
	}
	
	/**
	 * Adds the capacity of a place (if the place is a logical place then the capacities of the associated places are also added).
	 *
	 * @param capacity the capacity
	 */
	public void addCapacity(Capacity capacity) {
		CapacityAttribute ca = getCapacityAttribute();
		if (ca == null)
			ca = createCapacityAttribute();
		ca.addCapacity(capacity);
		
		if (nodes.size() > 1) {
			for (int i = 1; i < nodes.size(); i++) {
				Place temp = new Place(nodes.get(i));
				temp.addCapacity(capacity);
			}
		}
	}
	
	/**
	 * Removes the capacity of a place (if the place is a logical place then the capacities of the associated places are also removed).
	 */
	public void removeCapacity() {
		CapacityAttribute ca = getCapacityAttribute();
		if (ca == null)
			ca = createCapacityAttribute();
		ca.removeCapacity();
		
		if (nodes.size() > 1) {
			for (int i = 1; i < nodes.size(); i++) {
				Place temp = new Place(nodes.get(i));
				temp.removeCapacity();
			}
		}
	}
	
	/**
	 * Gets the capacity of a place.
	 *
	 * @return the capacity
	 */
	public Capacity getCapacity() {
		CapacityAttribute ca = getCapacityAttribute();
		if (ca == null)
			return new Capacity(Integer.MAX_VALUE);
		return ca.getCapacity();
	}
	
	/**
	 * Clears all tokens (discrete, continuous) of a place (if the place is a logical place then all tokens of the associated places are also cleared).
	 */
	public void clearTokens() {
		TokenAttributeDiscrete tad = getTokenAttributeDiscrete();
		if (tad == null)
			createTokenAttributeDiscrete();
		else
			tad.setDefaultValue();
		
		TokenAttributeContinuous tac = getTokenAttributeContinuous();
		if (tac == null)
			createTokenAttributeContinuous();
		else
			tac.setDefaultValue();
		
		if (nodes.size() > 1) {
			for (int i = 1; i < nodes.size(); i++) {
				Place temp = new Place(nodes.get(i));
				temp.clearTokens();
			}
		}
	}
	
}
