/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.petrinetelements.cpn;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import vanted.petrinetelements.AbstractToken;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Place;

// TODO: Auto-generated Javadoc
/**
 * The Class Marking contains the mapping between tokens and places.
 *
 * @author Kevin Pucknat
 */
public class Marking {
	
	/** The place token. */
	private HashMap<Place, List<AbstractToken>> placeToken = new HashMap<Place, List<AbstractToken>>();
	
	/** The pn. */
	private PetriNet pn;
	
	/**
	 * Instantiates a new marking.
	 *
	 * @param pn the pn
	 */
	public Marking(PetriNet pn) {
		this.pn = pn;
	}
	
	/**
	 * Adds the place token.
	 *
	 * @param place the place
	 */
	public void addPlaceToken(Place place){
		placeToken.put(place, place.getTokens());
	}
	
	/**
	 * Gets the place token.
	 *
	 * @return the place token
	 */
	public HashMap<Place, List<AbstractToken>> getPlaceToken(){
		return placeToken;
	}
	
	/**
	 * Gets the all tokens.
	 *
	 * @return the all tokens
	 */
	public List<AbstractToken> getAllTokens(){
		List<AbstractToken> tokens = new ArrayList<AbstractToken>();	

		for(List<AbstractToken> tokenlist : placeToken.values()){
			for(AbstractToken token : tokenlist){
				tokens.add(token);
			}
		}
		
		return tokens;
	}
	
	/**
	 * Gets the petri net.
	 *
	 * @return the petri net
	 */
	public PetriNet getPetriNet(){
		return pn;
	}
}
