/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.petrinetelements.misc;

import org.AttributeHelper;
import org.graffiti.graph.Edge;

import vanted.petrinetelements.InArc;
import vanted.petrinetelements.OutArc;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Transition;

// TODO: Auto-generated Javadoc
/**
 * The Class Check proofs if the Petri net structure.
 */
public class Check {
	
	/**
	 * Checks if the Petri net is pure.
	 *
	 * @param pn the pn
	 * @return true, if is pure pn
	 */
	public static boolean isPurePN(PetriNet pn) {
		for (Transition transition : pn.getAllTransitions())
			if (!transition.getIgnoreNode().getValue())
				for (InArc inArc : transition.getInArcs())
					for (OutArc outArc : transition.getOutArcs())
						if (inArc.getPlace().equals(outArc.getPlace()) && !inArc.getPlace().getIgnoreNode().getValue())
							return false;
		
		// check for visual reversible edges
		for (Transition transition : pn.getAllTransitions()) {
			for (InArc inArc : transition.getInArcs())
				if (Check.isReversibleEdge(inArc.getEdge()))
					return false;
			
			for (OutArc outArc : transition.getOutArcs())
				if (Check.isReversibleEdge(outArc.getEdge()))
					return false;
		}
		
		return true;
	}
	
	/**
	 * Checks if the edge is a visual inverted edge.
	 *
	 * @param edge the edge
	 * @return true, if is reversible edge
	 */
	public static boolean isReversibleEdge(Edge edge) {
		return !AttributeHelper.getArrowhead(edge).isEmpty() && !AttributeHelper.getArrowtail(edge).isEmpty();
	}
}
