/*************************************************************************************
* The Petri-Net Add-on is (c) 2012 Plant Bioinformatics Group, IPK Gatersleben,
* http://bioinformatics.ipk-gatersleben.de
* The source code for this project, is available under the GPL license v2.0 
* available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html. By using this
* Add-on and VANTED you need to accept the terms and conditions of this
* license. For further details see GPL_license.txt in the root folder of this project.
************************************************************************************/

package vanted.petrinetelements.misc;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.AttributeHelper;
import org.graffiti.editor.MainFrame;
import org.graffiti.editor.MessageType;
import org.graffiti.graph.Graph;
import org.graffiti.graph.Node;

import vanted.petrinetelements.InArc;
import vanted.petrinetelements.NodePN;
import vanted.petrinetelements.OutArc;
import vanted.petrinetelements.PetriNet;
import vanted.petrinetelements.Place;
import vanted.petrinetelements.Transition;

// TODO: Auto-generated Javadoc
/**
 * The Class HelperPNMergeNodes is a helper class to merge places with the same label.
 */
public class HelperPNMergeNodes {
	
	/**
	 * Merge nodes with same label.
	 *
	 * @param nodes the nodes
	 * @param pn the pn
	 */
	//based on class GraphHelperBio VANTED version 1.18
	@SuppressWarnings("unchecked")
	public static void mergeNodesWithSameLabel(
						List<Node> nodes,						
						PetriNet pn) {

		HashMap<String, ArrayList<Node>> label2nodeList = new HashMap<String, ArrayList<Node>>();
		deleteNodes = new ArrayList<NodePN>();
		Graph g2 = null;
		int firstn = nodes.size();
		//create list with same node labels
		for (Node n : nodes) {
			if (g2 == null)
				g2 = n.getGraph();
			String name = AttributeHelper.getLabel(n, null);
			if (name == null || name.length() <= 0)
				continue;

			if (!label2nodeList.containsKey(name))
				label2nodeList.put(name, new ArrayList<Node>());
			ArrayList<Node> nl = label2nodeList.get(name);
			nl.add(n);
		}

		try {
			int workSize = label2nodeList.values().size();
			int workI = 0;
			int lastn = nodes.size();
			for (ArrayList<Node> toBeCondensed : label2nodeList.values()) {
				MainFrame.showMessage("Condense Nodes (" + (int) (workI * 100d / workSize) + "%)", MessageType.PERMANENT_INFO);
				if (toBeCondensed.size() > 1) {
					List<NodePN> nodesPN = new ArrayList<NodePN>();
						for(NodePN nodePN : pn.getAllPlaces()){
							if(toBeCondensed.contains(nodePN.getNode()))
								nodesPN.add(nodePN);
						}
						for(NodePN nodePN : pn.getAllTransitions()){
							if(toBeCondensed.contains(nodePN.getNode()))
								nodesPN.add(nodePN);
						}
												
						mergeNode(nodesPN);
						lastn = lastn - toBeCondensed.size() + 1;
				}
				workI++;
			}
				MainFrame.showMessage("Condensed Nodes: " + firstn + " --> " + lastn, MessageType.INFO);
		} finally {
			
			for(NodePN node : deleteNodes){					
				pn.removeNode(node);					
			}
		}

	}
	
	/** Nodes to be deleted. */
	private static List<NodePN> deleteNodes;
	
	/**
	 * Merge places to one logical place.
	 *
	 * @param toBeMerged the to be merged
	 * @return the node
	 */
	// based on class MergeNodes VANTED version 1.4
	public static NodePN mergeNode(List<NodePN> toBeMerged) {
		NodePN mergedNode = toBeMerged.iterator().next();
		
//		HashSet<String> clusterIDs = new HashSet<String>();
		for (NodePN checkNode : toBeMerged) {
			if(checkNode instanceof Place){
				Place place = (Place)checkNode;

				for (OutArc inEdge : place.getInArcs()) {					
					if (!inN((Place)mergedNode).contains(inEdge.getTransition())) {
						inEdge.setPlace((Place)mergedNode);
						((Place)mergedNode).addInArc(inEdge);						
					}
				}
				for (InArc outEdge : place.getOutArcs()) {
					if (!outN((Place)mergedNode).contains(outEdge.getTransition())) {
						outEdge.setPlace((Place)mergedNode);
						((Place)mergedNode).addOutArc(outEdge);
					}
				}
			}
			else if(checkNode instanceof Transition){
				Transition transition = (Transition)checkNode;
				for (InArc inEdge : transition.getInArcs()) {
					if (!inN(transition).contains(inEdge.getPlace())) {
						inEdge.setTransition((Transition)mergedNode);
						((Transition)mergedNode).addInArc(inEdge);
					}
				}
				for (OutArc outEdge : transition.getOutArcs()) {
					if (!outN(transition).contains(outEdge.getPlace())) {
						outEdge.setTransition((Transition)mergedNode);
						((Transition)mergedNode).addOutArc(outEdge);
					}
				}
			}
			
			if(!mergedNode.equals(checkNode)){
				deleteNodes.add(checkNode);		
				if(!mergedNode.getNodes().contains(checkNode.getNode()))
					mergedNode.addNode(checkNode.getNode());
			}
		}

		return mergedNode;
	}
	
	/**
	 * Get all ingoing transitions of a place.
	 *
	 * @param place the place
	 * @return the list
	 */
	private static List<Transition> inN(Place place){
		List<Transition> transitions = new ArrayList<Transition>();
		for(OutArc arc : place.getInArcs()){
			transitions.add(arc.getTransition());
		}		
		return transitions;
	}
	
	/**
	 * Get all outgoing transitions of a place.
	 *
	 * @param place the place
	 * @return the list
	 */
	private static List<Transition> outN(Place place){
		List<Transition> transitions = new ArrayList<Transition>();
		for(InArc arc : place.getOutArcs()){
			transitions.add(arc.getTransition());
		}
		return transitions;
	}
	
	/**
	 * Get all ingoing places of a transition.
	 *
	 * @param transition the transition
	 * @return the list
	 */
	private static List<Place> inN(Transition transition){
		List<Place> places = new ArrayList<Place>();
		for(InArc arc : transition.getInArcs()){
			places.add(arc.getPlace());
		}		
		return places;
	}
	
	/**
	 * Get all outgoing places of a transition.
	 *
	 * @param transition the transition
	 * @return the list
	 */
	private static List<Place> outN(Transition transition){
		List<Place> places = new ArrayList<Place>();
		for(OutArc arc : transition.getOutArcs()){
			places.add(arc.getPlace());
		}		
		return places;
	}
}
