/*************************************************************************************
* This file is based on the Pipe2 project (http://pipe2.sourceforge.net/).
* The source code for this project is available under the OSL license 3.0 available at
* http://www.opensource.org/licenses/osl-3.0.php. By using this file you need to accept
* the terms and conditions of this license. For further details see OSL_license.txt 
* in the root folder of this project.
************************************************************************************/

package test;

import pipe.analysis.Matrix;


public class PNHelper {
	/**
	 * Gets the enabled transitions from marking.
	 *
	 * @param marking the marking
	 * @param rule the rule
	 * @return the enabled transitions from marking
	 */
	public static boolean[] getEnabledTransitionsFromMarking(double[] marking, Matrix incidenceMatrix, double[][] fMatrix, double[][] bMatrix, double[] capacityVector){ 
		return getEnabledTransitionsFromMarkingStrong2(marking, incidenceMatrix, fMatrix, bMatrix, capacityVector);			
	}

	
	//with side conditions
	/**
	 * Gets the enabled transitions from marking with the strong transition rule with side conditions.
	 *
	 * @param marking the marking
	 * @return the enabled transitions from marking strong2
	 */
	private static boolean[] getEnabledTransitionsFromMarkingStrong2(double[] marking, Matrix incidenceMatrix, double[][] fMatrix, double[][] bMatrix, double[] capacityVector){		
		boolean enabled[] = new boolean[incidenceMatrix.getColumnDimension()];
		for(int i=0; i<enabled.length;i++){
			enabled[i]=true;
		}
		
		for(int t=0; t<incidenceMatrix.getColumnDimension(); t++){ //transitions
			for(int p=0; p<incidenceMatrix.getRowDimension(); p++){ //places			
				if(marking[p]!=-1 &&	//omega
					((marking[p]<bMatrix[p][t]) ||	//pre-condition 
						(marking[p]+(fMatrix[p][t]-bMatrix[p][t]))>capacityVector[p])){	//post-condition						
					enabled[t]=false;
				}
			}
		}
		
		return enabled;			
	}
}
